% SUGAR_TOOLBOX
%
% Equations of state
%   eos_ch4              - calculates thermodynamic properties of CH4 
%   eos_co2              - calculates thermodynamic properties of CO2 
%   eos_n2               - calculates thermodynamic properties of N2 
%   eos_o2               - calculates thermodynamic properties of O2 
%   eos_h2s              - calculates density and fugacity coefficient of H2S 
%   eos_h2o              - calculates thermodynamic properties of pure water 
%   eos_sw               - calculates thermodynamic properties of seawater 
%   eos_IAPS84           - outdated formulation to calculate thermodynamic properties of seawater 
%   eos_swDriesner       - calculates thermodynamic properties of seawater for extreme conditions 
%
% Densities
%   density_ch4          - calculates the density of CH4 (calls eos_ch4)
%   density_ch4sw        - calculates the density of a CH4-seawater solution
%   density_co2          - calculates the density of CO2 (calls eos_co2)
%   density_co2sw        - calculates the density of a CO2-seawater solution
%   density_n2sw         - calculates the density of a N2-seawater solution
%   density_sw           - calculates the density of seawater (calls function with matching pTS range) 
%   density_swUnesco     - UNESCO formulation to calculate the density of seawater
%   density_swSpivey     - calculates the density of seawater for high p-T conditions
%   density_swSun        - calculates the density of seawater for high p-T conditions
%   density_ch4co2n2     - calculates the density and phase composition of CH4-CO2-N2 mixtures
%   density_cacl2brine   - calculates the density of a CaCl2-water solution
%
% Phase boundaries
%   hydrate_phasediagram - plots the phase diagrams of CH4 and CO2 hydrates in seawater
%   phase_ch4sw          - calculates the phase boundaries of the system CH4-seawater
%   phase_co2sw          - calculates the phase boundaries of the system CO2-seawater
%   phase_sw             - calculates the phase boundaries of seawater
%   sw_phasediagram      - plots the phase diagram of seawater
%   vlh_naclh2o          - calculates the 3-phase (vapor-liquid-halite) coexistence of the system NaCl-H2O
%
% Fugacities
%   fugacity_ch4gh       - calculates the fugacity of CH4 in CH4-hydrate
%   fugacity_ch4sw       - calculates the fugacity of CH4 dissolved in seawater
%   fugacity_co2gh       - calculates the fugacity of CO2 in CO2-hydrate
%   fugacity_co2sw       - calculates the fugacity of CO2 dissolved in seawater
%   fugacity_h2och       - calculates the fugacity of water in CO2 SI-hydrate
%   fugacity_h2omh       - calculates the fugacity of water in CH4 SI-hydrate 
%   fugacity_sw          - calculates the fugacity of water in solutions with dissolved salts, CH4 and CO2
%   fugacity_h2ssw       - calculates the fugacity of H2S dissolved in seawater
%   fugacity_n2sw        - calculates the fugacity of N2 dissolved in seawater
%   fugacity_o2sw        - calculates the fugacity of O2 dissolved in seawater
%
% Solubilities
%   solu_ch4             - calculates the solubility of CH4 in seawater (calls function with matching pTS range)
%   solu_ch4gas          - calculates the 2-phase (gas-water) equilibrium concentration of CH4 in seawater 
%   solu_ch4gh           - calculates the 3-phase (gas-water-hydrate) equilibrium concentration of CH4 in seawater  
%   solu_co2             - calculates the solubility of CO2 in seawater (calls function with matching pTS range)
%   solu_co2gl           - calculates the 2-phase (gas/liquid-water) equilibrium concentration of CO2 in seawater
%   solu_co2gh           - calculates the 3-phase (gas/liquid-water-hydrate) equilibrium concentration of CO2 in seawater
%   solu_h2ssw           - calculates the solubility of H2S in seawater
%   solu_n2sw            - calculates the solubility of N2 in seawater
%   solu_o2sw            - calculates the solubility of O2 in seawater
%   solu_co2ch4          - calculates the partial pressure of CO2 in methane gas 
%   solu_ch4co2          - calculates the solubility of CH4 in CO2
%   solu_n2co2           - calculates the solubility of N2 in CO2
%   solu_swco2           - calculates the solubility of water in liquid CO2
%   solu_nacl            - calculates the solubility of NaCl in water
%   solu_caso4           - calculates the solubility of CaSO4 in seawater
%   solu_opal            - calculates the solubility of opal in seawater (calls function with matching pTS range)
%   solu_opalFournier    - calculates the solubility of opal in seawater
%   solu_opalWalther     - calculates the solubility of opal in seawater
%   solu_quartz          - calculates the solubility of quartz in seawater (calls function with matching pTS range)
%   solu_quartzDamm      - calculates the solubility of quartz in seawater for hydrothermal conditions
%   solu_quartzFournier  - calculates the solubility of quartz in seawater
%   soluhenry_ch4sw      - calculates the solubility and Henry constant of CH4 in seawater
%   soluhenry_co2sw      - calculates the solubility and Henry constant of CO2 in seawater
%   soluhenry_h2ssw      - calculates the solubility and Henry constant of H2S in seawater
%   soluhenry_n2sw       - calculates the solubility and Henry constant of N2 in seawater
%   soluhenry_o2sw       - calculates the solubility and Henry constant of O2 in seawater
%
% Viscosities
%   visco_ch4            - calculates the dynamic viscosity of CH4
%   visco_co2            - calculates the dynamic viscosity of CO2
%   visco_h2o            - calculates the dynamic viscosity of pure water
%   visco_n2             - calculates the dynamic viscosity of N2
%   visco_sw             - calculates the dynamic viscosity of seawater (calls function with matching pTS range) 
%   visco_swSpivey       - calculates the dynamic viscosity of seawater
%   visco_swMao          - calculates the dynamic viscosity of seawater
%   visco_swPalliser     - calculates the dynamic viscosity of seawater
%   visco_co2sw          - calculates the dynamic viscosity of CO2-seawater solutions
%
% Thermal Properties
%   heatcap_ch4gh        - calculates the heat capacity of CH4 hydrate
%   heatcond_ch4         - calculates the thermal conductivity of CH4
%   heatcond_co2         - calculates the thermal conductivity of CO2
%   heatcond_n2          - calculates the thermal conductivity of N2
%   heatcond_sw          - calculates the thermal conductivity of seawater
%
% Diffusion Coefficients
%   diffcoeff_h2oco2     - calculates the diffusion coefficient of water in liquid CO2
%   diffcoeff_sw         - calculates the diffusion coefficients of dissolved species in seawater
%
% Acid-Base and Mineral Equilibria
%   kco2_sw              - calculates equilibrium constants for the carbonic acid system in seawater
%   kequilib_sw          - calculates stoichiometric equilibrium constants and solubility products
%   ph_analyt            - calculates pH-system in seawater analytically from given TAlk,TCO2,TH2S,TBOH4
%   ph_model             - calculates pH-system in seawater numerically from variable input combinations
%
% Unit Conversions
%   cl2salin             - converts seawater chloride concentration to salinity
%   molal2molar_sw       - converts molal concentrations to molar concentrations for seawater salts
%   molar2molal_sw       - converts molar concentrations to molal concentrations for seawater salts
%   mkgh2o2mkgsol        - converts mol/kg{H2O} to mol/kg{solution}
%   mkgsol2mkgh2o        - converts mol/kg{solution} to mol/kg{H2O}
%   mol2pp_ch4           - calculates the CH4 partial pressure from its molal concentration in seawater
%   mol2pp_co2           - calculates the CO2 partial pressure from its molal concentration in seawater
%   mol2pp_n2            - calculates the N2 partial pressure from its molal concentration in seawater
%   mol2pp_o2            - calculates the O2 partial pressure from its molal concentration in seawater
%   mol2pp_h2s           - calculates the H2S partial pressure from its molal concentration in seawater
%   pp2pv_ch4            - converts CH4 partial pressure to partial volume
%   pp2pv_co2            - converts CO2 partial pressure to partial volume
%   pp2pv_n2             - converts N2 partial pressure to partial volume
%   pp2pv_o2             - converts O2 partial pressure to partial volume
%   pp2pv_h2s            - converts H2S partial pressure to partial volume
%   
% Auxiliary Functions
%   i_sw                 - calculates the ionic strength of seawater
%   permeab              - estimates the permeability of the sediment from porosity and grain size
%   seasalt              - calculates the molal concentrations of major salts in seawater
%   seasalt_molarity     - calculates the molar concentrations of major salts in seawater
%   tortuos              - calculates the tortuosity correction for diffusion coefficients
