function rho = density_cacl2brine(press,temp,mCaCl2)

% DENSITY_CACL2BRINE -  calculates the density of aqueous CaCl2 solutions
%
%       Usage: rho = density_cacl2brine(p,T,m)
%
%       Input parameters (scalars or vectors): 
%           p        - pressure [MPa]              (0.1 < p < 68.5) 
%           T        - temperature [degC]          (10 < T < 199)
%           m        - molality of CaCl2 [mol/kg]  (0 < m < 6)
%   
%       Output parameter:
%          rho      - density of CaCl2 solution [kg/m^3]
%
%       References:
%       Al-Ghafri et al. (2012) J. Chem. Eng. Data 57, 1288-1304.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('DENSITY_CACL2BRINE:Input','DENSITY_CACL2BRINE: Missing input parameter! Please provide p,T and S!')
else


%------------------------------------------------------------------------
% Check data range validity!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>68.5) && ~any(temp<(10) | temp>199) && ~any(mCaCl2<(0) | mCaCl2>6) 
        
else
    bad = find(press<0.1 | press > 68.5);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(10) | temp>199);
    temp(bad)=NaN*ones(size(bad));
    bad = find(mCaCl2<(0) | mCaCl2>6);
    mCaCl2(bad)=NaN*ones(size(bad));

 warning('DENSITY_CACL2BRINE:OutOfParameterRange','DENSITY_CACL2BRINE: Input values out of range!')
 warning('DENSITY_CACL2BRINE:OutOfParameterRange','Equations valid for p = 0.1-68.5 MPa, T=10-199 degC, m = 0-6 mol/kg.')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------


[P, T, m] = ndgrid(press,temp + 273.15, mCaCl2);

Tc = 647.1; % critical temperature of water [K]
pc = 22.064; % critical pressure of water [MPa]
rhoc = 322; % critical density of water [kg/m^3]
phi = (1 - T./Tc);

%create empty matrices for a, b and B 
a = zeros(3,3);
b = zeros(2,4);
B = zeros(size(T));

a(1,1) = 2546.760;
a(1,2) = -39884.946;
a(1,3) = 102056.957;
a(1,4) = -98403.334;
a(1,5) = 33976.048;
a(2,1) = -1362.157;
a(2,2) = 22785.572;
a(2,3) = -59216.108;
a(2,4) = 57894.824;
a(2,5) = -20222.898;
a(3,1) = 217.778;
a(3,2) = -3770.645;
a(3,3) = 9908.135;
a(3,4) = -9793.484;
a(3,5) = 3455.587;
b(2,1) = 307.24;
b(2,2) = -1259.10;
b(2,3) = 2034.03;
b(2,4) = -1084.94;
g1 = -0.00493;
g2 = 0.00231;

c1 = 1.992741;
c2 = 1.099653;
c3 = -0.510839;
c4 = -1.754935;
c5 = -45.517035;
c6 = 674694.450000;
b(1,1) = -1622.40;
b(1,2) = 9383.80;
b(1,3) = -14893.80;
b(1,4) = 7309.10;
g0 = 0.11725;
s1 = -7.859518;
s2 = 1.844083; 
s3 = -11.786650;
s4 = 22.680741; 
s5 = -15.961872; 
s6 = 1.801225;

%------------------------------------------------------------------------
% Calculation of reference pressure pr (water vapor pressure)
%------------------------------------------------------------------------
lnpr = (Tc./T).*(s1.*phi + s2.*phi.^(3/2) + s3.*phi.^(3) + ...
    s4.*phi.^(7/2) + s5.*phi.^(4) + s6*phi.^(15/2));
pr = exp(lnpr).*pc;

%------------------------------------------------------------------------
% Calculation  rho_0 (density of pure water at vapor pressure)
%------------------------------------------------------------------------


rho0 = rhoc.*(1 + c1.*phi.^(1/3) + c2.*phi.^(2/3) + c3.*phi.^(5/3) + ...
    c4.*phi.^(16/3) + c5.*phi.^(43/3) + c6*phi.^(110/3));

%------------------------------------------------------------------------
% calculation of terms B(T,m) and C(m)
%------------------------------------------------------------------------

for i = 0:1
    for j = 0:3
        B = B + b(i+1,j+1).*m.^i .* (T./Tc).^j;
    end
end

C = g0 + g1.*m + g2.*m.^(3/2);

%------------------------------------------------------------------------
% calculation of reference density rhor
%------------------------------------------------------------------------

rhor = rho0;

for i = 1:3
    rhor = rhor + a(i,1).*m.^((i+1)/2);
end

%error in paper: j runs from 1:4, not 1:3
for i = 1:3
    for j = 1:4
        rhor = rhor + a(i,j+1).*m.^((i+1)/2).*(T./Tc).^((j+1)/2);
    end
end

%------------------------------------------------------------------------
% calculation of density rho
%------------------------------------------------------------------------
rho = rhor./(1 - C.*log((B + P)./(B + pr)));

end
   
