function [rho, x, y, V, exitflag] = density_ch4co2n2(p,T,molCH4,molCO2,molN2)

% DENSITY_CH4CO2N2 -  calculates the density of a mixture of CH4, CO2 and N2
%      and returns the phase composition (mixture might separate into
%      a liquid and a vapor phase) and the vapor phase fraction.
%      Calculations are based on the Peng-Robinson EOS.
%    
%      Usage: [rho x y V exitflag] = density_ch4co2n2(p,T,mn2,mco2,mch4)
% 
%      Input parameters (scalars!): 
%           p        - pressure [MPa]      (p > 1 MPa)
%           T        - temperature [degC] 
%           mn2      - total amount of N2 in mixture [mol]
%           mco2     - total amount of CO2 in mixture [mol]
%           mch4     - total amount of CH4 in mixture [mol]
%
%      Output parameters:
%           rho      - density of mixture [kg/m^3]
%                        (scalar for single phase; 2-component vector, if 
%                         mixture consists of liquid and vapor phase) 
%           x        - mole fractions of N2, CO2, CH4 in liquid phase
%                        (3-component vector)
%           y        - mole fractions of N2, CO2, CH4 in vapor phase
%                        (3-component vector)
%           V        - vapor phase fraction
%           exitflag - exitflag of solver (to check the goodness of the
%                      numerical solution): =999 if solver has not been used
%
%      References:
%      Peng & Robinson (1976) Ind. Eng. Chem. Fund. 15, 59-64.
%      Risnes et al. (1981) Developments in Petroleum Science 13, 329-350.
%      Michelsen (1982) Fluid Phase Equilibria 9, 1-19.
%
%      The calculation of the multi phase system follows the procedure
%      outlined in the course documentation of M. Adewumi "Phase relations
%      in reservoir engineering", Penn State College of Earth and Mineral 
%      Sciences, www.e-education.psu.edu/png520/m17_p4.html
%
%      Comments:
%      1. Only scalar input!
%      2. Results for pure fluids deviate from results of the Span & Wagner
%         EOS by about 10% at 10 MPa. The deviation increases with 
%         increasing pressure.
%      3. V is set to NaN, if only 1 phase exists, but it is unknown if
%         the phase is liquid or vapor 
%      4. This script can be used to calculate solubilities of CH4 and N2
%         in liquid CO2: Vary the amount of CH4 or N2 until the 1-phase
%         solution becomes a 2-phase solution: The highest amount with
%         a 1-phase solution is the maximum amount that can be dissolved
%         in the given amount of CO2 (see code example following this 
%         header, calculation is time consuming). Obtained values for
%         solubility of CH4 in CO2 are smaller than those calculated from
%         the toolbox script solu_ch4co2.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (October 2012)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Code example to calculate solubilities with the output of density_co2ch4n2
%------------------------------------------------------------------------
% 
% molCH4 = 0;
% f = [1 1];
% n = 0;
% 
% while f(2) == f(1) && n < 1002
%     molCO2 = 1 - (n - 1)/1000;
%     molN2 = (n-1)/1000;
%      rho = density_ch4co2n2(press,temp,molCH4,molCO2,molN2);
%      f(1) = f(2);
%      f(2) = length(rho);
%      n = n+1;
% end
% 
% if n == 1002
%     mN2 = Inf;
% else 
%     molN2 = (n-2)/1000;
%     molCO2 = 1 - (n - 1)/1000;
%     mN2 = molN2/molCO2/44.01d-3;
% end



%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 5
     warning('DENSITY_CH4CO2N2:Input','DENSITY_CH4CO2N2: Missing input parameter! Please provide p,T, moles_N2, moles_CO2 and moles_CH4!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if p<1
    warning('DENSITY_CH4CO2N2:OutOfParameterRange','DENSITY_CH4CO2N2: Input values out of range!')
    warning('DENSITY_CH4CO2N2:OutOfParameterRange','Equations valid for p>1.')
end
    
    
%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------
P = p*1d6; % p in Pa
t = T + 273.15; % T in Kelvin
moles = [molN2 molCO2 molCH4]; % vector with substance amounts

R = 8.314472; % gas constant [J/(mol K)]
exitflag = 999; % if exitflag is still 999 at output, no numerical solver 
                %was involved in calculating the mixture properties

z(1) = molN2/(molN2 + molCO2 + molCH4); % mole fraction of N2 in feed
z(2) = molCO2/(molN2 + molCO2 + molCH4); % mole fraction of CO2 in feed
z(3) = molCH4/(molN2 + molCO2 + molCH4); % mole fraction of NCH4in feed

% critical constants (Pc, Tc and Vc), acentric factor(w) and molar mass
% (mm) (from NIST database)

%%%%%% N2 %%%%%%%%
     Pc(1) = 3.3978d6;    % Pa
	 Tc(1) = 126.19;   % Kelvin
	% Vc(1) = 89.4142;  % cm^3/mol
	 w(1) = 0.039;
     mm(1) = 28.01348;   % g/mol
     
%%%%%% CO2 %%%%%%%%
     Pc(2) = 7.3825d6;   
	 Tc(2) = 304.2;
	% Vc(2) = 94.11853;
	 w(2) = 0.239;	% 0.223621 
     mm(2) = 44.01;

%%%%%% CH4 %%%%%%%%
     Pc(3) = 4.61d6;
	 Tc(3) = 190.6;
	% Vc(3) = 98.62906;
	 w(3) = 0.011; % 0.008
     mm(3) = 16.04;
     
% calculate vapor pressure pv of CO2
[~,~,pv] = phase_co2sw(T,0);   

%----------------------------------------------------------------------
% if only gas phase exists (all components are gaseous or supercritical)
%----------------------------------------------------------------------
% N2 and CH4 are supercritical in the full pT range of interest. If CO2 is
% gaseous or supercritical or non-existent, or if the feed has only one 
% component, only 1 phase exists and the calculation of the
% mixture density is straightforward:

if (t > Tc(2)) || (P < pv*1d6) || (molCO2 == 0) || (sum(z>0) == 1)
    % calculate Z from Peng Robinson equation of state
    [Z] = PeRo(z,P,t,Pc,Tc,w,R);
    v = max(Z).*R.*t./P; % molar volume of vapor [m^3/mol]
    MM = sum(mm.*z); % g/mol
    rho = MM./v./1000; % density [kg/m^3]
    
    % fraction of moles in gas(supercritical) phase V
    if (molN2 == 0 && molCH4 == 0) && (P > pv*1d6) % pure liquid CO2
        V = 0;  % only liquid phase present
        y = [0 0 0];
        x = moles;
    else
        V = 1; % only gas (supercritical) phase present
        x = [0 0 0];
        y = moles;
    end
 
%----------------------------------------------------------------------
% in the pT range of liquid CO2, 2 phases might exist
%----------------------------------------------------------------------    
else % two phases (liquid and gas) might exist
    
    %----------------------------------------------------------------------
    % apply stability criteria test to find the number of phases in the system 
    %----------------------------------------------------------------------
    
    % calculate fugacity coefficient for single liquid or gas phase
    [~, phivz, philz] = PeRo(z,P,t,Pc,Tc,w,R);
    
    % first guess for K from Wilson's law (with PR acentric factors -> close
    % enough to Pitzer acentric factors)
    K0 = Pc./P.*exp(5.37.*(1 + w).*(1 - Tc./t));
    K = K0;
    
    %----------------------------------------------------------------------
    % create vapor like second test phase
    %----------------------------------------------------------------------
    
    % initialize flags
    crit = 0;
    cvap = 0;
    tvap = 0;
    
    while crit == 0;
        Y = z.*K; % vapor like second phase
        Sv = sum(Y); % check sum (sum of mole numbers)
        Y = Y./Sv; % normalize Y
    
        [~, phiv] = PeRo(Y,P,t,Pc,Tc,w,R); % calculate vapor phase fugacity coefficient
    
        Rf = philz.*z./phiv./Y./Sv; % correction factor
        K = K.*Rf; %  iteration step for K
        K(isnan(Rf))=0; % take care of NaN (necessary, if any component of z is 0)
        Rf(isnan(Rf))=1; % take care of NaN (necessary, if any component of z is 0)

        % convergence criterium
        if sum((Rf-1).^2) < 1d-10
            crit = 1;
        end
    
        % trivial solution criterium
        if sum((log(K)).^2) < 1d-4
            crit = 1;
            tvap = 1; % flag: trivial solution obtained
        end
    
    end
 
    % check stability criterium after convergence
    if Sv <=1
        cvap = 1; % flag: stability criterium
    end

    %----------------------------------------------------------------------
    % create liquid like second test phase
    %----------------------------------------------------------------------
    
    K = K0; % 
    % initialize flags
    crit = 0;
    cliq = 0;
    tliq = 0;
    
    while crit == 0;
        Y = z./K; % liquid like second phase
        Y(isnan(Y))=0; % take care of NaN (necessary, any component of z is 0)
        Sv = sum(Y); % check sum (sum of mole numbers)
        Y = Y./Sv; % normalize Y
    
        [~ , ~, phil] = PeRo(Y,P,t,Pc,Tc,w,R); % calculate liquid phase fugacity coefficient
    
        Rf = phil.*Y./phivz./z.*Sv; % correction factor
        K = K.*Rf; % iteration step for K
        K(isnan(Rf))=0; % take care of NaN (necessary, if any component of z is 0)
        Rf(isnan(Rf))=1; % take care of NaN (necessary, if any component of z is 0)
    
        % convergence criteria
        if sum((Rf-1).^2)< 1d-10
             crit = 1;
        end
    
         % trivial solution criteria
        if sum((log(K)).^2) < 1d-4
            crit = 1;
            tliq = 1; % flag: trivial solution obtained
        end
    
    end

        % check stability criterium after convergence
    if Sv <=1
        cliq = 1; % flag: stability criterium 
    end

    %----------------------------------------------------------------------
    % only 1 phase 
    %----------------------------------------------------------------------
    if (cliq == 1 && cvap == 1) || (tliq == 1 && tvap == 1) || ...
            (cliq == 1 && tvap == 1) || (tliq == 1 && cvap == 1) 
        
        Z = PeRo(z,P,t,Pc,Tc,w,R);
        v = max(Z).*R.*t./P; % molar volume of vapor [m^3/mol]
        MM = sum(mm.*z); % g/mol
        rho = MM./v./1000; % density [kg/m^3]
        x = [0 0 0];
        y = moles;
        V = NaN; % only 1 phase: gas or liquid???
       
    %----------------------------------------------------------------------
    % 2 phases 
    %----------------------------------------------------------------------
    else % calculate iterative solution for 2-phase problem
             z2 = z;
         if sum(z>0) == 2 % if only 2 components are present
            z(z==0)=1d-40; % aviod division by 0
         end
   
        
    %----------------------------------------------------------------------
    % create starting values for solver 
    %----------------------------------------------------------------------
   
   
    % starting value for V
    % polynomial coefficients for cubic equation in V0 (derived from Rachford-Rice
    % equation
    pol(1) = (K0(1)-1).*(K0(2)-1).*(K0(3)-1);
    pol(2) = (K0(2)-1).*(K0(3)-1).*(1-z(1)) + (K0(2)-1).*(K0(1)-1).*(1-z(3)) ...
        + (K0(1)-1).*(K0(3)-1).*(1-z(2)) ;
    pol(3) = (K0(1)-1).*z(1) + (K0(2)-1).*z(2) + (K0(3)-1).*z(3);
    pol(4) = 0;
    
    V = roots(pol);
    V = V(imag(V) == 0); % ignore complex roots
    V=V(V>=0); % ignore negative roots
    if sum(V>0) > 0
        V0 = min(V(V>0)); % chose value > 0 if possible
    else
        V0=0;
    end
    % calculate starting values for x
    x0 = z./(1 + V0.*(K0-1));
    x=x0;
    
    % calculate starting values for y
    y0 = K0.*x;
    y = y0;
    
    options=optimset('Display','off','TolFun',1e-32); % set options for fitfunction  
        
       
    %----------------------------------------------------------------------
    % solve equilibrium condition, Rachford-Rice equation and material
    % balances for x, y and V with iteratve procedure
    %----------------------------------------------------------------------
    
    V = V0;
    ii=0;
    
    while ii < 2000
        
        [~ , ~, phil] = PeRo(x,P,t,Pc,Tc,w,R);
        [~, phiv] = PeRo(y,P,t,Pc,Tc,w,R);
      
        fl = phil.*x;
        fv = phiv.*y;
        K = fl./fv.*y./x;
        [V, ~, exitflag]= fsolve(@(V) RaRi(V,K,z),V,options);
        if V < 0
            V = 0;
        end
        if V > 1
            V = 1;
        end
        x = z./(1 + V.*(K-1));
         y = K.*x;
         ii = ii+1;
         te =sum((phil.*x./phiv./y - 1).^2); % termination condition
         if te < 1e-100
             ii = 3000;
         end
    end

    % the iterative procedure returns better values than the MATLAB
    % minimization function. The minimization procedure is coded and can be
    % activated if the iteration does not work properly
    
%     %----------------------------------------------------------------------
%     % solve equilibrium condition, Rachford-Rice equation and material
%     % balances for x, y and V by using a MATLAB minimization function
%     %----------------------------------------------------------------------
%      
%     X0=[x0 y0 V0];
% %    lb = [0 0 0 0 0 0]; % lower boundaries (only for lsqnonlin)
% %    ub = [1 1 1 1 1 1]; % upper boundaries (only for lsqnonlin)
%     
%     [X, fval, exitflag]= fsolve(@(X) minfunc(X,P,t,Pc,Tc,w,R,z),X0,options);
%    
%     % If the solver returns unrealistic values for x,y and V,use lsqnonlin 
%     % instead of fsolve: allows to set constraints for x, y, V
%     % (generally, fsolve is the better solver).
%     % [X, resnor, residual, exitflag]= lsqnonlin(@(X) minfunc(X,P,t,Pc,Tc,w,R,z),X,lb,ub,options);
% 
%        x(1:3) = X(1:3);
%        y(1:3) = X(4:6);
%        V = X(7);

    if sum(z2>0) == 2 % if only 2 components are present
       z = z2;       % undo setting all components of z to finite numbers
       x(z==0)=0;
       y(z==0)=0;
    end

    Zl = PeRo(x,P,t,Pc,Tc,w,R);
    Zv = PeRo(y,P,t,Pc,Tc,w,R);
       

        
        
    %----------------------------------------------------------------------
    % calculate density of liquid and vapor phase
    %----------------------------------------------------------------------    
    
    v = zeros(2,1);
    v(1) = max(Zv).*R.*t./P; % molar volume of vapor [m^3/mol]
    v(2) = min(Zl).*R.*t./P; % molar volume of liquid [m^3/mol]
    
    % molar mass of mixture (MM)

    MM(1) = sum(mm.*y); % g/mol
    MM(2) = sum(mm.*x); % g/mol

    rho = MM./v'./1000; % density [kg/m^3]
    x = x.*(1-V).*sum(moles); % convert mole fractions to substance amounts in moles
    y = y.*(V).*sum(moles);
    end
    
end
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%----------------------------------------------------------------------
% Peng Robinson equation of state
%----------------------------------------------------------------------
function [Z ,phiv ,phil] = PeRo(x,p,T,Pc,Tc,w,R)

kappa = 0.37464 + 1.54226.*w - 0.26992.*w.^2;

delta = zeros(3,1); % binary interaction parameters

delta(1) = -0.013; %0.017; % -0.013 (N2 - CO2)
delta(2) = 0.034; %0.026; % 0.034  (N2 - CH4)
delta(3) = 0.095;%0.084; % 0.095  (CO2 - CH4)

% calculation of EOS parameter a
ai = 0.45724.*R^2.*Tc.^2./Pc.*(1 + kappa.*(1 - (T./Tc).^(0.5))).^2;

% mixing rule (parameters a and b for multicomponent systems)

aiaj = zeros(3,3); % sqrt(ai*aj)

aiaj(1,1)= ai(1);
aiaj(2,2)= ai(2);
aiaj(3,3)= ai(3);
aiaj(1,2)= (1 - delta(1)).*sqrt(ai(1).*ai(2));
aiaj(1,3)= (1 - delta(2)).*sqrt(ai(1).*ai(3));
aiaj(2,3)= (1 - delta(3)).*sqrt(ai(2).*ai(3));
aiaj(2,1)= aiaj(1,2);
aiaj(3,2)= aiaj(2,3);
aiaj(3,1)= aiaj(1,3);

a = 0;
for i = 1:3
    for j = 1:3
        a = a + x(i).*x(j).*aiaj(i,j);
    end
end

bi = 0.0778.*R.*Tc./Pc;

b = sum(bi.*x);

% calculation of EOS parameters A and B

A = a.*p./R^2./T.^2;
B = b.*p./R./T;

% solving the equation of state for Z (Z = PV/(RT))
Z = roots([1 -(1-B) (A - 3*B.^2 - 2.*B) -(A.*B - B.^2 - B.^3)]);

Z = Z(imag(Z) == 0); % ignore complex roots
Z=Z(Z>0); % ignore negative roots

% vapor phase fugacity coefficient
Zv = max(Z); % vapor = largest number in case that more than 1 solution for z exists
phiv = exp(bi./b.*(Zv - 1) - log(Zv - B) - A./(2.*sqrt(2).*B) .* ...
    (2*x*aiaj/a - bi/b) .* log((Zv + (1 + sqrt(2))*B)/(Zv - (sqrt(2) - 1)*B)));


% vapor phase fugacity coefficient
Zl = min(Z); % liquid = smallest number in case that more than 1 solution for z exists
phil = exp(bi./b.*(Zl - 1) - log(Zl - B) - A./(2.*sqrt(2).*B) .* ...
    (2*x*aiaj/a - bi/b) .* log((Zl + (1 + sqrt(2))*B)/(Zl - (sqrt(2) - 1)*B)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%----------------------------------------------------------------------
% Rachford-Rice equation
%----------------------------------------------------------------------
function f = RaRi(V,K,z)

f = sum(z.*(K-1)./(1 + V.*(K-1)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%----------------------------------------------------------------------
% minimization function, can be used instead of iteration procedure
%----------------------------------------------------------------------

% function f = minfunc(X,p,T,Pc,Tc,w,R,z)
% % minimization function for solving the FLASH problem
% 
% % X(1:3) = x(i), X(4:6) = y(i), x(7)=V;
% x=X(1:3);
% y=X(4:6);
% 
% % fugacity coefficient for liquid phase
% [Zl phiv phil] = PeRo(x,p,T,Pc,Tc,w,R);
% % fugacity coefficient for gas phase
% [Zv phiv] = PeRo(y,p,T,Pc,Tc,w,R);
% 
% f = [X(4)/X(1) - phil(1)/phiv(1);
%      X(5)/X(2) - phil(2)/phiv(2);
%      X(6)/X(3) - phil(3)/phiv(3);
%     X(1)- z(1)/(1 + X(7)*(phil(1)/phiv(1)-1));
%     X(2)- z(2)/(1 + X(7)*(phil(2)/phiv(2)-1));
%     X(3)- z(3)/(1 + X(7)*(phil(3)/phiv(3)-1));
%      z(1)*(phil(1)/phiv(1)-1)/(1 + X(7)*(phil(1)/phiv(1)-1)) + z(2)*(phil(2)/phiv(2)-1)/(1 + X(7)*(phil(2)/phiv(2)-1)) ...
%      + z(3)*(phil(3)/phiv(3)-1)/(1 + X(7)*(phil(3)/phiv(3)-1))];
% 



