function rhoco2sw = density_co2sw(press,temp,sal,mco2) 

% DENSITY_CO2SW -  calculates the density of a solution of CO2 in seawater
%
%       Usage: rho = density_co2sw(p,T,S,m)
%
%       Input parameters (p,T,S: scalars or vectors): 
%          p        - pressure [MPa]     (0.1 < p < 100), 
%          T        - temperature [degC] (0 < T < 200),
%          S        - practical salinity (0 < S < 350)
%          m        - molality of dissolved CO2 [mol/kg]
%                         (default value is saturation concentration) 
%                         (input as scalar or p x T x S matrix)
%
%       Output parameter: 
%           rho     - density of CO2-rich seawater [kg/m^3]
%
%       References:
%       Duan et al. (2008) Energy & Fuels 22, 1666-1674.
%       Hu et al. (2007) Chemical Geology 238, 249-267.
%       Duan et al. (2006) GCA 70, 3369-3386.
%
%       Comments:
%       The validity range for salinity is not specified and the NaCl limit
%       is taken instead. The formalism follows in part the Duan 2006  
%       publication on density of CH4-seawater solutions.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2009: Validity of temperature range: Check number changed
%                      to 200 degC (line 58)
%       September 2011: input format for mch4 added to header
%       November 2012: syntax changed to Matlab2012 style
%       February 2012: Function call for seasalt updated
%       April 2013: simplified and more accurate mass calculation for seawater
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('DENSITY_CO2SW:Input','DENSITY_CO2SW: Missing input parameter! Please provide p,T and S!')
else

if nargin == 3 % if no value for mco2 is provided, use saturation concentration
    
% call external function to calculate molality of dissolved CO2 at saturation  
[mco2_sw_sat] = solu_co2(press,temp,sal); % dissolved CO2 at saturation [mol/kg]
mco2_sw = mco2_sw_sat;

else
    mco2_sw = mco2;
end

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) && ~any(temp<0 | temp>200) ...
        && ~any( sal<0 | sal>350)
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>200);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>350);
    sal(bad)=NaN*ones(size(bad));
    warning('DENSITY_CO2_SW:OutOfParameterRange','DENSITY_CO2_SW: Input values out of range!')
    warning('DENSITY_CO2_SW:OutOfParameterRange','Equations valid for p=0.1-100 MPa, T=0-200 degC, S=0-350.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------
[P,T,S] = ndgrid(press,temp+273.15,sal); % pressure in MPa, temperature in K

Mco2 = 44.0098; % Molar mass of CO2[g/mol];

%------------------------------------------------------------------------
% Calculation of seasalt composition and seawater properties
%------------------------------------------------------------------------

% call external funtion to calculate density and chlorinity of seawater for
% calculation of salt component molalities
[rsw] = density_sw(press,temp,sal); % rsw [kg/m^3], chl [g/kg]

% call external function to calculate molarities and molalities of salt
% components

rho_sw = rsw./1000; % [g/cm^3]

%------------------------------------------------------------------------
% Calculation of mass of solution per 1000 g water [g/(1000g water)]
%------------------------------------------------------------------------
Sa = S*1.005; % absolute salinity
m_sol = 1000 + Mco2 .* mco2_sw + Sa.*(1000./(1000-Sa)); % [g per 1000g water content]

%------------------------------------------------------------------------
% Calculation of the solution volume V_sol per 1000 g water content 
%------------------------------------------------------------------------

% call routine 'molvol' to calculate the partial molar volume of CO2
% dissolved in seawater [cm^3/mol]:
Vco2 = molvol(P,T);

V_sol =(1000 + Sa.*(1000./(1000-Sa)))./rho_sw+ mco2_sw.* Vco2; %[cm^3 per 1000 g water]

%------------------------------------------------------------------------
% Calculation of the density of the CO2-seawater-solution
%------------------------------------------------------------------------

rhoco2sw = m_sol./V_sol; %[g/cm^3]
rhoco2sw = rhoco2sw .* 1000; % [kg/m^3]

end
% ------------------------------------------------------------

function Vco2 = molvol(P,T)
% This function calculates the apparent partial molar volume of CO2 
% dissolved in pure water from a multiparametric equation. 
% According to Duan (2008), this volume can also be used for
% calculating the density of CO2 in seawater, since the concentration of
% CO2 is sufficiently small to neglect CO2-salt interactions.

%------------------------------------------------------------------------
% calculation of the molar volume Vh2o of pure water [cm^3/mol]
%------------------------------------------------------------------------

k01 = 0.000000327225;
k02 = -0.00042095;
k03 = 0.232594;
k04 = -41.692;
k05 = 5712.92;
k11 = -0.000000000232306;
k12 = 0.000000291138;
k13 = -0.000149662;
k14 = 0.0359860;
k15 = -3.55071;
k21 = 0.0000000000000257241;
k22 = -0.0000000000124336;
k23 = 0.000000542707;
k31 = -0.00000000000000000442028;
k32 = 0.00000000000000210007;
k33 = -0.0000000000811491;

k0 = k01.*T.^3 + k02.*T.^2 + k03.*T + k04 + k05./T;
k1 = k11.*T.^3 + k12.*T.^2 + k13.*T + k14 + k15./T;
k2 = k21.*T.^3 + k22.*T.^2 + k23;
k3 = k31.*T.^3 + k32.*T.^2 + k33;

% pressure is required in bar
Vh2o = k0 + k1.*P.*10 + k2.*(P.*10).^2 + k3.*(P.*10).^3; % [cm^3/mol]

%------------------------------------------------------------------------
% calculation of the apparent molar volume Vco2 of CO2 in pure water [cm^3/mol]
%------------------------------------------------------------------------

A11 =  0.38384020e-3; 
A21 = -0.57709332e-5;
A12 = -0.55953850; 
A22 =  0.82764653e-2;
A13 =  0.30429268e3; 
A23 = -0.43813556e1;
A14 = -0.72044305e5; 
A24 =  0.10144907e4;
A15 =  0.63003388e7; 
A25 = -0.86777045e5;

A1 = A11.*T.^2 + A12.*T + A13 + A14./T + A15./T.^2;
A2 = A21.*T.^2 + A22.*T + A23 + A24./T + A25./T.^2;

Vco2 = Vh2o .* (1 + A1 + A2.*P); %[cm^3/mol]

% ------------------------------------------------------------
