function [rho_w] = eos_IAPS84(press,temp)

% EOS_IAPS84 -  calculates thermodynamic properties of pure water based on
%     an equation of state from the outdated IAPS-84 formulation
%
%     Usage: rho = eos_IAPS84(p,T)
% 
%     Input parameters (scalars or vectors): 
%         p       - pressure [MPa]     (0 < p < 500) 
%         T       - temperature [degC] (0 < T < 1000)
%
%     Output parameters:
%         rho     - density of pure water [kg/m^3]
%
%     All thermodynamic properties are calculated from the Gibbs energy
%     (g = u + pV - Ts) and its derivatives with respect to p and T.
%
%     References: 
%     IAPS Formulation 1984 for the Thermodynamic Properties of Ordinary 
%       Water Substance for Scientific and General Use. http://iapws.org.
%     Kestin & Sengers (1986) J.Phys.Chem.Ref.Data 15, 305-320.
%     IAPSW (2007) Revised release for the IAPWS industrial formlation 1997
%       for the thermodynamic properties of water and steam. www.iapws.org
%     Marcus (2000) Phys Chem Chem Phys 2, 1465-1472.
%
%     Comments: 
%     Not valid for ice phase! (No check for ice phase implemented)
%
%     The formalism is outdated and for an accurate calculation, the
%     MATLAB script eos_h2o should be used. IAPS-84 might be needed for 
%     the function eos_swDriesner. Driesner did use the IAPS84
%     formulation to derive his density formalism. The deviations of
%     IAPS84 from the actual formulation are small except in the vicinity
%     of the critical point of water. For density calculations with 
%     eos_swDriesner close to the critical point, eos_swDriesner
%     might be edited to use IAPS84 instead of eos_h2o.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (July 2011)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('EOS_IAPS84:Input','EOS_IAPS84: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------
if ~any(press<=0 | press>500) && ~any(temp< 0 | temp>1000)
else
    bad = find(press<=0 | press > 500);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>1000);
    temp(bad)=NaN*ones(size(bad));
    warning('EOS_IAPS84:OutOfParameterRange','EOS_IAPS84: Input values out of range!')
    warning('EOS_IAPS84:OutOfParameterRange','Equations valid for 0 < p >= 500 MPa, T=0-1000 degC.')
end


%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------
[P,T] = ndgrid(press*1e6,temp+273.15); % pressure in Pa, temperature in K


Tstar = 647.27; %reference temperature [K]
rhostar = 317.763; % reference density [kg/m^3]
Pstar = 22.115e6; % reference pressure [Pa]

Ts = T/Tstar; % reduced temperature
Ps = P/Pstar; % reduced pressure

%------------------------------------------------------------------------
% calculation of density of pure water by minimizing the thermodynamic
% equation p = rho * R * T * (1 + d(phi_r)/d(delta))
%------------------------------------------------------------------------

% create empty data array for density of pure water
rho = zeros(size(P,1),size(P,2)); 

% Check if arrays for P and T contain NaN. If yes, also set corresponding 
% density values to NaN. Calculate density for allowed P and T only.
% If the fit function returns EXITFLAG <= 0, the iteration process has
% terminated with a non-satisfying result. In this case, the initial value
% is changed and the fit procedure is repeated.

options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

% use internal function to look up first guess for density

rho_init1 = guessfunc(press,temp);

for i=1:size(P,1)
    for j=1:size(P,2)
        if isnan(P(i,:));
            rho(i,:) = NaN;
        elseif isnan(T(:,j));
            rho(:,j) = NaN;
        else  
            EXITFLAG = -1; % Initialize EXITFLAG value
            counter = 0; % set counter to 0
            rho_init = rho_init1(i,j)./rhostar;
            while (EXITFLAG < 1) 
                counter = counter + 1;
                [rho(i,j),~,EXITFLAG] = fsolve(@(rho_pwat) minfunc(rho_pwat,Ps(i,j),Ts(i,j)), rho_init,options);
                rho_init = rho_init1(i,j)/rhostar + counter*0.01*(-1)^(counter); 
                 if rho(i,j) <= 0
                    EXITFLAG = -1; % reject negative densities  
                end
                if rho_init <= 0
                    EXITFLAG = 10; % make sure, that while loop terminates  
                end
            end
        end
    end
end

 rho_w = rho.*rhostar; % transform reduced density into density

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [A0, A1, y, A20, k, l, A3, z0, z, m4, n4, alpha, beta4, rho4, T4, A4, delta, tau] = parameters(Ts,rhos)

%  definition and calculation of the parameters that are needed for the calculation of the
%  thermodynamic potentials

A0 = zeros(18,1);

A0(1) =  -0.310840393653e2;
A0(2) =  -0.857020420940e2;
A0(3) =   0.765192919131e-2;
A0(4) =  -0.620600116069;
A0(5) =  -0.106924329402e2;
A0(6) =  -0.280671377296e1;
A0(7) =   0.119843634845e3;
A0(8) =  -0.823907389256e2;
A0(9) =   0.555864146443e2;
A0(10) = -0.310698122980e2;
A0(11) =  0.136200239305e2;
A0(12) = -0.457116129409e1;
A0(13) =  0.115382128188e1;
A0(14) = -0.214242224683;
A0(15) =  0.282800597384e-1;
A0(16) = -0.250384152737e-2;
A0(17) =  0.132952679669e-3 ;
A0(18) = -0.319277411208e-5;

A1 =zeros(5,1);

A1(1) =  0.15383053e1;
A1(2) = -0.81048367;
A1(3) = -0.68305748e1;
A1(4) = 0;
A1(5) =  0.86756271;

A2 = zeros(4,1);
A20 = 0.42923415e1;

A2(1) =  0.59402227e-1;
A2(2) = -0.28128238e-1;
A2(3) =  0.56826674e-3;
A2(4) = -0.27987451e-3;

y = rhos.*(A2(1) + A2(2).*log(Ts) + A2(3)./Ts.^3 + A2(4)./Ts.^5);

k = [1 1 1 1 2 2 2 2 3 3 3 3 4 4 4 4 5 5 5 5 6 6 6 6 7 7 7 7 9 9 9 9 3 3 1 5];

l = [1 2 4 6 1 2 4 6 1 2 4 6 1 2 4 6 1 2 4 6 1 2 4 6 1 2 4 6 1 2 4 6 0 3 3 3];

A3=zeros(36,1);
z0 = 0.317763;

A3(1) =  -0.76221190138079e1;
A3(2) =   0.32661493707555e2;
A3(3) =   0.11305763156821e2;
A3(4) =  -0.10015404767712e1;
A3(5) =   0.12830064355028e3;
A3(6) =  -0.28371416789846e3;
A3(7) =   0.24256279839182e3;
A3(8) =  -0.99357645626725e2;
A3(9) =  -0.12275453013171e4;
A3(10) =   0.23077622506234e4;
A3(11) = -0.16352219929859e4;
A3(12) =  0.58436648297764e3;
A3(13) =  0.42365441415641e4;
A3(14) = -0.78027526961828e4;
A3(15) =  0.38855645739589e4;
A3(16) = -0.91225112529381e3;
A3(17) = -0.90143895703666e4;
A3(18) =  0.15196214817734e5;
A3(19) = -0.39616651358508e4;
A3(20) = -0.72027511617558e3;
A3(21) =  0.11147126705990e5;
A3(22) = -0.17412065252210e5;
A3(23) =  0.99918281207782e3;
A3(24) =  0.33504807153854e4;
A3(25) = -0.64752644922631e4;
A3(26) =  0.98323730907847e4;
A3(27) =  0.83877854108422e3;
A3(28) = -0.27919349903103e4;
A3(29) =  0.11112410081192e4;
A3(30) = -0.17287587261807e4;
A3(31) = -0.36233262795423e3;
A3(32) =  0.61139429010144e3;
A3(33) =  0.32968064728562e2;
A3(34) =  0.10411239605066e3;
A3(35) = -0.38225874712590e2;
A3(36) = -0.20307478607599e3;

z = 1-exp(-z0.*rhos);

m4 = [2 2 2 4];
n4 = [0 2 0 0];
alpha = [34 40 30 1050];
beta4 = [20000 20000 40000 25];
rho4 = [0.10038928e1 0.10038928e1 0.10038928e1 0.4878492e1];
T4 = [0.98876821 0.98876821 0.99124013 0.41713659];
A4 = [-0.32329494e-2 -0.24139355e-1 0.79027651e-3 -0.13362857e1];

delta = (rhos - rho4)./rho4;
tau = (Ts - T4)./T4;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function dAdrho = derivative_rho(Ts,rhos)
%  calculation of derivative of Helmholtz function with respect to reduced
% density dAdrho

% call function "parameters" to get parameters for the equation of state
[~, A1, y, A20, k, l, A3, z0, z, m4, n4, alpha, beta4, rho4 , ~, A4, delta, tau] = parameters(Ts,rhos);

% implemet equations 2.3.2 to 2.3.5
Asr0 = 0;

Asr1 = zeros(size(rhos));
for i = 1:5
    Asr1 = Asr1 + A1(i).*(1./Ts.^(i-2));
end

Asr2 = A20.*Ts./rhos.*(1./(1-y) - 130*y./(3.*(1-y).^2) + ...
    169*y./(3.*(1-y).^3) - 14*y);

Asr3 = zeros(size(rhos));
for i = 1:36
    Asr3 = Asr3 + z0.*(1-z).*A3(i).*(1./Ts).^(l(i)).*k(i).*z.^(k(i)-1);
end

Asr4 = zeros(size(rhos));
for i = 1:4
    Asr4 = Asr4 + A4(1).*delta(i).^(n4(i)-1)./rho4(i).*...
        (n4(i) - alpha(i).*m4(i).*delta(i).^m4(i)).* ...
        exp(- alpha(i).*delta(i).^m4(i) - beta4(i).*tau(i).^2);
end


% calculate derivative of Helmholtz function with respect to reduced
% density
dAdrho = Asr0 + Asr1 + Asr2 + Asr3 + Asr4;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = minfunc(rhos,Ps,Ts)
% minimization function to calculate density of pure water 

% calculate derivative of Helmholtz function with respect to reduced
% density
dAdrho = derivative_rho(Ts,rhos);

RHOseos = sqrt(Ps./dAdrho);

% calculate difference between initial rho and "new" calculated rho
f = rhos - RHOseos;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function rho = guessfunc(press,temp) 
%   The formulation of the International Association for the Properties of Water and Steam (IAPSW)
%       (2007) ("Revised release for the IAPWS industrial formlation 1997 for
%       the thermodynamic properties of water and steam", IAPWS-IF) is used to
%       calculate a  start value for the iteration of the density. The
%       formalism is valid for 0 < T < 800C and 0 < P < 100 MPa. It
%       includes equations for water densities in the liquid region (region
%       1) and the gas and supercritical-gaslike region (region 2) of the phase diagram. 
%       Nevertheless, there is no direct equation for the water density in
%       teh supercritical liquid-like region (region 3). Therefore, starting values
%       for region 3 are calculated from the equation of Y. Marcus (PCCP2,
%       2002).
%       Since the calculated density is only considers as a start value and does only
%       need to be approximate but not exact, the formulation might be used
%       for 0 < T < 1400C and P < 167.5 MPa or 0 < T < 227C and p < 225 MPa

[P,T] = ndgrid(press,temp+273.15); %  temperature in K
R = 0.461526; % specific gas constant [kJ/(kg K)]

% rhoc = double(322); % critical density of pure water [kg/m^3]
% Tc = double(647.096); % critical temperature of pure water [K]
% tau = Tc./T; % inverse of reduced temperature

%------------------------------------------------------------------------
% Calculation of phase boundary pressure p_b1 
%------------------------------------------------------------------------
n = zeros(8,1);

n(1) = 0.11670521452767d4;     n(6) = 0.14915108613530d2;
n(2) = -0.72421316703206d6;    n(7) = -0.48232657361591d4;
n(3) = -0.17073846940092d2;    n(8) = 0.40511340542057d6;
n(4) = 0.12020824702470d5;     n(9) = -0.23855557567849;
n(5) = -0.32325550322333d7;    n(10) = 0.65017534844798d3;

theta = T + n(9)./(T - n(10));
A = theta.^2 + n(1).*theta + n(2);
B = n(3).*theta.^2 + n(4).*theta + n(5);
C = n(6).*theta.^2 + n(7).*theta + n(8);

p_b1 = (2.*C./(-B + (B.^2 - 4.*A.*C).^(0.5))).^4;

%------------------------------------------------------------------------
% Calculation of "phase boundary pressure" p_b2 above the citical point
%------------------------------------------------------------------------

p_b2 = 0.34805185628969d3 - 0.11671859879975d1.*T + 0.10192970039326d-2.*T.^2;

%------------------------------------------------------------------------
% Calculation of "phase boundary" for the given input temperatures
%------------------------------------------------------------------------

% depending on the temperature values, decide, which equation to use 

decide = T > 623.15;

p_b = p_b2.*decide - p_b1.*(decide-1);

%------------------------------------------------------------------------
% calculation of specific volume in region 1 (liquid, T< 623.15 K)
%------------------------------------------------------------------------

ps = 16.53; % reference pressure [MPa]
Ts = 1386; % reference temerature [K]

p_i = P./ps; % dimensionless pressure
tau = Ts./T; % dimensionless temperature

% first exponent 
I = [0 0 0 0 0 0 0 0 1 1 1 1 1 1 2 2 2 2 2 3 3 3 4 4 4 5 8 8 21 23 29 30 31 32];

% second exponent 
J = [-2 -1 0 1 2 3 4 5 -9 -7 -1 0 1 3 -3 0 1 3 17 -4 0 6 -5 -2 10 -8 -11 ...
    -6 -29 -31 -38 -39 -40 -41];

n = zeros(34,1);

n(1)  =  0.14632971213167;
n(18) = -0.44141845330846d-5;
n(2) = -0.84548187169114 ;
n(19)= -0.72694996297594d-15;
n(3) = -0.3756303672040d1 ;
n(20)= -0.31679644845054d-4;
n(4) = 0.33855169168385d1 ;
n(21)= -0.28270797985312d-5;
n(5) = -0.95791963387872;
n(22)= -0.85205128120103d-9;
n(6) = 0.15772038513228;
n(23)= -0.22425281908000d-5;
n(7) = -0.16616417199501d-1 ;
n(24)= -0.65171222895601d-6;
n(8) = 0.81214629983568d-3;
n(25)= -0.14341729937924d-12;
n(9) = 0.28319080123804d-3;
n(26)= -0.40516996860117d-6;
n(10)= -0.60706301565874d-3 ;
n(27)= -0.12734301741641d-8;
n(11)= -0.18990068218419d-1 ;
n(28)= -0.17424871230634d-9;
n(12)= -0.32529748770505d-1 ;
n(29)= -0.68762131295531d-18;
n(13)= -0.21841717175414d-1 ;
n(30)= 0.14478307828521d-19;
n(14)= -0.52838357969930d-4 ;
n(31)= 0.26335781662795d-22;
n(15)= -0.47184321073267d-3 ;
n(32)= -0.11947622640071d-22;
n(16)= -0.30001780793026d-3 ;
n(33)= 0.18228094581404d-23;
n(17)= 0.47661393906987d-4 ;
n(34)= -0.93537087292458d-25;

% derivative of the dimensionless Gibbs energy with respect to pressure

g_p = zeros(size(P)); % empty matrix

for i=1:34
    g_p = g_p - n(i).*I(i).*(7.1-p_i).^(I(i)-1).*(tau-1.222).^J(i);
end

% specific volume in m^3/kg
v1 = R.*1000.*T./(P.*1d6).*p_i.*g_p;


%------------------------------------------------------------------------
% calculation of specific volume in region 2 (gas, T< 1073.15K)
%------------------------------------------------------------------------

ps = 1; % reference pressure [MPa]
Ts = 540; % reference temerature [K]

p_i = P./ps; % dimensionless pressure
tau = Ts./T; % dimensionless temperature

g_p_0 = 1./p_i; % ideal gas part of dimensionless Gibbs free energy

I = zeros(43,1);
J = zeros(43,1);
n = zeros(43,1);

I(1) = 1;    J(1) = 0;  n(1) = -0.17731742473213d-2;
I(2) = 1;    J(2) = 1;  n(2) = -0.17834862292358d-1;
I(3) = 1;    J(3) = 2;  n(3) = -0.45996013696365d-1;
I(4) = 1;    J(4) = 3;  n(4) = -0.57581259083432d-1;
I(5) = 1;    J(5) = 6;  n(5) = -0.50325278727930d-1;
I(6) = 2;    J(6) = 1;  n(6) = -0.3303264167020d-4;
I(7)= 2;     J(7) =2;   n(7) =-0.18948987516315d-3;
I(8)= 2;     J(8) =4;   n(8) =-0.39392777243355d-2;
I(9)= 2;     J(9) =7;   n(9) =-0.43797295650573d-1;
I(10)= 2;    J(10) =36; n(10) =-0.26674547914087d-4;
I(11)= 3;    J(11) =0;  n(11) =0.20481737692309d-7;
I(12)= 3;    J(12) =1;  n(12) =0.43870667284435d-6;
I(13)= 3;    J(13) = 3; n(13) =-0.32277677238570d-4;
I(14)= 3;    J(14) =6;  n(14) =-0.15033924542148d-2;
I(15)= 3;    J(15) =35; n(15) =-0.40668253562649d-1;
I(16)= 4;    J(16) =1;   n(16) =-0.78847309559367d-9;
I(17)= 4;    J(17) =2;   n(17) =0.12790717852285d-7;
I(18)= 4;    J(18) =3;   n(18) =0.48225372718507d-6;
I(19)= 5;    J(19) =7;   n(19) =0.22922076337661d-5;
I(20)= 6;    J(20) =3;   n(20) =-0.16714766451061d-10;
I(21)= 6;    J(21) =16;  n(21) =-0.21171472321355d-2;
I(22)= 6;    J(22) =35;  n(22) =-0.23895741934104d2;
I(23)= 7;    J(23) =0;   n(23) =-0.59059564324270d-17;
I(24)= 7;    J(24) =11;  n(24) =-0.12621808899101d-5;
I(25)= 7;    J(25) =25;  n(25) =-0.38946842435739d-1;
I(26)= 8;    J(26) =8;   n(62) =0.11256211360459d-10;
I(27)= 8;    J(27) =36;  n(27) =-0.82311340897998d1;
I(28)= 9;    J(28) =13;  n(28) =0.19809712802088d-7;
I(29)= 10;   J(29) =4;   n(29) =0.10406965210174d-18;
I(30)= 10;   J(30) =10;  n(30) =-0.10234747095929d-12;
I(31)= 10;   J(31) =14;  n(31) =-0.10018179379511d-8;
I(32)= 16;   J(32) =29;  n(32) =-0.80882908646985d-10;
I(33)= 16;   J(33) =50;  n(33) =0.10693031879409;
I(34)= 18;   J(34) =57;  n(34) =-0.33662250574171;
I(35)= 20;   J(35) =20;  n(35) = 0.89185845355421d-24;
I(36)= 20;   J(36) =35;  n(36) =0.30629316876232d-12;
I(37)= 20;   J(37) =48;  n(37) =-0.42002467698208d-5;
I(38)= 21;   J(38) =21;  n(38) =-0.59056029685639d-25;
I(39)= 22;   J(39) =53;  n(39) =0.37826947613457d-5;
I(40)= 23;   J(40) =39;  n(40) =-0.12768608934681d-14;
I(41)= 24;   J(41) =26;  n(41) =0.73087610595061d-28;
I(42)= 24;   J(42) =40;  n(42) =0.55414715350778d-16;
I(43)= 24;   J(43) =58;  n(43) =-0.94369707241210d-6;

g_p_r = zeros(size(P)); % empty matrix for residual Gibbs free energy

for i=1:43
    g_p_r = g_p_r + n(i).*I(i).*(p_i).^(I(i)-1).*(tau-0.5).^J(i);
end

% specific volume in m^3/kg
v2 = R.*1000.*T./(P.*1d6).*p_i.*(g_p_0 + g_p_r);

%------------------------------------------------------------------------
% calculation of specific volume in region 3 using the equation of Marcus.
% Marcus' equations are valid for 720K < T < 880K and 40MPa < p < 100 MPa
% and are assumed by Marcus to be reasonable for 670K < T < 880K and 
% 35 MPa < p < 100 MPa. A visual check reveals that the density at lower
% temperatures (< 700K) looks good at low pressure (< 50 MPa).
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

RM = 8.31451;% Gas constant [J/mol/K]
T0 = 637; % reference temperature [K]
PM = P; % pressure in MPa

alpha0 = -1.7e-5 + 0.44.*PM.^(-2);
alpha1 = -0.2e-5 + 0.55.*PM.^(-2);

E = 18 + 0.028.*PM.^2;
p_M = 1 - exp(-E./RM./(T-T0)); % probability of bond formation

%------------------------------------------------------------------------
% molar volumes for water with different number of hydrogen bonds
%------------------------------------------------------------------------

V00 = 27.1 - 0.27.*PM;
V01 = 29.6 - 0.072.*PM;
V02 = 3/2*V01;

V0 = V00.*(1 + alpha0.*(T - T0).^2);
V1 = V01.*(1 + alpha1.*(T - T0).^2);
V2 = V02.*(1 + alpha1.*(T - T0).^2);

%------------------------------------------------------------------------
% fraction of water with different number of hydrogen bonds
%------------------------------------------------------------------------

f0 = 1./(1 + 4.*p_M./(1 - p_M) + 6.*p_M.^2./(1 - p_M).^2);
f1 = 4.*p_M.*f0./(1 - p_M);
f2 = 6.*p_M.^2.*f0./(1 - p_M).^2;

%------------------------------------------------------------------------
% specific volume calculated from molar volume fractions for different
% numbers of hydrogen bonds
%------------------------------------------------------------------------

Vmol = f0.*V0 + f1.*V1 + f2.*V2; % molar volume [cm^3/mol]

v_Marcus = Vmol./18.015./1000; % specific volume [m^3/kg]


%------------------------------------------------------------------------
% determine, which region applies
%------------------------------------------------------------------------

% first: distinguish between region 1 and region 3 (Marcus' formalism)

% use Marcus for temperatures > 720K
decide = T > 720;

v1 = -v1.*(decide-1) + v_Marcus.*(decide);

% at pressures below 50 Mpa, use Marcus for temperatures > 660 K

decide_p = P < 500;
decide_T = T > 660;

decide = decide_p.*decide_T;

v1 = -v1.*(decide-1) + v_Marcus.*(decide);

% second: distinguish between liquid like (2 and 3) and gas like (1)
% regions


% decide, if given p/T combinations are in liquid or vapor regime
decide = P >= p_b;


v = v1.*decide - v2.*(decide-1);

rho = 1./v; % density in kg/m^3