function [rho, cv, cp, s, u, h, f, JT, pv, cs] = eos_co2(press,temp)

% EOS_CO2 -  calculates thermodynamic properties of CO2 
%     based on an equation of state
%
%     Usage: [rho cv cp s u h f JT pv cs] = eos_co2(p,T)
% 
%     Input parameters (scalars or vectors): 
%         p       - pressure [MPa]     (0 < p < 800) 
%         T       - temperature [degC] (-57 (0)< T < 827)
%
%     Output parameters:
%         rho     - density [kg/m^3]
%         cv      - specific isochoric heat capacity [J/(kg K)]
%         cp      - specific isobaric heat capacity [J/(kg K)]
%         s       - specific entropy [J/(kg K)] 
%         u       - specific internal energy [J/kg]
%         h       - specific enthalpy [J/kg] 
%         f       - fugacity coefficient 
%         JT      - Joule-Thomson coefficient [K/MPa]
%         pv      - vapor pressure [MPa]
%         cs      - saturated liquid heat capacity [J/kg/K]
%
%     All thermodynamic properties (except for pv) are calculated from the 
%     dimensionless Helmholtz energy Phi = A/(RT) 
%     ( A(rho,T) = h_0(T)-R*T-T*s_0(rho,T) ) and its derivatives with
%     respect to p and T. 
%     The reduced Helmholtz energy is split into an ideal gas part (PHI_0)
%     and a residual part (PHI_r):
%        A/(R*T) = PHI(del,tau) = PHI_0 + PHI_r
%     with  del = RHO/RHOc  and  tau = Tc/T
%     The pressure is related to the Helmholtz energy by:
%        P(RHO,T) = -(dA/dV)_T  
%     and thus to the reduced Helmholtz energy PHI by:
%        P(del,tau)/(RHO*R*T) = 1 + del*(dPHI_r/ddel)_tau
%     Rearranging gives:
%        RHO = P / (R*T*(1+del*(dPHI_r/ddel)_tau))
%     Since the density is part of both sides of the above equation, RHO is
%     determined iteratively by minimizing the following expression: 
%        0 = RHO - RHOeos
% 
%     References: 
%     Span & Wagner (1996) J. Phys. Chem. Ref. Data 25, 1509-1597.
% 
%     Comments:
%     Caution should be taken because although the paper states that the
%     equations are valid for temperatures between -57 degC and 827 degC,
%     the curves show singularities at temperatures below 0 degC.
%     The exact positions depend on p and occur mainly, but not exclusively,
%     at temperatures below -20 degC.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------
 
% ------------------------------------------------------------------
%  Modification history: 
%       December 2009: Parameter names in header corrected (lines 15-20)
%       September 2011: name changed in function call: 
%                       phase_co2h2o -> phase_co2sw
%       Juli 2012:     corrected "usage" line in header
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('EOS_CO2:Input','EOS_CO2: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0 | press>800) && ~any(temp< -57 | temp>827)
else
    bad = find(press<0 | press > 800);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-57 | temp>827);
    temp(bad)=NaN*ones(size(bad));
    warning('EOS_CO2:OutOfParameterRange','EOS_CO2: Input values out of range!')
    warning('EOS_CO2:OutOfParameterRange','Equations valid for p = 0-800 MPa, T=(-57) - 827 degC.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T] = ndgrid(press,temp+273.15); %  temperature in K

rhoc = 467.6; % critical density of CO2 [kg/m^3]
Tc = 304.1282; % critical temperature of CO2 [K]
pc = 7.3773;   % critical pressure in [MPa]
R = 0.1889241; % specific gas constant [kJ/(kg K)]
tau = Tc./T; % inverse of reduced temperature

% define empty data structure for density 
rho= zeros(size(P,1),size(P,2),size(P,3)); 

%------------------------------------------------------------------------
% calculation of starting guess for RHO
%------------------------------------------------------------------------

% call external funtion to calculate vapor pressure
[~,~,pv]=phase_co2sw(temp,0); 
Pv= zeros(size(press,2),size(temp,2)); % initialize matrix for pv
for i=1:size(press,2) % create matrices for phase boundaries
Pv(i,:) = pv(:); 
end




%------------------------------------------------------------------------
% calculation of density by solving the EOS 
%------------------------------------------------------------------------

% Check if arrays for P and T contain NaN. If yes, also set corresponding 
% density values to NaN. Calculate density for allowed P and T only.
% If the fit function returns EXITFLAG <= 0, the iteration process has
% terminated with a non-satisfying result. In this case, the initial value
% is changed and the fit procedure is repeated.

options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction
 
% call minimizing function
for i=1:size(P,1)
    for k=1:size(P,2)
        if isnan(P(i,k)); % make sure that input value for P is a number
            rho(i,k) = NaN;
        elseif isnan(T(i,k)); % make sure that input value for T is a number
            rho(i,k) = NaN;
        else 
             EXITFLAG = -1; % Initialize EXITFLAG value
            counter = 0; % set counter to 0
            % use internal function to look up first guess for density 
            rho_init1 = guessfunc(P(i,k),T(i,k),R,Pv(i,k));
            rho_init = rho_init1; % first initial value for density
            while (EXITFLAG < 1) 
                counter = counter + 1;
        [rho(i,k),~,EXITFLAG]= fsolve(@(rho_co2) func(rho_co2,P(i,k),T(i,k),rhoc,Tc),rho_init,options);
        % change initial value for fit
        rho_init = rho_init1 + counter*1*(-1)^(counter); 
                 if rho(i,k) <= 0
                    EXITFLAG = -1; % reject negative densities  
                end
                 if rho_init <= 0
                    EXITFLAG = 10; % make sure, that while loop terminates  
                end
            end
        end
    end
end

delta = rho./rhoc; % reduced density

%------------------------------------------------------------------------
% calculation of ideal gas part of Helmholtz energy and derivatives
%------------------------------------------------------------------------
[phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta,tau);

%------------------------------------------------------------------------
% calculation of residual part of Helmholtz energy and derivatives
%------------------------------------------------------------------------
[phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta,tau);

%------------------------------------------------------------------------
% calculation of derivative of vapor pressure with respect to Temperature
%------------------------------------------------------------------------
[pv, dpvdT] = vappress(T,Tc,pc);

%------------------------------------------------------------------------
% calculation of thermodynamic properties
%------------------------------------------------------------------------

% Entropy [J/kgK]:
s = R.*(tau.*(dphi0dtau + dphirdtau) - phi0 - phir).*1000 ;

% Internal Energy [J/kg]:
u = R.*T.*tau.*(dphi0dtau + dphirdtau).*1000;

% Isochoric heat capacity [J/kgK]:
cv = -R.*tau.^2.*(dphi0dtautau + dphirdtautau).*1000;

% Enthalpy [J/kg]: 
h = R.*T.*(1 + tau.*(dphi0dtau + dphirdtau) + delta.*dphirddelta).*1000;

% Isobaric heat capacity [J/kgK]:
cp = R.*(-tau.^2.*(dphi0dtautau + dphirdtautau) + ...
    (1 + delta.*dphirddelta - delta.*tau.*dphirddeltatau).^2./...
    (1 + 2.*delta.*dphirddelta + delta.^2.*dphirddeltadelta)).*1000;

% Fugacity coefficient:
f = exp(phir + delta.*dphirddelta - log(1 + delta .*dphirddelta));

% Saturated liquid heat capacity [J/kgK]
cs =  R.*(-tau.^2.*(dphi0dtautau + dphirdtautau) + ...
    (1 + delta.*dphirddelta - delta.*tau.*dphirddeltatau) ./...
    (1 + 2.*delta.*dphirddelta + delta.^2.*dphirddeltadelta) .* ...
    ((1 + delta.* dphirddelta - delta.*tau.*dphirddeltatau) - ...
    1./rhoc./R./delta.*dpvdT)).*1000;

% Joule-Thomson coefficient [K/kPa]
JT = 1./R./rho .*(-(delta.*dphirddelta + delta.^2.*dphirddeltadelta + ...
    delta.*tau.*dphirddeltatau)./((1 + delta.*dphirddelta - ...
    delta.*tau.*dphirddeltatau).^2 - tau.^2.*(dphi0dtautau + ...
    dphirdtautau).*(1 + 2.*delta.*dphirddelta + delta.^2.*dphirddeltadelta)));

JT = JT .* 1000 ; % convert to [K/MPa]

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta,tau)

%  calculation of the ideal gas part (phi0) of the Helmholtz energy and its
%  first and second derivatives with respect to the reduced temperature
%  (dphi0dtau, dphi0dtautau)

% a, theta: coefficients of the correlation equation
a1 =   [8.37304456
        -3.70454304
        2.5];
    
 a2 =   [1.99427042
        0.62105248
        0.41195293
        1.04028922
        0.08327678];
    
theta =[3.15163
        6.1119
        6.77708
        11.32384
        27.08792];

% calculate sum terms
sum1 = zeros(size(tau,1),size(tau,2));
sum2 = zeros(size(tau,1),size(tau,2));
sum3 = zeros(size(tau,1),size(tau,2));

for i = 1:5
    sum1 = sum1 + a2(i).*log(1 -exp(-tau.*theta(i)));
    sum2 = sum2 + a2(i).*theta(i).*((1 -exp(-tau.*theta(i))).^(-1)-1);
    sum3 = sum3 + a2(i).*theta(i).^2.*exp(-tau.*theta(i))...
        .*(1 -exp(-tau.*theta(i))).^(-2);
end

% calculate phi and derivatives
phi0 = log(delta) + a1(1) + a1(2).*tau + a1(3).* log(tau) + sum1;  
dphi0dtau = a1(2) + a1(3)./tau + sum2;
dphi0dtautau = - a1(3)./tau.^2 - sum3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta,tau)

%  calculation of the residual part (phir) of the Helmholtz energy and its
%  first and second derivatives with respect to the inverse reduced temperature
%  (dphirdtau and dphirdtautau) and first and second derivatives with respect to
%  the reduced density (dphirddelta, dphirddeltadelta) as well as 
%  the mixed derivative (dphirddeltadtau)

%----------------------------------------------------------------------
% calculation of sum terms with parameters ni,di,ti
%----------------------------------------------------------------------

ni = [0.38856823203161D+0
      0.29385475942740D+1
     -0.55867188534934D+1
     -0.76753199592477D+0
      0.31729005580416D+0
      0.54803315897767D+0
      0.12279411220335D+0];
  
di = [1 1 1 1 2 2 3];
ti = [0. .75 1. 2. .75 2. .75];

sumr1 =  zeros(size(tau,1),size(tau,2));
sumrd1 =  zeros(size(tau,1),size(tau,2));
sumrdd1 =  zeros(size(tau,1),size(tau,2));
sumrt1 =  zeros(size(tau,1),size(tau,2));
sumrtt1 =  zeros(size(tau,1),size(tau,2));
sumrdt1 =  zeros(size(tau,1),size(tau,2));

for i = 1:7
    sumr1 = sumr1 + ni(i).*delta.^di(i).*tau.^ti(i);
    sumrd1 = sumrd1 + ni(i).*di(i).*delta.^(di(i)-1).*tau.^ti(i);
    sumrdd1 = sumrdd1 + ni(i).*di(i).*(di(i)-1).*delta.^(di(i)-2).*tau.^ti(i);
    sumrt1 = sumrt1 + ni(i).*ti(i).*delta.^di(i).*tau.^(ti(i)-1);
    sumrtt1 = sumrtt1 + ni(i).*ti(i).*(ti(i)-1).*delta.^di(i).*tau.^(ti(i)-2);
    sumrdt1 = sumrdt1 + ni(i).*ti(i).*di(i).*delta.^(di(i)-1).*tau.^(ti(i)-1);
end


%----------------------------------------------------------------------
% calculation of sum terms with parameters nj,dj,tj,cj
%----------------------------------------------------------------------

nj = [0.21658961543220D+1
      0.15841735109724D+1
     -0.23132705405503D+0
      0.58116916431436D-1
     -0.55369137205382D+0
      0.48946615909422D+0
     -0.24275739843501D-1
      0.62494790501678D-1
     -0.12175860225246D+0
     -0.37055685270086D+0
     -0.16775879700426D-1
     -0.11960736637987D+0
     -0.45619362508778D-1
      0.35612789270346D-1
     -0.74427727132052D-2
     -0.17395704902432D-2
     -0.21810121289527D-1
      0.24332166559236D-1
     -0.37440133423463D-1
      0.14338715756878D+0
     -0.13491969083286D+0
     -0.23151225053480D-1
      0.12363125492901D-1
      0.21058321972940D-2
     -0.33958519026368D-3
      0.55993651771592D-2
     -0.30335118055646D-3];
 
dj = [1 2 4 5 5 5 6 6 6 1 1 4 4 4 7 8 2 3 3 5 5 6 7 8 10 4 8];
tj = [1.5 1.5 2.5 0. 1.5 2. 0. 1. 2. 3. 6. 3. 6. 8. ...
      6. 0. 7. 12. 16. 22. 24. 16. 24. 8. 2. 28. 14.];
cj = [1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 3 3 3 4 4 4 4 4 4 5 6];

sumr2 =  zeros(size(tau,1),size(tau,2));
sumrd2 =  zeros(size(tau,1),size(tau,2));
sumrdd2 =  zeros(size(tau,1),size(tau,2));
sumrt2 =  zeros(size(tau,1),size(tau,2));
sumrtt2 =  zeros(size(tau,1),size(tau,2));
sumrdt2 =  zeros(size(tau,1),size(tau,2));

for i = 1:27
    sumr2 = sumr2 + nj(i).*delta.^dj(i).*tau.^tj(i).*exp(-delta.^cj(i));
    sumrd2 = sumrd2 + nj(i).*delta.^(dj(i)-1).*tau.^tj(i)....
        .*exp(-delta.^cj(i)).*(dj(i)-cj(i).*delta.^cj(i));
    sumrdd2 = sumrdd2 + nj(i).*delta.^(dj(i)-2).*tau.^tj(i)...
        .*exp(-delta.^cj(i)).*((dj(i)-cj(i).*delta.^cj(i))...
        .*(dj(i)-1-cj(i).*delta.^cj(i))-cj(i)^2.*delta.^cj(i));
    sumrt2 = sumrt2 + nj(i).*delta.^dj(i).*tau.^(tj(i)-1).*tj(i)...
        .*exp(-delta.^cj(i));
    sumrtt2 = sumrtt2 + nj(i).*delta.^dj(i).*tau.^(tj(i)-2).*tj(i)... 
        .*(tj(i)-1).*exp(-delta.^cj(i));
    sumrdt2 = sumrdt2 + nj(i).*delta.^(dj(i)-1).*tau.^(tj(i)-1).*tj(i)...
        .*exp(-delta.^cj(i)).*(dj(i)-cj(i).*delta.^cj(i));
end

%----------------------------------------------------------------------
% calculation of sum terms with parameters nk,dk,tk,ak,bk,ck,ek
%----------------------------------------------------------------------

nk = [-0.21365488688320D+3
       0.26641569149272D+5
      -0.24027212204557D+5
      -0.28341603423999D+3
       0.21247284400179D+3];

dk = [2 2 2 3 3];
tk = [1. 0. 1. 3. 3.];
ak = [25 25 25 15 20];
bk = [325 300 300 275 275];
ck = [1.16 1.19 1.19 1.25 1.22];
ek = [1. 1. 1. 1. 1.];


sumr3 =  zeros(size(tau,1),size(tau,2));
sumrd3 =  zeros(size(tau,1),size(tau,2));
sumrdd3 =  zeros(size(tau,1),size(tau,2));
sumrt3 =  zeros(size(tau,1),size(tau,2));
sumrtt3 =  zeros(size(tau,1),size(tau,2));
sumrdt3 =  zeros(size(tau,1),size(tau,2));

for i = 1:5
    sumr3 = sumr3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2);
    sumrd3 = sumrd3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(dk(i)./delta - 2.*ak(i).*(delta -ek(i)));
     sumrdd3 = sumrdd3 + nk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(- 2.*ak(i).*delta.^dk(i) + 4.*ak(i)^2.*delta.^dk(i)...
        .*(delta - ek(i)).^2 - 4.*dk(i).*ak(i).*delta.^(dk(i)-1)...
        .*(delta - ek(i)) + dk(i).*(dk(i)-1).*delta.^(dk(i)-2));
     sumrt3 = sumrt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(tk(i)./tau - 2.*bk(i).*(tau -ck(i)));
     sumrtt3 = sumrtt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*((tk(i)./tau - 2.*bk(i).*(tau -ck(i))).^2 - tk(i)./tau.^2 - 2.*bk(i));
     sumrdt3 = sumrdt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(tk(i)./tau - 2.*bk(i).*(tau -ck(i)))...
        .*(dk(i)./delta - 2.*ak(i).*(delta -ek(i)));
end

%----------------------------------------------------------------------
% calculation of sum terms with parameters nl,al,bl,betl,Al,Bl,Cl,Dl
%----------------------------------------------------------------------

nl = [-0.66642276540751D+0
       0.72608632349897D+0
       0.55068668612842D-1];
al = [3.5 3.5 3.];
bl = [.875 .925 .875];
betl = [.3 .3 .3];
Al = [.7 .7 .7];
Bl = [.3 .3 1.];
Cl = [10. 10. 12.5];
Dl = [275 275 275];

sumr4 =  zeros(size(tau,1),size(tau,2));
sumrd4 =  zeros(size(tau,1),size(tau,2));
sumrdd4 =  zeros(size(tau,1),size(tau,2));
sumrt4 =  zeros(size(tau,1),size(tau,2));
sumrtt4 =  zeros(size(tau,1),size(tau,2));
sumrdt4 =  zeros(size(tau,1),size(tau,2));

for i = 1:3
    % calculation of terms and their derivatives which are frequently used in the sums 
    psi = exp(-Cl(i).*(delta-1).^2 - Dl(i).*(tau -1).^2);
    dpsidd = -2.*Cl(i)*(delta -1).*psi;                 % d(psi)/d(delta)
    dpsiddd = (2.*Cl(i)*(delta -1).^2-1).*2.*Cl(i).*psi;% d^2(psi)/d(delta^2)
    dpsidt = -2.*Dl(i)*(tau -1).*psi;                   % d(psi)/d(tau)
    dpsidtt = (2.*Dl(i)*(tau -1).^2-1).*2.*Dl(i).*psi;  % d^2(psi)/d(tau^2)
    dpsiddt = 4.*Cl(i).*Dl(i).*(delta -1).*(tau - 1).*psi;% d^2(psi)/d(delta)d(tau)
    
    theta = (1 - tau) + Al(i).*((delta -1).^2).^(1/(2*betl(i)));
    
    Delta = theta.^2 + Bl(i).*((delta - 1).^2).^al(i);
    dDeltadd = (delta -1)...
        .*(Al(i).*theta.*2./betl(i).*((delta-1).^2).^(1./(2.*betl(i))-1)...
        + 2.*Bl(i).*al(i).*((delta -1).^2).^(al(i)-1)); % d(Delta)/d(delta)
    dDeltaddd = 1./(delta -1).*dDeltadd + (delta - 1).^2.*...
        (4.*Bl(i).*al(i).*(al(i)-1).*((delta - 1).^2).^(al(i)-2)...
        + 2.*Al(i).^2 .* (1./betl(i).^2) .* ...
        (((delta - 1).^2).^(1./2./betl(i)-1)).^2 + ...
        Al(i).*theta.*4./betl(i).*(1./2./betl(i) -1) .*...
        ((delta - 1).^2).^(1./2./betl(i) - 2)); % d^2(Delta)/d(delta^2)
    
    Deltab = Delta.^bl(i);
    dDeltabdd = bl(i).*Delta.^(bl(i)-1).*dDeltadd; % d(Deltab)/d(delta)
    dDeltabddd = bl(i).*(Delta.^(bl(i)-1).*dDeltaddd + ...
        (bl(i)-1).*Delta.^(bl(i)-2).*dDeltadd.^2); % d^2(Delatb)/d(delta^2)
    dDeltabdt = -2.*theta.*bl(i).*Delta.^(bl(i)-1); % d(Deltab)/d(tau)
    dDeltabdtt = 2.*bl(i).*Delta.^(bl(i)-1) + ...
        4 .* theta.^2 .* bl(i).*(bl(i)-1).*Delta.^(bl(i)-2); % d^2(Deltab)/d(tau^2)
    dDeltabddt = - Al(i).*bl(i).*2./betl(i).*Delta.^(bl(i)-1).*(delta-1)...
        .*((delta-1).^2).^(1./2./betl(i)-1) - ...
        2.*theta.*bl(i).*(bl(i)-1).*Delta.^(bl(i)-2).*dDeltadd;% d^2(Deltab)/d(delta)d(tau)
    
    sumr4 = sumr4 + nl(i).*Deltab.*delta.*psi;
    sumrd4 = sumrd4 + nl(i).*(Deltab .*(psi + delta .* dpsidd) ...
        + dDeltabdd.*delta.*psi);
    sumrdd4 = sumrdd4 + nl(i).*(Deltab.*(2.*dpsidd + delta.*dpsiddd)...
        + 2.*dDeltabdd.*(psi + delta.*dpsidd) + dDeltabddd.*delta.*psi);
    sumrt4 = sumrt4 + nl(i).*delta.*(dDeltabdt.*psi + Deltab.*dpsidt);
    sumrtt4 = sumrtt4 + nl(i).* delta .*(dDeltabdtt .* psi ...
        + 2.*dDeltabdt .* dpsidt + Deltab .* dpsidtt);
    sumrdt4 = sumrdt4 + nl(i).*(Deltab .*(dpsidt + delta.*dpsiddt) + ...
        delta.*dDeltabdd .* dpsidt + dDeltabdt.*(psi + delta .* dpsidd) ...
        + dDeltabddt .* delta .* psi);
    
end


%----------------------------------------------------------------------
% calculation of function output values
%----------------------------------------------------------------------

phir = sumr1 + sumr2 + sumr3 + sumr4; % phi_r: residual of Helmholtz energy
dphirddelta = sumrd1 + sumrd2 + sumrd3 + sumrd4;        % d(phi_r)/d(delta)
dphirddeltadelta = sumrdd1 + sumrdd2 + sumrdd3 + sumrdd4; % d^2(phi_r)/d(delta^2)
dphirdtau = sumrt1 + sumrt2 + sumrt3 + sumrt4;          % d(phi_r)/d(tau)
dphirdtautau = sumrtt1 + sumrtt2 + sumrtt3 + sumrtt4;   % d^2(phi_r)/d(tau^2)
dphirddeltatau = sumrdt1 + sumrdt2 + sumrdt3 + sumrdt4; % d^2(phi_r)/d(delta)d(tau)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ps, dpsdT] = vappress(T,Tc,pc)

%  calculation of the vapor pressure p_s and its first derivative d(p_s)/d(T)

% coefficients a and t
a = [ -7.0602087 1.9391218 -1.6463597 -3.2995634];
t = [1 1.5 2 4];

%calculation of sum terms
sum1 = zeros(size(T,1),size(T,2));
sum2 = zeros(size(T,1),size(T,2));

for i = 1:4
    sum1 = sum1 + a(i).*(1-T./Tc).^t(i);
    sum2 = sum2 + a(i).*t(i).*(1-T./Tc).^(t(i)-1);
end

% calculation of p_s and d(p_s)/d(T)
ps = pc.*exp((Tc./T.*sum1));
dpsdT = pc.*(-Tc./T.^2.*sum1 - 1./T.*sum2) .* exp(Tc./T.*sum1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = func(RHO,P,T,RHOc,Tc)
% minimization function to calculate density of CO2

R = 0.1889241;  % +/- 0.0000116 kJ/kg/K specific gas constant
delta = RHO/RHOc;
tau = Tc./T;
[~, ~, ~, ~, dphirddelta] = helmholtz_residual(delta,tau);
RHOeos = 1000 * P / (R*T*(1+delta*dphirddelta));
f = RHO - RHOeos;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function guess = guessfunc(P,T,R,Pv)
if P<Pv
% If the pressure is below the vapor pressure, the ideal gas equation is used to calculate a first guess for the density
R = R*1000; % [J/kg/K];
P = P * 1000000; % [Pa]

guess = P./T./R;
else
 g = [1173.4
     1166.14
     1158.8
     1151.4
     1143.9
     1136.3
     1128.68
     1120.9
     1113.8
     1105.1
     1097.1
     1088.87
     1080.6
     1072.1
     1063.5
     1054.8
     1045.97
     1036.9
     1027.7
     1018.3
     1008.7
     998.89
     988.8
     978.5
     967.9
     957
     945.83
     934.3
     922.3
     909.9
     897
     883.58
     869.5
     854.7
     839.1
     822.5
     804.67
     785.3
     764.1
     740.2
     712.8
     679.24
     633.7
     530.3];
t_i = T-216;
 t_i = ceil(t_i/2);
t_i(t_i < 1) = 1;
t_i(t_i > 43) = 43;

guess = g(t_i);
end


