function [rho, cv, cp, s, u, h, f] = eos_n2(press,temp)

% EOS_N2 -  calculates thermodynamic properties of gaseous (supercritical) 
%     N2 based on an equation of state
%
%     Usage: [rho cv cp s u h f] = eos_n2(p,T)
% 
%     Input parameters (scalars or vectors): 
%         p       - pressure [MPa]     (0 < p < 1000) 
%         T       - temperature [degC] (-73 (0)< T < 727)
%
%     Output parameters:
%         rho     - density [kg/m^3]
%         cv      - specific isochoric heat capacity [J/(kg K)]
%         cp      - specific isobaric heat capacity [J/(kg K)]
%         s       - specific entropy [J/(kg K)] 
%         u       - specific internal energy [J/kg] 
%         h       - specific enthalpy [J/kg]
%         f       - fugacity coefficient
%
%     All thermodynamic properties (except for pv) are calculated from the 
%     dimensionless Helmholtz energy PHI = A/(RT) 
%     ( A(rho,T) = h_0(T)-R*T-T*s_0(rho,T) ) and its derivatives with 
%     respect to p and T. 
%     The reduced Helmholtz energy is split into an ideal gas part (PHI_0)
%     and a residual part (PHI_r):
%         A/(R*T) = PHI(del,tau) = PHI_0 + PHI_r
%     with  del = RHO/RHOc  and  tau = Tc/T
%     The pressure is related to the Helmholtz energy by:
%         P(RHO,T) = -(dA/dV)_T  
%     and thus to the reduced Helmholtz energy PHI by:
%         P(del,tau)/(RHO*R*T) = 1 + del*(dPHI_r/ddel)_tau
%     Rearranging gives:
%         RHO = P / (R*T*(1+del*(dPHI_r/ddel)_tau))
%     Since the density is part of both sides of the above equation, 
%     RHO is determined iteratively by minimizing the following expression:
%         0 = RHO - RHOeos
%
%     Reference: 
%     Span et al. (2000) J. Phys. Chem. Ref. Data 29, 1361-1433.
%
%     Comments: 
%     For (supercritical) gas phase only (to keep things simple).
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (Juli 2012)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('EOS_N2:Input','EOS_N2: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0 | press>2200) && ~any(temp< -73 | temp>727)
else
    bad = find(press<0 | press > 1000);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-73 | temp>727);
    temp(bad)=NaN*ones(size(bad));
    warning('EOS_N2:OutOfParameterRange','EOS_N2: Input values out of range!')
    warning('EOS_N2:OutOfParameterRange','Equations valid for p = 0-1000 MPa, T=(-73) - 727 degC.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T] = ndgrid(press*1e6,temp+273.15); %  pressure in Pa, temperature in K

rhoc = 313.3; % critical density of CO2 [kg/m^3]
Tc = 126.192; % critical temperature of CO2 [K]
R = 8.314510; % gas constant [J/(mol K)]
tau = Tc./T; % inverse of reduced temperature
mm  = 28.01348; % molar mass (g/mol)

% define empty data structure for density 
rho= zeros(size(P,1),size(P,2)); 


%------------------------------------------------------------------------
% calculation of density by solving the EOS 
%------------------------------------------------------------------------

% Check if arrays for P and T contain NaN. If yes, also set corresponding 
% density values to NaN. Calculate density for allowed P and T only.
% If the fit function returns EXITFLAG <= 0, the iteration process has
% terminated with a non-satisfying result. In this case, the initial value
% is changed and the fit procedure is repeated.

options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction
 
% call minimizing function
for i=1:size(P,1)
    for k=1:size(P,2)
        if isnan(P(i,k)); % make sure that input value for P is a number
            rho(i,k) = NaN;
        elseif isnan(T(i,k)); % make sure that input value for T is a number
            rho(i,k) = NaN;
        else 
             EXITFLAG = -1; % Initialize EXITFLAG value
            counter = 0; % set counter to 0
            % use ideal gas law for first guess for density 
            rho_init1 = P(i,k)./R./T(i,k).*mm./1000;
            rho_init = rho_init1; % first initial value for density
            while (EXITFLAG < 1) 
                counter = counter + 1;
        [rho(i,k),~,EXITFLAG]= fsolve(@(rho_co2) func(rho_co2,P(i,k),T(i,k),rhoc,Tc,mm,R),rho_init,options);
        % change initial value for fit
        rho_init = rho_init1 + counter*1*(-1)^(counter); 
                 if rho(i,k) <= 0
                    EXITFLAG = -1; % reject negative densities  
                end
                 if rho_init <= 0
                    EXITFLAG = 10; % make sure, that while loop terminates  
                end
            end
        end
    end
end

delta = rho./rhoc; % reduced density


%------------------------------------------------------------------------
% calculation of ideal gas part of Helmholtz energy and derivatives
%------------------------------------------------------------------------
[phi0, dphi0dtau ,dphi0dtautau] = helmholtz_idealgas(delta,tau);

%------------------------------------------------------------------------
% calculation of residual part of Helmholtz energy and derivatives
%------------------------------------------------------------------------
[phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta,tau);

%------------------------------------------------------------------------
% calculation of thermodynamic properties
%------------------------------------------------------------------------

% Entropy [J/kgK]:
s = R.*(tau.*(dphi0dtau + dphirdtau) - phi0 - phir); % [J/mol/K]
s = s/mm*1000;  % [J/kg/K]

% Internal Energy [J/kg]:
u = R.*T.*tau.*(dphi0dtau + dphirdtau); % [J/mol]
u = u/mm*1000;  % [J/kg]

% Isochoric heat capacity [J/kgK]:
cv = -R.*tau.^2.*(dphi0dtautau + dphirdtautau); % [J/mol/K]
cv = cv/mm*1000;  % [J/kg/K]

% Enthalpy [J/kg]: 
h = R.*T.*(1 + tau.*(dphi0dtau + dphirdtau) + delta.*dphirddelta); % [J/mol]
h = h/mm*1000;  % [J/kg]

% Isobaric heat capacity [J/kgK]:
cp = R.*(-tau.^2.*(dphi0dtautau + dphirdtautau) + ...
    (1 + delta.*dphirddelta - delta.*tau.*dphirddeltatau).^2./...
    (1 + 2.*delta.*dphirddelta + delta.^2.*dphirddeltadelta)); % [J/mol/K]
cp = cp/mm*1000;  % [J/kg/K]

% Fugacity coefficient:
f = exp(phir + delta.*dphirddelta - log(1 + delta .*dphirddelta));

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = func(RHO,P,T,RHOc,Tc,Mm,R)
% minimization function to calculate density of N2

delta = RHO/RHOc;
tau = Tc./T;
[~, ~, ~, ~, dphirddelta] = helmholtz_residual(delta,tau);
RHOeos =  P*Mm /1000/ (R*T*(1+delta*dphirddelta));
f = RHO - RHOeos;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta,tau)

%  calculation of the ideal gas part (phi0) of the Helmholtz energy and its
%  first and second derivatives with respect to the reduced temperature
%  (dphi0dtau, dphi0dtautau)

% a, theta: coefficients of the correlation equation
a1 =  2.5;
a2 = -12.76952708;
a3 = -0.00784163;
a4 = -1.934819e-4;
a5 = -1.247742e-5;
a6 =  6.678326e-8;
a7 = 1.012941;
a8 = 26.65788;
    
% calculate phi and derivatives
phi0 = log(delta) + a1.*log(tau) + a2 + a3.*tau + a4./tau + a5./tau.^2 ...
       + a6./tau.^3 + a7.*log(1-exp(-a8.*tau));
dphi0dtau = (a1 + a3.*tau - a4./tau -2.*a5./tau.^2 - 3.*a6./tau.^3 ...
            + a7.*a8.*tau.*(1./(exp(a8.*tau)-1)))./tau;
dphi0dtautau = (-a1 + 2.*a4./tau + 6.*a5./tau.^2 + 12.*a6./tau.^3 ...
                -a7.*a8.^2.*tau.^2.*exp(a8.*tau)./(exp(a8.*tau)-1).^2)./tau.^2;

            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta,tau)

%  calculation of the residual part (phir) of the Helmholtz energy and its
%  first and second derivatives with respect to the inverse reduced temperature
%  (dphirdtau and dphirdtautau) and first and second derivatives with respect to
%  the reduced density (dphirddelta, dphirddeltadelta) as well as 
%  the mixed derivative (dphirddeltadtau)

%----------------------------------------------------------------------
% parameters N, i, j, l, PHI, beta, gamma
%----------------------------------------------------------------------

N = [0.924803575275
    -0.492448489428
     0.661883336938
    -0.192902619201d1
    -0.622469309629d-1
     0.349943957581
     0.564857472498
    -0.161720005987d1
    -0.481395031883
     0.421150636384
    -0.161962230825d-1
     0.172100994165
     0.735448924933d-2
     0.168077305479d-1
    -0.107626664179d-2
    -0.137318088513d-1
     0.635466899859d-3
     0.304432279419d-2
    -0.435762336045d-1
    -0.723174889316d-1
     0.389644315272d-1
    -0.212201363910d-1
     0.408822981509d-2
    -0.551990017984d-4
    -0.462016716479d-1
    -0.300311716011d-2
     0.368825891208d-1
    -0.255856846220d-2
     0.896915264558d-2
    -0.441513370350d-2
     0.133722924858d-2
     0.264832491957d-3
     0.196688194015d2
    -0.209115600730d2
     0.167788306989d-1
     0.262767566274d4];
 
 i =[1
     1
     2
     2
     3
     3
     1
     1
     1
     3
     3
     4
     6
     6
     7
     7
     8
     8
     1
     2
     3
     4
     5
     8
     4
     5
     5
     8
     3
     5
     6
     9
     1
     1
     3
     2];
 
 j =[0.25
     0.875
     0.5
     0.875
     0.375
     0.75
     0.5
     0.75
     2
     1.25
     3.5
     1
     0.5
     3
     0
     2.75
     0.75
     2.5
     4
     6
     6
     3
     3
     6
     16
     11
     15
     12
     12
     7
     4
     16
     0
     1
     2
     3];
 
 l =[0
     0
     0
     0
     0
     0
     1
     1
     1
     1
     1
     1
     1
     1
     1
     1
     1
     1
     2
     2
     2
     2
     2
     2
     3
     3
     3
     3
     4
     4
     4
     4
     2
     2
     2
     2];
 
 PHI = [20
       20
       15
       25];
 
 beta =[325
        325
        300
        275];
 
 gamma = [1.16
          1.16
          1.13
          1.25];
 
%----------------------------------------------------------------------
% create empty matrices for output parameters
%----------------------------------------------------------------------

phir = zeros(size(tau,1), size(tau,2)); 
dphirdtau  = zeros(size(tau,1), size(tau,2)); 
dphirdtautau  = zeros(size(tau,1), size(tau,2)); 
dphirddeltatau  = zeros(size(tau,1), size(tau,2)); 
dphirddelta  = zeros(size(tau,1), size(tau,2)); 
dphirddeltadelta = zeros(size(tau,1), size(tau,2)); 

%----------------------------------------------------------------------
% calculate residual reduced Helmholtz energy phir
%----------------------------------------------------------------------

 for k = 1:6
    phir = phir + N(k).*delta.^i(k).*tau.^j(k);
 end
 
  for k = 7:32
    phir = phir + N(k).*delta.^i(k).*tau.^j(k).*exp(-delta.^l(k));
 end
  
 for k = 33:36
    phir = phir + N(k).*delta.^i(k).*tau.^j(k)...
        .*exp(-PHI(k-32).*(delta - 1).^2 - beta(k-32).*(tau -gamma(k-32)).^2);
 end
 
%----------------------------------------------------------------------
% calculate first derivative with respect to reduced density: dphirddelta
%----------------------------------------------------------------------

 for k = 1:6
    dphirddelta = dphirddelta + i(k).*N(k).*delta.^i(k).*tau.^j(k);
 end
 
  for k = 7:32
    dphirddelta = dphirddelta + N(k).*delta.^i(k).*tau.^j(k).*exp(-delta.^l(k))...
        .*(i(k) - l(k).*delta.^l(k));
 end
  
 for k = 33:36
    dphirddelta = dphirddelta + N(k).*delta.^i(k).*tau.^j(k)...
        .*exp(-PHI(k-32).*(delta - 1).^2 - beta(k-32).*(tau -gamma(k-32)).^2)...
        .*(i(k) - 2.*delta.*PHI(k-32).*(delta - 1));
 end
  
 dphirddelta = dphirddelta./delta;
 
%----------------------------------------------------------------------
% calculate second derivative with respect to reduced density:
% dphirddeltadelta
%----------------------------------------------------------------------

 for k = 1:6
    dphirddeltadelta = dphirddeltadelta + i(k).*(i(k)-1).*N(k).*delta.^i(k).*tau.^j(k);
 end
 
  for k = 7:32
    dphirddeltadelta = dphirddeltadelta + N(k).*delta.^i(k).*tau.^j(k).*exp(-delta.^l(k))...
        .*((i(k) - l(k).*delta.^l(k)).*(i(k) - 1 - l(k).*delta.^l(k)) - l(k).^2.*delta.^l(k));
 end
  
 for k = 33:36
    dphirddeltadelta = dphirddeltadelta + N(k).*delta.^i(k).*tau.^j(k)...
        .*exp(-PHI(k-32).*(delta - 1).^2 - beta(k-32).*(tau -gamma(k-32)).^2)...
        .*((i(k) - 2.*PHI(k-32).*delta.*(delta - 1)).^2 ...
    - i(k) - 2.*delta.^2.*PHI(k-32));
 end
  
 dphirddeltadelta = dphirddeltadelta./delta.^2;
 
%----------------------------------------------------------------------
% calculate first derivative with respect to reduced temperature: dphirdtau
%----------------------------------------------------------------------

 for k = 1:6
    dphirdtau = dphirdtau + j(k).*N(k).*delta.^i(k).*tau.^j(k);
 end
 
  for k = 7:32
    dphirdtau = dphirdtau + j(k).*N(k).*delta.^i(k).*tau.^j(k).*exp(-delta.^l(k));
 end
  
 for k = 33:36
    dphirdtau = dphirdtau + N(k).*delta.^i(k).*tau.^j(k)...
        .*exp(-PHI(k-32).*(delta - 1).^2 - beta(k-32).*(tau -gamma(k-32)).^2)...
        .*(j(k) - 2.*tau.*beta(k-32).*(tau - gamma(k-32)));
 end
  
 dphirdtau = dphirdtau./tau;
 
%----------------------------------------------------------------------
% calculate second derivative with respect to reduced temperature:
% dphirdtautau
%----------------------------------------------------------------------

 for k = 1:6
    dphirdtautau = dphirdtautau + j(k).*(j(k) -1).*N(k).*delta.^i(k).*tau.^j(k);
 end
 
  for k = 7:32
    dphirdtautau = dphirdtautau + j(k).*(j(k)-1).*N(k).*delta.^i(k).*tau.^j(k).*exp(-delta.^l(k));
 end
  
 for k = 33:36
    dphirdtautau = dphirdtautau + N(k).*delta.^i(k).*tau.^j(k)...
        .*exp(-PHI(k-32).*(delta - 1).^2 - beta(k-32).*(tau -gamma(k-32)).^2)...
        .*((j(k) - 2.*tau.*beta(k-32).*(tau - gamma(k-32))).^2 ...
    - j(k) - 2.*tau.^2.*beta(k-32));
 end
  
 dphirdtautau = dphirdtautau./tau.^2;
 
%----------------------------------------------------------------------
% calculate mixed derivative with respect to reduced temperature and
% reduced density: dphirddeltatau
%----------------------------------------------------------------------

 for k = 1:6
    dphirddeltatau = dphirddeltatau + j(k).*i(k).*N(k).*delta.^i(k).*tau.^j(k);
 end
 
  for k = 7:32
    dphirddeltatau = dphirddeltatau + j(k).*N(k).*delta.^i(k).*tau.^j(k) ...
        .*exp(-delta.^l(k)).*(i(k) - l(k).*delta.^l(k));
 end
  
 for k = 33:36
    dphirddeltatau = dphirddeltatau + N(k).*delta.^i(k).*tau.^j(k)...
        .*exp(-PHI(k-32).*(delta - 1).^2 - beta(k-32).*(tau -gamma(k-32)).^2)...
        .*(j(k) - 2.*tau.*beta(k-32).*(tau - gamma(k-32))) ...
    .* (i(k) - 2.*delta.*PHI(k-32).*(delta - 1));
 end
  
 dphirddeltatau = dphirddeltatau./tau./delta;
 
 
 

