function [f_gh] = fugacity_ch4gh(temp,N_CH4)

% FUGACITY_CH4GH -  calculates the fugacity of CH4 in CH4-hydrate
%
%       Usage: f = fugacity_ch4gh(T,N)
%
%       Input parameters (scalars or vectors): 
%           T    - temperature [degC] 
%           N    - moles of CH4 per mole of water in hydrate phase (0 < N < 4/23)
%   
%       Output parameter:
%           f    - fugacity of CH4 in CH4_hydrate [MPa]
%
%       References:
%       Cole & Goodwin (1990) Chem. Eng. Sci. 45, 569-573.
%       Klauda & Sandler (2003) Chem. Eng. Sci. 58, 27-41. (Langmuir constants)
% 
%       Comments: 
%       Function returns NaN, if no solution for f exists. This
%       happens if N is too large (100% cage occupancy does not occur).
%
%       An approximate N can be derived from the output of the
%       toolbox function fugacity_h2omh: 
%       N = 1/23 * theta_s + 3/23 * theta_l
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       November 2009: indices set in correct order (lines 63-64)
%       September 2011: name changed in function call: 
%                       phase_ch4h2o -> phase_ch4sw
%       March 2013:     function was completely rewritten (different set of
%                       equations)
%------------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('FUGACITY_CH4GH:Input','FUGACITY_CH4GH: Missing input parameter! Please provide T and N!')
else

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[T, N] = ndgrid(temp+273.15, N_CH4); %  T in K

nu_s = 1/23; % number of small cavities
nu_l = 3/23; % number of large cavities


%------------------------------------------------------------------------
% Calculation of the Langmuir constants (s = small cage, l = large cage)
%------------------------------------------------------------------------

Ac_CH4_s = -23.6453;
Ac_CH4_l = -22.0651;
Bc_CH4_s = 2714.5643;
Bc_CH4_l = 2760.1604;
Dc_CH4_s = 0;
Dc_CH4_l = 0;

% Langmuir constants:
C_CH4_s = exp(Ac_CH4_s + Bc_CH4_s./T + Dc_CH4_s./T.^2);
C_CH4_l = exp(Ac_CH4_l + Bc_CH4_l./T + Dc_CH4_l./T.^2);

%------------------------------------------------------------------------
% Calculation of CH4 fugacity in CH4-hydrate
%------------------------------------------------------------------------

% solve quadratic equation
r1 = C_CH4_s .* C_CH4_l .* (N - nu_s - nu_l);
r2 = N.*C_CH4_s + N.*C_CH4_l - C_CH4_s.*nu_s - C_CH4_l.*nu_l;

p1 = r2./r1;
p2 = N./r1;

f_gh = - p1/2 + sqrt(p1.^2/4 - p2);

f_gh = f_gh.*1d-6; % fugacity in [MPa]
end
