function [fN2] = fugacity_n2sw(press,temp,sal,mn2)

% FUGACITY_N2SW -  calculates the fugacity of dissolved N2 gas
%     in equilibrium with seawater
%
%     Usage: f = fugacity_n2sw(p,T,S,m)
%
%     Input parameters (scalars or vectors, m: scalar/matrix of size pxtxS): 
%           p     -  pressure [MPa]               (0.1 < p < 35)
%           T     -  temperature [degC]           (0 < T < 317)
%           S     -  practical salinity           (S < 350)
%           m     -  molality of dissolved N2 in seawater [mol/kg],
%                       default is saturation concentration
%
%     Output parameters: 
%           f     -  fugacity of N2 in seawater [MPa]
% 
%     References: 
%     Harvey (1996) AIChE 42, 1491-1494. (Henry constant)
%     Plyasunov et al. (2000) GCA 64, 495-512. (partial molar volume of N2)
%     Mao & Duan (2006) Fluid Phase Equilibria 248, 103-114.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLUHENRY_N2SW:Input','SOLUHENRY_N2SW: Missing input parameter! Please provide p,T,S and pp(optional)!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>35) && ~any(temp<0 | temp>80) ...
        && ~any( sal<0 | sal>120)
else
    bad = find(press<0.1 | press > 35);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>120);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLUHENRY_N2SW:OutOfParameterRange','SOLUHENRY_N2SW: Input values out of range!')
    warning('SOLUHENRY_N2SW:OutOfParameterRange','Equations valid for p = 0.1-35 MPa, T= 0-80 degC, S = 0-120.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

   if nargin == 3 % if no N2 concentration is provided by user, assume saturation
       %call external function to calculate solubility
        mn2 = solu_n2sw(press,temp,sal);
   end

%------------------------------------------------------------------------
% Calculation of solubility
%------------------------------------------------------------------------
% call external function to calculate pressure and salinity corrected Henry
% constant
[~,~,kH] = soluhenry_n2sw(press,temp,sal,0.1); % [1/MPa]

x = mn2./(mn2 + 55.5); % mole fraction of N2

fN2 = x./kH; % fugacity

end
