function [f_sw] = fugacity_sw(press,temp,sal,M_CH4,M_CO2)

% FUGACITY_SW -  calculates the fugacity of water in solutions 
%       with dissolved salts, CH4 and CO2
%
%       Usage: f = fugacity_sw(p,T,S,mch4,mco2)
%
%       Input parameters (scalars or vectors): 
%           p       - pressure [MPa]      (0.1 < p < 400) 
%           T       - temperature [degC] 
%           S       - practical salinity  (0 < S < 350)
%           mch4    - molality of dissolved CH4 [mol/kg]
%           mco2    - molality of dissolved CO2 [mol/kg]
%   
%       Output parameter:
%           f       - fugacity of water in seawater with CO2 and CH4 [MPa]
%
%       References:
%       Jager et al. (2003) Fluid Phase Equilibria 211, 85-107.
%
%       Comments: 
%       Pressure range reduced from 5000 bar to 4000 bar to avoid
%       handling of different cases during integration. The activity of 
%       the solvent water is calculated and used to modify the pure water
%       fugacity as derived from the Wagner & Pruss EOS. 
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2009: pressure range in header corrected (line10)
%       September 2011: function call changed: eos_seawater -> eos_sw
%       September 2011: Output changed from fugacity coefficient to
%                       fugacity
%       November 2012: syntax changed to Matlab2012 style
%       December 2012: matrix extension for RSW and fw is now correctly
%                      handled, equation for a_NaCl corrected 
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% define global variables
%------------------------------------------------------------------------
global R I m_Na m_Cl RSW epsilon

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 5
     warning('FUGACITY_SW:Input','FUGACITY_SW: Missing input parameter! Please provide p, T, S, mch4 and mco2!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>400) && ~any(sal<(0) | sal>350)
else
    bad = find(press<0.1 | press > 400);
    press(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | temp>350);
    sal(bad)=NaN*ones(size(bad));
    warning('FUGACITY_SW:OutOfParameterRange','FUGACITY_SW: Input values out of range!')
    warning('FUGACITY_SW:OutOfParameterRange','Equations valid for p = 1-400 bar, S=0 - 350.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P, T, S, m_ch4, m_co2] = ndgrid(press.*10,temp+273.15, sal,M_CH4,M_CO2); %  p in bar,t in K

R = 8.314472; % gas constant [J/mol/K]
lch4 = length(M_CH4);
lco2 = length(M_CO2);

% call external function to calculate density of seawater
RSW_1 = density_sw(press,temp,sal);

% call external function to calculate the ionic strength
I = i_sw(S);

% call external function to calculate the fugacity coefficient of pure water
[~, ~, ~, ~, ~, ~, ~, ~, ~, ~, ~, ~, ~, fw] = eos_sw(press,temp,sal);

% extend matrix of I and RSW to the same size as P, T, ...
RSW = zeros(size(P));
FW = zeros(size(P));

if lco2 == 1 && lch4 == 1;
        RSW(:,:,:) = RSW_1;
        FW(:,:,:) = fw;
        
elseif lco2 == 1;
    
   for i=1:lch4
        RSW(:,:,:,i) = RSW_1;
        FW(:,:,:,i) = fw;
    end

else
    
for i=1:lch4
    for j = 1:lco2
        RSW(:,:,:,i,j) = RSW_1;
        FW(:,:,:,i,j) = fw;
    end
end
end

% calculation of molality of a NaCl solution of equivalent ionic strength
% as the seasalt solution
m_Na = I;
m_Cl = I;

%------------------------------------------------------------------------
% Calculation of the dielectric dielectric constant of water
%------------------------------------------------------------------------

a10 =  243.9576;
a20 =  0.039037;
a30 = -1.01261e-5;
a11 = -0.7520846;
a21 = -2.12309e-4;
a31 =  6.04961e-8;
a12 =  6.60648e-4;
a22 =  3.18021e-7;
a32 = -9.33341e-11;

A0 = a10 + a20.*P + a30 .*P.^2;
A1 = a11 + a21.*P + a31 .*P.^2;
A2 = a12 + a22.*P + a32 .*P.^2;

epsilon = A0 + A1.*T + A2.*T.^2; % dielectric constant of water


%------------------------------------------------------------------------
% Calculation of the activity of seawater
%------------------------------------------------------------------------

[a_sw] = activity(T,S,m_ch4,m_co2); % call subroutine for calculation

%------------------------------------------------------------------------
% Calculation of the fugacity coefficient
%------------------------------------------------------------------------

phi_sw  = FW.*a_sw; % fugacity coefficient
f_sw=phi_sw.*(P./10); % fugacity in [MPa]

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [a_sw] = activity(T,S,m_ch4, m_co2)
% calculation of the activity of the water

global R I m_Na m_Cl epsilon RSW gammaP1_Na gammaP1_Cl gammaP1_CO2

e = 1.60218e-19; % charge of one electron [C]
epsilon_0 = 8.85419e-12; % permittivity of vacuum [C^2/N/m^2]
Na = 6.0222045e23; % Avogadro's number
Mw = 18.01528/1000; % molecular weight of water [kg/mol]

B = -0.554860699 + 4.2795e-3.*T - 6.529e-6.*T.^2;
C = -0.016131327 -1.25089e-5.*T +  5.89e-8.*T.^2;
D = -1.12161e-3 + 2.49474e-5.*T - 4.603e-8.*T.^2;

%------------------------------------------------------------------------
% Calculation of activity of water with dissolved NaCl, CH4 and/or CO2
%------------------------------------------------------------------------

% Debeye-Hckel constant
A_DH = (e.^2./epsilon_0./epsilon./R./T).^(3/2).*Na.^2./8./pi.*sqrt(2.*RSW);
% activity coefficients
gamma_DH = 2.*A_DH .* ((1 - (1 + sqrt(I)).^2)./(1 + sqrt(I)) + ...
    2.*log(1+sqrt(I)));
gamma_B = (0.13816 + 0.6.*B).*I.*2./3.*((1 + 3.*I)./(1 + 3.*I./2).^2 - ...
    log(1 + 3./2.*I)./3.*2./I) + 2.*(0.5.*B.*I.^2 + ...
    2./3.*C.*I.^3 + 3./4.*D.*I.^4);
gamma_I = - Mw.*(2.*I + gamma_DH + gamma_B);
gammaP2_CH4 = -5e-3.*(1 - (1 + 2.*sqrt(I) + 2.*I).*exp(- 2.*sqrt(I)));


%Calculation of activity coefficients CH4-Cl and CH4-Na
gammaP1_Na = 0.025  - 5e-3 ./2./I .*(1 - (1 + 2.*sqrt(I)).*exp(-2.*sqrt(I)));
gammaP1_Cl = gammaP1_Na;

gammaP1_CO2 = 0.107 - 4.5e-4 .*T; % interaction: CO2-CO2

a_NaCl = gamma_I;
a_NaCl(S ==0) = 0;
a_Ch4Na = m_ch4.*m_Na .* (gammaP1_Na - gammaP2_CH4./2./I);
a_Ch4Na(S ==0) = 0;
a_Ch4Cl = m_ch4.*m_Cl .*(gammaP1_Cl - gammaP2_CH4./2./I);
a_Ch4Cl(S ==0) = 0;


% calculation of water activity
lna = a_NaCl - Mw.*(m_ch4 + m_co2 + ...
m_co2.^2 .*gammaP1_CO2 + a_Ch4Na + a_Ch4Cl);

a_sw = exp(lna);
