function c = heatcap_ch4gh(temp)

% HEATCAP_CH4GH - calculates the heat capacity of CH4 hydrate
%
%       Usage: cp = heatcap_ch4gh(T)
%
%       Input parameter (scalar or vector): 
%           T    - temperature [degC] (-25 < T < 11)
%   
%       Output parameter:
%           cp    - heat capacity of methane hydrate [J/kg/K]
%           
%       The heat capacity of methane hydrate is calculated from an
%       empirical equation as function of temperature. The effect of
%       pressure is assumed negligible (see e.g. the heat capacity of ice
%       tabulated in Feistel & Wagner(2006) J.Phys.Chem.Ref.Data 35, 10211047.
%       Heat capacities for hydrates and ice are generally believed to be
%       similar in values.
%
%       References:
%       Gupta (2007) PhD thesis, Colorado School of Mines.
%
%       Comments:
%       The calculated values seem to deviate from the experimental
%       values of Gupta for temperatures above 11 degC.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 1
     warning('HEATCAP_CH4GH:Input','HEATCAP_CH4GH: Missing input parameter! Please provide T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if  ~any(temp< -25 | temp>11)
else
    bad = find(temp<-25 | temp>11);
    temp(bad)=NaN*ones(size(bad));
    warning('HEATCAP_CH4GH:OutOfParameterRange','HEATCAP_CH4GH: Input values out of range!')
    warning('HEATCAP_CH4GH:OutOfParameterRange','Equations valid for T=(-25) - 11 degC.')
end

T = temp + 273.15; % temperature in K

% calculate c [J/g/K]
c = 1.9370547e-05.*T.^3 - 1.5151760e-02 .*T.^2 + 3.9553876 .*T - 342.70565;

c = c.*1000; %[J/kg/K]
end
