function lambda = heatcond_co2(press,temp)

% HEATCOND_CO2 - calculates the thermal conductivity of CO2 from a
%       multiparameter equation
%
%       Usage: lambda = heatcond_co2(p,T)
%
%       Input parameters (scalars or vectors): 
%           p          - pressure [MPa]     (0.1 < p < 200) 
%           T          - temperature [degC] (-48 < T < 727)
%   
%       Output parameter:
%           lambda     - thermal conductivity [W/m/K]
%
%       References:
%       Scalabrin et al. (2006) J.Phys.Chem.Ref.Data 35, 1549-1575.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------

% define global constants
global Tr rhor

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('HEATCOND_CO2:Input','HEATCOND_CO2: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>200) && ~any(temp<(-48) | temp>727)
else
    bad = find(press<0.1 | press > 200);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(-48) | temp>727);
    temp(bad)=NaN*ones(size(bad));
    warning('HEATCOND_CO2:OutOfParameterRange','HEATCOND_CO2: Input values out of range!')
    warning('HEATCOND_CO2:OutOfParameterRange','Equations valid for p = 0.1-200 MPa, T=(-48) - 727 degC.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[~, T] = ndgrid(press,temp+273.15); %  pressure in MPa, temperature in K

% call external funtion to calculate the density of CO2
rho = density_co2(press, temp);

% Critical point of CO2
% Pc = 7.3773;    % [MPa]
Tc = 304.1282;  % [K]
rho_c = 467.6;   % [kg/m^3]
lambda_c = 4.81384; % [mW/m/K]

Tr = T./Tc; % reduced temperature
rhor = rho./rho_c; % reduced density

nc = 0.775547504; % factor for lambda_ce

%------------------------------------------------------------------------
% calculation of the critical enhancement term
%------------------------------------------------------------------------

lambda_ce = critical_enhancement; 

%------------------------------------------------------------------------
% calculation of the thermal conductivity
%------------------------------------------------------------------------

lambda_r = thermal_conductivity; % calculation of the reduced thermal conductivity

lambda_r = lambda_r + nc.*lambda_ce; % add critical enhancement term

% calculate thermal conductivity from reduced thermal conductivity
lambda = lambda_r .* lambda_c ./1000; % [W/m/K] 

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function l_ce = critical_enhancement

% calculation of the critical enhancement term of the reduced thermal
% conductivity

% Definition of parameters and variables
global Tr rhor

a = [3
    6.70697
    0.94604
    0.3
    0.3
    0.39751
    0.33791
    0.77963
    0.79857
    0.9
    0.02
    0.2];

alpha = 1 - a(10).*acosh(1 + a(11).*((1 - Tr).^2).^a(12));

% Calculation of critical enhancement term

l_ce = rhor.*exp(-rhor.^a(1)./a(1)...
    - (a(2).*(Tr-1)).^2 - (a(3).*(rhor-1)).^2) ./ ...
    ((((1 - 1./Tr) + a(4).*((rhor - 1).^2).^(1/2/a(5))).^2).^a(6) ...
    + ((a(7) .* (rhor - alpha)).^2).^a(8)).^a(9);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function l_r = thermal_conductivity

% calculation of the the reduced thermal conductivity without critical
% enhancement

% Definition of parameters and variables
global Tr rhor

g1 = [0 0 1.5];
g2 = [0 1 1.5 1.5 1.5 3.5 5.5];
h1 = [1 5 1];
h2 = [1 2 0 5 9 0 0];

n1 = [ 7.69857587
     0.159885811
     1.56918621];
n2 = [-6.73400790
      16.3890156
      3.69415242
      22.3205514
      66.1420950
     -0.171779133
      0.00433043347];
 
 % calculation of reduced thermal conductivity
 
 sum1 = zeros(size(Tr,1), size(Tr,2));
 sum2 = zeros(size(Tr,1), size(Tr,2));
 
 for i = 1:3
     sum1 = sum1 + n1(i).*Tr.^g1(i) .* rhor.^h1(i);
 end
 
 for i = 1:7
     sum2 = sum2 + n2(i).*Tr.^g2(i) .* rhor.^h2(i);
 end
 
 l_r = sum1 + exp(-5.*rhor.^2).*sum2;
 