function lambda = heatcond_n2(press,temp)

% HEATCOND_N2 - calculates the thermal conductivity of N2 from a
%     multiparametric equation
%
%     Usage: lambda = heatcond_n2(p,T)
%
%     Input parameters (scalars or vectors): 
%           p        -  pressure [MPa]     (0.1 < p < 100), 
%           T        -  temperature [degC] (-120 < T < 727) 
%
%     Output parameter: 
%           lambda   - thermal conductivity [W/m/K]
%
%     References:
%     Lemmon & Jacobsen (2004) Int.J.Thermophys 25(1), 21-69.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (January 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('HEATCOND_N2:Input','HEATCOND_N2: Missing input parameter! Please provide p and T!')
else


%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) &&  ~any( temp<(-120) | temp>727)
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(-120) | temp>727);
    temp(bad)=NaN*ones(size(bad));
    warning('HEATCOND_N2:OutOfParameterRange','HEATCOND_N2: Input values out of range!')
    warning('HEATCOND_N2:OutOfParameterRange','Equations valid for p=0.1-100 MPa, T=(-120)-727 degC.')
end


%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[~,T] = ndgrid(press,temp+273.15); % P [MPa], T [K]

Tc = 126.192;   % critical temperature [K]
rhoc = 11.1839; % critical molarity [mol/dm^3]
% pc = 3.3958;    % critical pressure [MPa]
M = 28.01348;   % molar weight [g/mol]
ek = 98.94;     % Lennard Jones energy parameter epsilon/k [K]
sigma = 0.3656; % Lennard Jones parameter sigma [nm]

% call external function to calculate density of CO2
rho = eos_n2(press,temp); %[kg/m^3]
rho = rho/M; % [mol/dm^3]

tau = Tc./T;
Ts = T./ek;
delta = rho./rhoc;


b(1) = 0.431;
b(2) =-0.4623;
b(3) = 0.08406;
b(4) = 0.005341;
b(5) =-0.00331;

N(1) = 1.5112;
N(2) = 2.117;
N(3) = -3.332;

t(1) = -1;
t(2) = -0.7;

N2(1) =  8.862;
N2(2) =  31.11;
N2(3) = -73.13;
N2(4) =  20.03;
N2(5) = -0.7096;
N2(6) =  0.2672;

t2(1) = 0;
t2(2) = 0.03;
t2(3) = 0.2;
t2(4) = 0.8;
t2(5) = 0.6;
t2(6) = 1.9;

d(1) = 1;
d(2) = 2;
d(3) = 3;
d(4) = 4;
d(5) = 8;
d(6) = 10;

l(1) = 0;
l(2) = 0;
l(3) = 1;
l(4) = 2;
l(5) = 2;
l(6) = 2;

gamma(1) = 0;
gamma(2) = 0;
gamma(3) = 1;
gamma(4) = 1;
gamma(5) = 1;
gamma(6) = 1;


%------------------------------------------------------------------------
% Calculation of the dilute gas viscosity eta0
%------------------------------------------------------------------------

Omega = exp(b(1)+ b(2).*(log(Ts)) + b(3).*(log(Ts)).^2+ ...
    b(4).*(log(Ts)).^3+ b(5).*(log(Ts)).^4 );

eta0 = 0.0266958.*sqrt(M.*T)./(sigma.^2.*Omega);

%------------------------------------------------------------------------
% Calculation of the residual thermal conductivity lambdar
%------------------------------------------------------------------------
% create empty matrix)
lambdar = zeros(size(T));

for i = 1:6
    lambdar = lambdar + N2(i).*tau.^t2(i).*delta.^d(i).*exp(-gamma(i).*delta.^l(i));
end

%------------------------------------------------------------------------
% Calculation of the dilute gas thermal conductivity lambda0
%------------------------------------------------------------------------

lambda0 = N(1).*eta0 + N(2).*tau.^t(1) + N(3).*tau.^t(2);

%------------------------------------------------------------------------
% Calculation of the thermal conductivity lambda
%------------------------------------------------------------------------

lambda = lambda0 + lambdar; % [mW/m/K]
lambda = lambda*1d-3;% [W/m/K]

end