function lambda = heatcond_sw(press,temp,sal)

% HEATCOND_SW - calculates the thermal conductivity of seawater from a
%       multiparameter equation
%
%       Usage: lambda = heatcond_sw(p,T,S)
%
%       Input parameters (scalars or vectors): 
%           p           - pressure [MPa]       (0 < p < 140) 
%           T           - temperature [degC]   (0 < T < 60)
%           S           - practical salinity   (0 < S < 45)
%   
%       Output parameter:
%           lambda      - thermal conductivity [W/m/K]
%
%       Reference:
%       Caldwell (1974) Deep Sea Research 21, 131-137. 
%
%       Comments: 
%       The estimated accuracy for the equation is 0-5%.
%       The authors recommend their equation for salinities from 25-45,
%       but the calculated values for pure water are well within the 
%       accuracy. 
%           pure water @ 1 bar / 20 degC: 
%           lambda_calc = 0.604 [W/m/K]
%           lambda_ref = 0.598 [W/m/K] (tabulated reference value)
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('HEATCOND_SW:Input','HEATCOND_SW: Missing input parameter! Please provide p, T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0 | press>140) && ~any(temp<(0) | temp>60) ...
        && ~any(sal<(0) | sal>60)
else
    bad = find(press<0 | press > 140);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(0) | temp>60);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | sal>45);
    sal(bad)=NaN*ones(size(bad));
 warning('HEATCOND_SW:OutOfParameterRange','HEATCOND_SW: Input values out of range!')
    warning('HEATCOND_SW:OutOfParameterRange','Equations valid for p = 0-140 MPa, T=0-60 degC, S=0-45.')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press./100,temp, sal*1.005); %  pressure in kbar. absolute salinity

%------------------------------------------------------------------------
% Calculation of thermal conductivity
%------------------------------------------------------------------------

lambda = 0.001365.*...
    (1 + 0.003.*T - 1.025e-5.*T.^2 + 0.0653.*P - 0.00029.*S);% [cal/cm/s/C]

lambda = lambda .* 4.187 .*100; % [W/m/K]

end