function I = i_sw(sal)

% I_SW -  calculates the ionic strength of seawater I = 0.5*sum(m_i*z_i^2)
%
%     Usage: I = i_sw(S)
%
%     Input parameter (scalars or vectors):
%          S     - practical salinity
%
%     Output parameter: 
%          I     - ionic strength [mol/kg]
%
%     Comments: 
%     For contribution of total borate (mTBOh4) and dissolved inorganic
%     carbon (DIC) to the ionic strength, the composition at a pH of ~8
%     is assumed, where B(OH)4- is ~50% of the total borate,
%     HCO3- is ~90% of the DIC and CO32- is ~5% of the DIC.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       February 2013: input changed from (density + chlorinity) to salinity 
%           function call for seasalt changed, additional ions integrated
%       March 2015: typo corrected: 0.5*mDIC replaced by 0.05*mDIC
%------------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 1
     warning('I_SW:Input','I_SW: Missing input parameter! Please provide p,T and S!')
else
    
%------------------------------------------------------------------------
% Calculate ionic strength
%------------------------------------------------------------------------
%call external functions to calculate ion molalities in seawater
[mNa,mMg,mCa,mK,mSr,mLi,mCl,mBr,mF,mSO4,mTBOH4,mDIC,mOH] = seasalt(sal);

%calculation of ionic strength
I = 0.5 .*(mNa + mK + mLi + mCl + mBr + mF + 4.*mMg + 4.*mCa + ...
    0.5*mTBOH4 + 0.9*mDIC + mOH + 4.*mSr + 4.*mSO4 + 4*0.05*mDIC);
end