function [keq] = kco2_sw(press,temp,sal,species,mco2) 

% KCO2_SW -  calculates the equilibrium constants of the carbonic acid system 
%      at high CO2 concentrations
%
%      Usage: keq = kco2_sw(p,T,S,species,mco2)
%
%      Input parameters (p,T,S:scalars or vectors, species: string, 
%                        mco2: scalar or matrix of size p x T x S): 
%           p        - pressure [MPa]      (0.1-100) 
%           T        - temperature [degC]  (0-45) 
%           S        - practical salinity  (0-45)
%           mco2     - concentration of dissolved CO2 [mol/kg]
%           species  - choice of chemical substance for which
%                      the equilibrium constant is calculated:
%   1        'co2'      CO2 + H2O <--> H+ + HCO3- [mol/kg]
%   2        'hco3'     HCO3- <--> H+ + CO32- [mol/kg]
%   3        'h2o'      H20 <--> H+ + OH- [(mol/kg)^2]
%   4        'so4'      HSO4- <--> H+ + SO4-- [mol/kg]
%   5        'hf'       HF <--> H+ + F- [mol/kg]
%
%      Output parameter: 
%           keq      - equilibrium constant [mol/kg] or [mol/kg^2]
%
%       Default: Default value for mco2 is saturation concentration
%
%       References:
%       Wong et al. (2005), J. Chem. Eng. Data 50, 822-831.
%       Tishchenko, personal communication. (for kw)
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       September 2011: Format for mco2 specified in header
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 4
     warning('KCO2_SW:Input','KCO2_SW: Missing input parameter! Please provide p,T,S and species!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) && ~any(temp< 0 | temp>45) ...
        && ~any( sal<0 | sal>45)
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>45);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>45);
    sal(bad)=NaN*ones(size(bad));

    warning('KCO2_SW:OutOfParameterRange','KCO2_SW: Input values out of range!')
    warning('KCO2_SW:OutOfParameterRange','Equations valid for p=0.1-100 MPa, T=0-45 degC, S=0-45.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T,S] = ndgrid(press.*10,temp+273.15,sal*1.005); % pressure in bar, temperature in K, absolute salinity
  
if nargin == 4
   % if mco2 is not provided at function call, call external function to 
   % calculate molalities in saturated CO2/seawater solution 
    [mco2] = solu_co2(press,temp,sal); % dissolved CO2 [mol/kg] 
end
mh2o = 1000./18;% molality of water

%------------------------------------------------------------------------
% Calculation of equilibrium constants
%------------------------------------------------------------------------

switch species
    
    case 'co2'
    % call external function for calculation of equilibrium constants in
    % seawater
    [k1_0] = kequilib_sw(press,temp,sal,'co2');
    lnK1 = log(k1_0);
    
    % calculation of equilibrium constants corrected for the influence of
    % CO2
    lnk1 = lnK1 + mco2.*(-6.908 + 1289.964./T + 0.00695.*T - 1.0856e-4 .* P);
    keq = exp(lnk1);

    case 'hco3'
     % call external function for calculation of equilibrium constants in seawater  
    [k2_0] = kequilib_sw(press,temp,sal,'hco3');
    lnK2 = log(k2_0);
    
    % calculation of equilibrium constants corrected for the influence of
    % CO2
    lnk2 = lnK2 + mco2.*(6.2048 - 1138.82./T - 6.95e-3 .* T);
    keq = exp(lnk2);
    
    case 'h2o'
    % call external function for calculation of equilibrium constants in
    % seawater
    [kw_0] = kequilib_sw(press,temp,sal,'h2o');
    lnKw = log(kw_0);
    
    % calculation of equilibrium constants corrected for the influence of
    % CO2
    lnkw = lnKw + mco2.*(0.3852 - 119.338./T) - mco2./mh2o.*(1 + mco2.*(-0.2546 ...
    + 75.571./T - 5.428e-5 .* P) + S.*(2.2843e-2 - 2.2534e-4 .* T + ...
    4.518e-7 .* T.^2));
    keq = exp(lnkw);
    
    case 'so4'
    % call external function for calculation of equilibrium constants in
    % seawater
    [ks_0] = kequilib_sw(press,temp,sal,'so4');
    lnKs = log(ks_0);
    
    % calculation of equilibrium constants corrected for the influence of
    % CO2
    lnks = lnKs - 0.2.*mco2;
    keq = exp(lnks);
    
    case 'hf'
    % call external function for calculation of equilibrium constants in
    % seawater
    [kf_0] = kequilib_sw(press,temp,sal,'hf');
    lnKf = log(kf_0);

    % calculation of equilibrium constants corrected for the influence of CO2
    lnkf = lnKf + mco2.* (-0.124 + 31.804./T - 1.0856e-4 .* P);
    keq = exp(lnkf);
    
    otherwise
      warning('KCO2_SW:Input','KCO2_SW: Invalid species selected!')  
end

end


