function [Pm,Ps,Pv,Pgh] = phase_co2sw(temp,sal)

% PHASE_CO2SW -  calculates the phase boundaries of the binary system CO2-H2O 
%
%     Usage: [pm,ps,pv,pgh] = phase_co2sw(T,S)
%
%     Input parameters (scalars or vectors):
%         T      - temperature [degC] 
%         S      - practical salinity (0 < S < 40)
%
%     Output parameters:
%         Pm     - CO2 melting pressure [MPa]     
%         Ps     - CO2 sublimation pressure [MPa]
%         Pv     - CO2 vapor pressure [MPa]
%         Pgh    - CO2 hydrate pressure [MPa]
%
%      References:
%      Tishchenko et al. (2009) 8th International CO2 Conference, Jena.
%      Span & Wagner (1994) J.Phys.Chem.Ref.Data 25, 1509-1597.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       December 2009: Equation for dissociation pressure above quadrupel
%           point substituted: Now it is P(T), not T(P) (which had to be solved
%           numerically for P) (line 138-140)
%       September 2011: Function name changed: phase_co2h2o -> phase_co2sw
% ------------------------------------------------------------------


%     Example: 
%       Plotting the phasediagram of CO2-seawater in the 
%       temperature range of -10 to +40 degC
%       at a salinity of 35.
%
%       clf;clear;hold on;
%       t=[-10:.01:40];
%       s=35;
%       [pm,ps,pv,pgh]=phase_co2sw(t,s);
%       plot(t,pm,'k-','LineWidth',2);
%       plot(t,ps,'b-','LineWidth',2);
%       plot(t,pv,'r-','LineWidth',2);
%       set(gca,'YLim',[0 100]);
%       xlabel('Temperature / C','FontSize',12,'FontWeight','bold');
%       ylabel('Pressure / bar','FontSize',12,'FontWeight','bold');
%       title('Phase diagram of CO_2-H_2O','FontSize',14,'FontWeight','bold');
%       plot(30.9782,73.773,'r.','MarkerSize',20);
%       text(30,74,'critical point of CO_2 \rightarrow','HorizontalAlignment','right');
%       text(-80,30,'CO_2 hydrate','FontWeight','bold');
%       text(-30,50,'liquid CO_2 + H_2O','FontWeight','bold');
%       text(0,20,'CO_2 gas and liquid H_2O','FontWeight','bold');
%
%


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('PHASE_CO2SW:Input','PHASE_CO2SW: Missing input parameter! Please provide T and S!')
else


%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(sal<0 | sal>40)
else
    bad = find(sal<0 | sal > 40);
    sal(bad)=NaN*ones(size(bad));
    warning('PHASE_CO2SW:OutOfParameterRange','PHASE_CO2SW: Input values out of range!')
    warning('PHASE_CO2SW:OutOfParameterRange','Equations valid for S=0-40.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[T,S] = ndgrid(temp+273.15,sal); % temperature in [K]

% Triple point of CO2
Pt = 0.51795;   % +/- 0.0010 MPa
Tt = 216.592;   % +/- 0.003 K
%RHOlt = 1178.53;    % +/- 0.18 kg/m^3 saturated liquid  
%RHOgt = 13.7614;    % +/- 0.0034 kg/m^3 saturated vapor   
%
% Critical point of CO2
Pc = 7.3773;    % +/- 0.0030 MPa
Tc = 304.1282;  % +/- 0.0150 K
%RHOc = 467.6;   % +/- 0.6 kg/m^3
%
redTt = T./Tt;  % reduced T
redTc = T./Tc;  % reduced T
%

%------------------------------------------------------------------------
% Calculation of output parameters
%------------------------------------------------------------------------

% Melting pressure (for T = Tt - >270 K)
Pm = Pt .* (1 + 1955.5390*(redTt-1) + 2055.4593*(redTt-1).^2);
bad=find(T<Tt) ; Pm(bad)=NaN*ones(size(bad));

%
% Sublimation pressure (for T = <170 K - Tt)
lnPsPt = redTt .* (-14.740846.*(1-redTt) ...
    + 2.4327015.*(1-redTt).^1.9 - 5.3061778.*(1-redTt).^2.9);
Ps = Pt .* exp(lnPsPt);
bad=find(T>Tt) ; Ps(bad)=NaN*ones(size(bad));

%
% Vapor pressure (for T = Tt - Tc)
lnPvPc = 1./redTc .* (-7.0602087.*(1-redTc) + 1.9391218.*(1-redTc).^1.5 ...
    - 1.6463597.*(1-redTc).^2 - 3.2995634.*(1-redTc).^4);
Pv = Pc .* exp(lnPvPc);
bad=find(T<Tt | T>Tc) ; Pv(bad)=NaN*ones(size(bad));

% equation for the quadruple point
% Temperature at quadruple point
TQ = 283.15 - 4.976e-2 .*S + 5.96e-5 .*S.^2;
% pressure at quadruple point
PQ = 4.4974 - 5.463e-3 .* S + 8e-6 .*S.^2;

%
% Dissociation pressures of CO2 hydrate in saline water (for S = 0-40
% g/kg) below the quadruple point
lnPdissw = -4.995297252D3 - 1.0564255D2.*S + 1.6436576.*S.^2 ...
    + T .* (5.41052D1 + 1.148516.*S - 1.775663D-2.*S.^2) ...
    + T.^2 .* (-1.957554D-1 - 4.162262D-3.*S + 6.393503D-5.*S.^2) ...
    + T.^3 .* (2.366122D-4 + 5.028447D-6.*S - 7.672643D-8.*S.^2) ;
Pgh = exp(lnPdissw); % [MPa]

% Dissociation pressures of CO2 hydrate in saline water (for S = 0-40
% g/kg) above the quadruple point

Pgh2 = PQ + (7.9895 + 7.645e-3.*S - 5.117e-4.*S.^2 + 9.901e-6.*S.^3)...
    .*(T - TQ) + (-0.20936 + 5.7675e-3.*S - 4.915e-5 .* S.^2).*(T - TQ).^2 ...
     + 0.1061.*(T - TQ).^3;

decide=Pgh > PQ;
Pgh(decide) = Pgh2(decide);

end
