function [P_sat, P_VL, Tcrit, Pcrit, T_VLH, P_VLH, x_vap, PmNaCl, PsNaCl, PbNaCl] = phase_sw(temp,sal)

% PHASE_SW -  calculates the phase boundaries of the system H2O-NaCl based
%     on multiparametric equations (also valid for hydrothermal systems)
%
%     Usage: 
%         [P_sat P_VL Tcrit Pcrit T_VLH P_VLH x_vap PmNaCl PsNaCl PbNaCl]
%               = phase_sw(T,S)
%
%     Input parameters (T is scalar or vector; S is scalar): 
%         T         - temperature [degC]     (0 < T < 1000)
%         S         - practical salinity     (0 < S < 1000)
%
%     Output parameter:
%         P_sat      - saturation pressure [MPa] at given temperature + 
%                       NaCl concentration (liquid-halite coexistence line) 
%         P_VL       - liquid-vapor coexistence pressure [MPa] at given
%                       temperature and NaCl concentration  
%         Tcrit      - critical temperature [degC] at given NaCl concentration
%         Pcrit      - critical pressure [MPa] at given NaCl concentration
%         T_VLH      - temperature [degC] at liquid-vapor-halite
%                       coexistence for given NaCl concentration
%         P_VLH      - pressure [MPa] at liquid-vapor-halite coexistence
%                       for given NaCl concentration
%         xvap       - mole fraction of NaCl in the vapor phase at
%                       liquid-vapor coexistence
%         PmNaCl     - melting pressure [MPa] of NaCl at given temperatures
%         PsNaCl     - sublimation pressure [MPa] of NaCl at given
%                       temperatures
%         PbNaCl     - boiling pressure [MPa] of NaCl at given temperatures
%
%     References:
%     Driesner & Heinrich (2007) GCA 71, 4880-4901.
%     IAPWS (1992) Revised supplementary release on saturation properties
%        of ordinary water substance. http://iapws.org
%
%     Comments: 
%     The vapor-liquid coexistence line is calculated only for temperatures
%     below the triple point temperature of NaCl (800.7 degC).
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2011)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
%       January 2013: name in internal function call changed: solu_NaCl ->
%       solu_nacl; VLH_NaClH2O -> vlh_naclh2o
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('PHASE_SW:Input','PHASE_SW: Missing input parameter! Please provide T and S!')
      P_sat = NaN;
    P_VL = NaN;
    Tcrit = NaN;
    Pcrit = NaN;
    T_VLH = NaN;
    P_VLH = NaN;
    x_vap = NaN;
    PmNaCl = NaN;
    PsNaCl = NaN;
    PbNaCl = NaN;
elseif length(sal) > 1
     warning('PHASE_SW:Input','PHASE_SW: Only scalar input for Salinity!')
    P_sat = NaN;
    P_VL = NaN;
    Tcrit = NaN;
    Pcrit = NaN;
    T_VLH = NaN;
    P_VLH = NaN;
    x_vap = NaN;
    PmNaCl = NaN;
    PsNaCl = NaN;
    PbNaCl = NaN;

else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if  ~any(temp<(0) | temp>1000) && ~any(sal<(0) | sal>1000)

else
    bad = find(temp<(0) | temp>1000);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | sal>1000);
    sal(bad)=NaN*ones(size(bad));

 warning('PHASE_SW:OutOfParameterRange','PHASE_SW: Input values out of range!')
 warning('PHASE_SW:OutOfParameterRange','Equations valid for  T=0-1000 degC, sal=0-1000.')
end


%------------------------------------------------------------------------
% Transforming salinity to mole fraction of NaCl
%------------------------------------------------------------------------

mmH2O = 18.015;  % molar mass of H2O

% call external function to calculate ionic strength of seawater with given
% salinity
isw = i_sw(sal);

% moles of NaCl in 1 kg solution, that have the same ionic strength as
% seawater with given salinity 
nNaCl = isw;  

nH2O = (1000 - sal*1.005)/mmH2O; % moles of H2O in 1 kg solution
xNaCl = nNaCl/(nNaCl + nH2O); % mole fraction of NaCl

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------
 

TtNaCl = 800.7; % triple point temperature of NaCl [C]
PtNaCl = 5e-4; % triple point pressure of NaCl [bar]
PcH2O  = 2.2054915e2; % critical pressure of pure H2O [bar]
TcH2O  = 373.976;  % critical temperature of pure H2O [C]


%------------------------------------------------------------------------
% Calculation of melting pressure for pure NaCl
%------------------------------------------------------------------------
Tm = temp; % temperature vector for melting curve

% melting curve does exist only at temperatures above the triple point -> Set
% lower temperature to NaN.
Tm(Tm <= TtNaCl)=NaN;

PmNaCl = (Tm - TtNaCl)./2.4726e-2 + PtNaCl; % [bar]

%------------------------------------------------------------------------
% Calculation of sublimation pressure for pure NaCl
%------------------------------------------------------------------------
Ts = temp; % temperature vector for sublimation curve

% sublimation curve does exist only at temperatures below the triple point -> Set
% higher temperature to NaN.
Ts(Ts >= TtNaCl)=NaN;

logPsNaCl = log10(PtNaCl) + 1.18061e4.*(1./(TtNaCl+273.15) - 1./(Ts + 273.15));

PsNaCl = 10.^logPsNaCl; % [bar]

%------------------------------------------------------------------------
% Calculation of boiling pressure for pure NaCl
%------------------------------------------------------------------------
Tb = temp; % temperature vector for sublimation curve

% boiling curve does exist only at temperatures above the triple point -> Set
% lower temperature to NaN.
Tb(Tb <= TtNaCl)=NaN;

logPbNaCl = log10(PtNaCl) + 0.941812e4.*(1./(TtNaCl+273.15) - 1./(Tb + 273.15));

PbNaCl = 10.^logPbNaCl; % [bar]

%------------------------------------------------------------------------
% Calculation of critical pressure and temperature of NaCl solution for
% given NaCl fraction 
%------------------------------------------------------------------------

% The critical point for mole fractions of NaCl above 0.1345 is above 1000C
% and is not covered by the parameter range of the equations.

if xNaCl > 0.1345
      warning('PHASE_SW:OutOfParameterRange','Critical temperature for given salinity is > 1000 C!')
      warning('PHASE_SW:OutOfParameterRange','Critical point is not calculated!')
      Tcrit = NaN;
      Pcrit = NaN;
else
    % call subroutine to calculate critical point for given xNaCl
    [Tcrit, Pcrit] = critpoint(xNaCl,TcH2O,PcH2O); 
end


%------------------------------------------------------------------------
% Calculation of the pressures, at which saturation equals given
% mole fraction at given temperatures; this gives also the
% vapor-liquid-halite coexistence line
%------------------------------------------------------------------------
P_test = (0:10:500); % pressure vector  in MPa for calculation of a start value for P_sat
P_sat = zeros(length(temp),1); % empty matrix for saturation pressure
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

for i = 1:length(temp)
     mx0 = solu_nacl(0,temp(i));  % saturation concentration at p=0
     x0 = m2x(mx0);
     mx500 = solu_nacl(500,temp(i)); % saturation concentration at 5000 bar and given temperature
     x500 = m2x(mx500);
     % if saturation concentration at zero pressure is larger than at
    % maximum concentration, swap values to ensure correctness of the
    % following if conditions
     if x0 > x500 
        x02 = x500;
        x5002 = x0;
     else 
         x02 = x0;
        x5002 = x500;
    end
    if xNaCl < x02  % saturation concentration at p=0
        P_sat(i) = NaN; % set saturation pressure to NaN
        warning('PHASE_SW:OutOfParameterRange','NaCl concentration is too low for a saturated solution!')
        warning('PHASE_SW:OutOfParameterRange','Saturation pressure is not calculated!')
    else
        if xNaCl >= x5002 % above this value, the saturation pressure is ambiguous
                P_sat(i) = NaN; % set saturation pressure to NaN
                warning('PHASE_SW:OutOfParameterRange','Saturation pressure is ambiguous at the given NaCl mole fraction!')
                warning('PHASE_SW:OutOfParameterRange','Saturation pressure is not calculated!')
        else
                % saturation mole fraction at test pressures
                mx_test = solu_nacl(P_test,temp(i)); 
                x_test = m2x(mx_test);
                 % find test mole fraction close to given mole fraction
                f = find(x_test < xNaCl);   
                if isempty(f) % if saturation is not reached at given temperature and allowed pressure range
                    P_sat(i) = NaN; % set saturation pressure value to NaN
                    warning('PHASE_SW:OutOfParameterRange','Pressure at which liquid saturates is above > 500 MPa!')
                    warning('PHASE_SW:OutOfParameterRange','Saturation pressure is not calculated!')
                else  
                    % check for NaN before calling fsolve
                    if isnan(sal) || isnan(temp(i))
                        P_sat(i) = NaN;
                    else
                        % set initial guess for pressure at saturation to pressure that
                        % corresponds to mole fraction cloe to given mole fraction
                        mNaCl = x2m(xNaCl);
                        Ps_init = P_test(f(1));      
                         P_sat(i) = fsolve(@(P_s) minfunc_Psat(P_s,temp(i),mNaCl),Ps_init,options); % call solver
                        % P_sat is in MPa
                    end
                end 
        end
    end
end

P_sat = P_sat.*10; % [bar]


%------------------------------------------------------------------------
% Calculation of the halite - halite saturated vapor - halite saturated
% liquid coexistence point pressure P_VLH at given temperatures and
% salinity
%------------------------------------------------------------------------

T_test = (0:10:800); % temperature vector in C for calculation of a start value for P_sat

options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction



% saturation mole fraction at test temperatures
[~,mx_test] = vlh_naclh2o(T_test); 
x_test = m2x(mx_test);
% find test mole fraction close to given mole fraction
f = find(x_test <= xNaCl);   
if isempty(f) % if saturation is not reached at given temperature and allowed pressure range
    T_VLH = NaN; % set pressure at saturation value to NaN
    P_VLH = NaN;
    warning('PHASE_SW:OutOfParameterRange','No liquid vapor halite coexistence for given salinities!')
    warning('PHASE_SW:OutOfParameterRange','VLH coexistence is not calculated!')
else  
       % check for NaN before calling fsolve
       if isnan(sal)
          P_VLH = NaN;
       else
        % set initial guess for temperature at saturation to temperature that
        % corresponds to mole fraction cloe to given mole fraction
        T_init = T_test(f(end));      
        T_VLH = fsolve(@(T_vlh) minfunc_T_VLH(T_vlh,xNaCl),T_init,options); % call solver
        P_VLH = vlh_naclh2o(T_VLH); % call external function to calculate corresponding pressure in MPa
       end
end

P_VLH = P_VLH.*10; % [bar]


%------------------------------------------------------------------------
% Calculation of the vapor liquid coexistence line
%------------------------------------------------------------------------

P_VL = zeros(length(temp),1); % empty matrix for saturation pressure
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

if xNaCl == 0 % for zero salinity, take pure water phase boundary
    P_VL = IAPWS(temp); 
    P_VL (temp > TcH2O) = NaN;
else

for i = 1:length(temp) % loop over temperature values

   if temp(i) >= TtNaCl
        P_VL(i) = NaN;
        warning('PHASE_SW:OutOfParameterRange','Equations for vapor liquid coexistence line not implemented for T >800.7C!')
        warning('PHASE_SW:OutOfParameterRange','Coexistence pressure is not calculated!')
    elseif temp(i) < T_VLH % no liquid vapor coexistence below vapor liquid halide coexistence point
         P_VL(i) = NaN;
   else
       % pressure vector  in bar for calculation of a start value for
        % P_sat
        if temp(i) > TcH2O
            [P_test1] =critpress(temp(i),TcH2O,PcH2O); 
        else
            P_test1 = IAPWS(temp(i)); 
        end
        P_test = (0:P_test1/1000:P_test1); 
    	% saturation mole fraction at test pressures
        x_test = VLl_line(P_test,temp(i),TcH2O,PcH2O,TtNaCl); 
        % find test mole fraction close to given mole fraction
        f = find(x_test <= xNaCl);   
        if isempty(f) % no coexistence lilne for given xNaCl and temepratures
            P_VL(i) = NaN; % set pressure at saturation value to NaN
            warning('PHASE_SW:OutOfParameterRange','No liquid vapor coexistence for given T and xNaCl!')
            warning('PHASE_SW:OutOfParameterRange','Coexistence pressure is not calculated!')
        else  
             % check for NaN before calling fsolve
             if isnan(sal) || isnan(temp(i))
                P_VL(i) = NaN;
             else
                % set initial guess for pressure at saturation to pressure that
                % corresponds to mole fraction close to given mole fraction
                Ps_init = P_test(f(1));      
                P_VL(i) = fsolve(@(P_s) minfunc_P_VL(P_s,temp(i),xNaCl,TcH2O,PcH2O,TtNaCl),Ps_init,options); % call solver
                 % P_VL is in bar
             end
        end
    end
end 
end

%------------------------------------------------------------------------
% Calculation of the vapor NaCl content at the vapor liquid coexistence line
%------------------------------------------------------------------------

x_vap = zeros(length(temp),1); % empty matrix for vapor mole fraction of NaCl

for i = 1:length(temp)
    if isnan(P_VL(i))
        x_vap(i) = NaN;
    else 
        xref = VLl_line(P_VL(i),temp(i),TcH2O,PcH2O,TtNaCl); % NaCl concentration of the liquid phase 
        x_vap(i) = xvapor(P_VL(i),temp(i),xref,TtNaCl,PtNaCl,TcH2O,PcH2O);
    end
end


%------------------------------------------------------------------------
% Calculation of vapor NaCl content at the halite vapor coexistence line
%------------------------------------------------------------------------

% This part is commented out but is tested and can be invoked for further use:
% The composition of the vapor phase at the vapor-halite coexistence line
% is independent of the liquid composition (no liquid present). With only
% one given variable left (the temperature), the system is not well enough
% defined to perform a meaningful calculation: Either the NaCl mole
% fraction in the vapor or the pressure (in bar) have to be defined. In the
% first case, a solver needs to be implemented.

% x_vaphal = zeros(length(temp),1); % empty matrix for vapor mole fraction of NaCl
% 
% for i = 1:length(temp)
%     % no solutions for pressure below liquid vapor halite coexistence
%     % surface
%     if press(i) < (vlh_naclh2o(temp(i))/10) 
%         x_vaphal(i) = NaN;
%     else 
%         mxref = solu_nacl(press(i)/10,temp(i)); % NaCl concentration of the liquid phase 
%         xref = m2x(mxref)
%         x_vaphal(i) = xvapor(P_VL(i),temp(i),xref,TtNaCl,PtNaCl,TcH2O,PcH2O);
%     end
% end
% 

% return pressures in MPa
P_sat = P_sat./10;
Pcrit = Pcrit./10;
P_VLH = P_VLH./10;
P_VL = P_VL./10;
PmNaCl = PmNaCl./10;
PsNaCl = PsNaCl./10;
PbNaCl = PbNaCl./10;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Tcrit, Pcrit] = critpoint(x,TcH2O,PcH2O) 
% this function calculates the critical point of the NaCl solution as a
% function of the NaCl mole fraction

%------------------------------------------------------------------------
% solve equation for critical temperature:
%------------------------------------------------------------------------

% use simple linear approach to guess starting value Tc_init for solver
Tc_init = TcH2O + 3760*x;

options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

% check for NaN before calling fsolve
if isnan(x) 
   Tcrit = NaN;
else
    Tcrit = fsolve(@(Tc) minfunc_Tc(Tc,TcH2O,x),Tc_init,options); % call solver
end

%------------------------------------------------------------------------
% calculate critical pressure for calculated critical temperature
%------------------------------------------------------------------------

% critical pressure Pcrit [bar]: there are three different equation for
% three temperature regimes

if Tcrit < TcH2O % extrapolation of the curve below the pure water critical temperature
   
    Pcrit = NaN;
    Tcrit = NaN;
    warning('PHASE_SW:OutOfParameterRange','Given temperatures are too low for critical behavior!')
    warning('PHASE_SW:OutOfParameterRange','Critical point is not calculated!')
    
else % for temperatures above 500 C
    
Pcrit = critpress(Tcrit,TcH2O,PcH2O);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Pcrit] =critpress(Temp,TcH2O,PcH2O) 
% calculates the critical pressure as function of temperature. Thi is done
% also for the region below the pur water critical temperature where it has
% no physical meaning

%------------------------------------------------------------------------
% definition of parameters for the critical curve
%------------------------------------------------------------------------

cA = [1 1.5 2 2.5 3 4 5 1 2 2.5 3];

c(1) = -2.36; 
c(2) =  1.28534e-1; 
c(3) = -2.3707e-2;
c(4) =  3.20089e-3;
c(5) = -1.38917e-4;
c(6) =  1.02789e-7;
c(7) = -4.8376e-11;
c(8) =  2.36;
c(9) = -1.31417e-2;
c(10)= 2.98491e-3 ;
c(11)=-1.30114e-4 ;
c(12)= 581.0101; 
c(13)= 3.4055;
c(14)=-4.88336e-4;

Pcrit=zeros(length(Temp),1); % empty matrix


%------------------------------------------------------------------------
% calculate critical pressure for given temperature
%------------------------------------------------------------------------

% critical pressure Pcrit [bar]: there are three different equation for
% three temperature regimes

for i=1:length(Temp)
    T = Temp(i);

    if T < TcH2O % extrapolation of the curve below the pure water critical temperature
   
        Pcrit(i) = PcH2O + c(1).*(TcH2O - T).^cA(1) + c(2).*(TcH2O - T).^cA(2) + ...
        c(3).*(TcH2O - T).^cA(3) +  c(4).*(TcH2O - T).^cA(4) + ...
        c(5).*(TcH2O - T).^cA(5) + c(6).*(TcH2O - T).^cA(6) + ...
        c(7).*(TcH2O - T).^cA(7);
 
    
    elseif T < 500 % critical pressure for temperatures TcH2O < T < 500 C

        Pcrit(i) = PcH2O + c(8).*(T - TcH2O).^cA(8) + c(9).*(T - TcH2O).^cA(9) + ...
        c(10).*(T - TcH2O).^cA(10) +  c(11).*(T - TcH2O).^cA(11); 

    else % for tempeatures above 500 C
    
    Pcrit(i) = c(12) + c(13).*(T - 500) + c(14).*(T - 500).^2;

    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [x] =xNaClcrit(temp,TcH2O) 

% function for critical NaCl mole fraction as function of critical
% temperature

d(1) = 8.00000e-5;
d(2) = 1.00000e-5;
d(3) = -1.37125e-7;
d(4) = 9.46822e-10;
d(5) = -3.50549e-12;
d(6) = 6.57369e-15;
d(7) = -4.89423e-18;
d(8) = 7.77761e-2;
d(9) =2.7042e-4;
d(10)= -4.244821e-7;
d(11)= 2.580872e-10;


x = zeros(length(temp),1);
% different equations apply for Tc > 600C and Tc > 600C
for i = 1:length(temp)
    Tc = temp(i);
    if Tc <= 600
        x(i) = d(1).*(Tc-TcH2O) + d(2).*(Tc-TcH2O).^2 + d(3).*(Tc-TcH2O).^3 + ...
            d(4).*(Tc-TcH2O).^4 + d(5).*(Tc-TcH2O).^5 + d(6).*(Tc-TcH2O).^6 + ...
            d(7).*(Tc-TcH2O).^7;
    else
        x(i) = d(8) + d(9).*(Tc-600) + d(10).*(Tc-600).^2 + ...
            d(11).*(Tc-600).^3;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [x_VL ] = VLl_line(P,T,TcH2O,PcH2O,TtNaCl) 
% this function calculates the liquid branch of the vapor liquid
% coexistence line for given prssures and temperatures

%------------------------------------------------------------------------
% definition of parameters for the critical curve
%------------------------------------------------------------------------

% call subroutine to calculate vapor liquid curve for pure water
pH2O = IAPWS(T); 
% call subroutine to calculate critical pressure for given temperature
Pcrit = critpress(T,TcH2O,PcH2O); 
% call subroutine to calculate critical NaCl mole fraction  for given
% temperature
Xcrit = xNaClcrit(T,TcH2O); 
% call external function to get NaCl mole fraction at VHL intersection with
% VLH surface
[P_VLH,mx_VLH] = vlh_naclh2o(T);
x_VLH = m2x(mx_VLH);
P_VLH = P_VLH.*10; % [bar]

h(1)  = 1.68486e-3;
h(2)  = 2.19379e-4;
h(3)  = 4.3858e2;
h(4)  = 1.84508e1;
h(5)  =-5.6765e-10;
h(6)  = 6.73704e-6;
h(7)  = 1.44951e-7;
h(8)  = 3.84904e2;
h(9)  = 7.07477;
h(10) = 6.06896e-5;
h(11) = 7.62859e-3;

g(1) = h(2) + (h(1)-h(2))./(1 + exp((T-h(3))./h(4))) + h(5).*T.^2;

g(2) = h(7) + (h(6)-h(7))./(1 + exp((T-h(8))./h(9))) + h(10).*exp(-h(11).*T);

% coexistence line only for pressures above the three phase boundary
% pressure and below the critical pressure
P( P > Pcrit) = NaN; %
P ( P < P_VLH) = NaN;

%------------------------------------------------------------------------
% calculate NaCl mole fraction at liquid vapor coexistence line
%------------------------------------------------------------------------

if T < TtNaCl % range of validity for this particular equation
    
% the parameter g0 has to be determined from the known NaCl mole fraction
% at the three phase point. For temperatures below the critical temperature
% of oure water, a fictional critical composition has to be calculated from
% fulfiiling the condition of x=0 at the pure water phase boundary

for i = 1:length(T)
    if T(i) <= TcH2O
        P(P > pH2O) = NaN;
        var1 = sqrt(Pcrit - pH2O)./sqrt(Pcrit-P_VLH);
        Xcrit(i) = 1./(1 - var1).*(var1.*(g(1).*(Pcrit - P_VLH) + ...
            g(2).*(Pcrit - P_VLH).^2 - x_VLH) - ...
            g(1) .* (Pcrit - pH2O) - g(2).*(Pcrit - pH2O).^2);
    end
end

g0 = (x_VLH - Xcrit - g(1).*(Pcrit -P_VLH) - g(2).*(Pcrit - P_VLH).^2)./sqrt(Pcrit - P_VLH);

for i = 1:length(T)
    if T(i) <= TcH2O
        Xcrit(i) = -( g0.*sqrt(Pcrit(i) - pH2O(i)) + g(1).*(Pcrit(i) -pH2O(i)) + g(2).*(Pcrit(i) - pH2O(i)).^2);
    end
end

% calculate NaCl mole fraction at phase vapor liquid phase boundary
x_VL = Xcrit + g0.*sqrt(Pcrit - P) + g(1).*(Pcrit -P) + g(2).*(Pcrit - P).^2;
else
    x_VL(T>=TtNaCl) = NaN;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function x = xvapor(p,T,xref,TtNaCl,PtNaCl,TcH2O,PcH2O) 
% calculates the mole fraction of NaCl in the vapor phase at the vapor
% liquid phase boundary

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------
 
% empty matrices for liquid phase NaCl mole fractions at the saturation
% curve and at the vapor liquid phase boundary 
xlNaCls = zeros(length(T),1);

% call subroutine to calculate critical pressure at given temperatures
Pcrit = critpress(T,TcH2O,PcH2O);

k0 = -0.235694;
k1 = -0.188838;
k2 = 0.004;
k3 = 0.0552466;
k4 = 0.66918;
k5 = 396.848;
k6 = 45.0;
k7 = -3.2719e-7;
k8 = 141.699;
k9 = -0.292631;
k10= -0.00139991;
k11=  1.95965e-6;
k12= -7.3653e-10;
k13=  0.904411;
k14=  0.000769766;
k15= -1.18658e-6;

j0 = k0 + k1.*exp(-k2*T);
j1 = k4 + (k3 - k4)./(1 + exp((T - k5)./k6)) + k7.*(T - k8).^2;
j2 = k9 + k10.*T + k11.*T.^2 + k12.*T.^3;
j3 = k13 + k14.*T + k15.*T.^2;


% Calculation of sublimation pressure for pure NaCl at given temperaures
logPsNaCl = log10(PtNaCl) + 1.18061e4.*(1./(TtNaCl+273.15) - 1./(T + 273.15));

PsNaCl = 10.^logPsNaCl; % [bar]


% Calculation of boiling pressure for pure NaCl at given temperatures
logPbNaCl = log10(PtNaCl) + 0.941812e4.*(1./(TtNaCl+273.15) - 1./(T + 273.15));

PbNaCl = 10.^logPbNaCl; % [bar]

%if pressure is above triple point pressure for halite, use boiling curve
%pressure instead of sublimiation curve pressure
pNaCl = PsNaCl;
pNaCl(T > TtNaCl) = PbNaCl(T > TtNaCl);

% calculation of liquid phase NaCl mole fractions at the saturation
% curve for each pair of T and p
for i = 1:length(T)
    mxlNaCls = solu_nacl(pNaCl(i)./10,T(i));
    xlNaCls(i) = m2x(mxlNaCls);
end

%normalized pressure pn
pn = (p - pNaCl)./(Pcrit - pNaCl);

%------------------------------------------------------------------------
% Calculation of NaCl mole fraction in the vapor phase x
%------------------------------------------------------------------------

logkb = 1 + j0.*(1 - pn).^j1 + j2.*(1 - pn) + j3.*(1 - pn).^2 ...
       - (1 + j0 + j2 + j3).*(1 - pn).^3;

logks = logkb .*(log10(pNaCl./Pcrit) - log10(xlNaCls)) + log10(xlNaCls);
   
logxlxv = logks - log10(pNaCl./p);

xlxv = 10.^logxlxv;


x = xref./xlxv;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function pH2O = IAPWS(temp) 

% calculates the vapor pH2O pressure of pure water from the IAPWS92
% formulation. 

temp = temp + 273.15; % temperature in K

Tc = 647.096; % critical temperature of pure H2O in K
Pc = 220.64;  % critical pressure of pure H2O in bar

theta = temp./Tc;
tau = 1-theta;

a1 =  -7.85951783;
a2 =   1.84408259;
a3 = -11.7866497;
a4 =  22.6807411;
a5 = -15.9618719;
a6 =   1.80122502;

lnppc = Tc./temp.*(a1.*tau + a2.*tau.^1.5 + a3.*tau.^3 + a4.*tau.^3.5 + ...
    a5.*tau.^4 + a6.*tau.^7.5);

pH2O = exp(lnppc).*Pc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = minfunc_Tc(Tc,TcH2O,x) 

% function for critical NaCl mole fraction as function of critical
% temperature, has to be solved to provide critical temperature a function
% of critical NaCl mole fraction

% call subroutine to calculate critical mole fraction for given
% temperature
xcrit = xNaClcrit(Tc,TcH2O);

f = xcrit - x;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = minfunc_Psat(P_s,T_s,m) 
% function that has to be solved for getting the pressure at which a liquid
% is saturated at given xNaCl and temperature

f = solu_nacl(P_s,T_s) - m;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = minfunc_T_VLH(T_vlh,x) 
% function that has to be solved for getting the temperature of vapor
% liquid halite coexistence for given xNaCl and temperature 

[~, msat] = vlh_naclh2o(T_vlh);
xsat = m2x(msat);
f = xsat - x;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = minfunc_P_VL(P_vl,T_vl,x,TcH2O,PcH2O,TtNaCl) 
% function that has to be solved for getting the temperature of vapor
% liquid halite coexistence for given xNaCl and temperature 
xVL = VLl_line(P_vl,T_vl,TcH2O,PcH2O,TtNaCl);
if imag(xVL)
    xVL = 1.1;
end
f = xVL - x;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function x = m2x(m)
% calculates mole fraction from molality

mmH2O = 18.015;  % molar mass of H2O
x = m.*mmH2O./(1000 + m.*mmH2O);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function m = x2m(x)
% calculates  molality from mole fraction

mmH2O = 18.015;  % molar mass of H2O
m = 1000.*x./(1 - x)./mmH2O;