function pvch4 = pp2pv_ch4(press,temp,pch4)

% PP2PV_CH4 - converts the partial pressure of CH4 in a gas mixture into
%     partial volume
%
%     Usage: pv = pp2pv_ch4(p,T,pp)
%
%     Input parameter (p,T: scalars or vectors; pp: scalar or vector of same size as p): 
%           p      - overall pressure [MPa]        (0.012 < p < 1000)
%           T      - temperature [degC]            (-183 < T < 347)
%           pp     - partial pressure of CH4 [MPa] (0 < pp < p)
%           
%     Output parameter: 
%           pv     - partial volume of CH4 [ppmV]
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (May 2012)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       December 2012: handling for size(pch4) ~= size(press) implemented
%       June 2013: checks for input and parameter range implemented
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('PP2PV_CH4:Input','PP2PV_CH4: Missing input parameter! Please provide p, T and pp!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.012 | press>1000) && ~any(temp< -183 | temp>347)
else
    bad = find(press<0.012 | press > 1000);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-183 | temp>347);
    temp(bad)=NaN*ones(size(bad));
    warning('PP2PV_CH4:OutOfParameterRange','PP2PV_CH4: Input values out of range!')
    warning('PP2PV_CH4:OutOfParameterRange','Equations valid for p = 0-1000 MPa, T=(-183) - 347 degC.')
end


rho = eos_ch4(press,temp); % density of CH4 at total pressure

if size(press) == size(pch4)
    rhop = eos_ch4(pch4,temp); % density of CH4 at partial pressure
else
    rhop2 = eos_ch4(pch4,temp); % density of CH4 at partial pressure
    rhop = zeros(length(press),length(temp));
    for i=1:length(press)
        rhop(i,:) = rhop2(:,:);
    end
end

pvch4 = rhop./rho; % volume fraction of CH4 in gas phase
pvch4 = pvch4.*1d6; % partial volume in ppmV

end