function [mNa,mMg,mCa,mK,mSr,mLi,mCl,mBr,mF,mSO4,mTBOH4,mDIC,mOH] = seasalt(S)

% SEASALT -  calculates the major salt concentrations in seawater
%      based on its salinity
%
%      Usage: [mNa,mMg,mCa,mK,mSr,mLi,mCl,mBr,mF,mSO4,mTBOH4,mDIC,mOH] 
%       = seasalt(S)
%
%      Input parameters (scalar, vector or matrix):
%           S      - practical salinity of seawater 
%                       (absolute salinity = S*1.005 [g/kg])
%
%       Output parameters: 
%           mI     - molality of salt component I [mol/kg]
%
%      References: 
%      Millero (2008) Deep-Sea Research I 55, 5072.
%      Stoffyn-Egly & MacKenzie (1984) GCA 48, 859-872.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       September 2011: name changed in function call: information for size
%           of input parameters added to header
%       February 2013: Reference composition changed from Stumm and Morgan
%           to Millero 2008 (additional output of mHCO3, mOH and mBOH3), 
%           input parameters changed from density and chlorinity to salinity
%           output is now really in molalities not in mol/(kg seawater) as
%           in the former version
%------------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 1
     warning('SEASALT:Input','SEASALT: Missing input parameter! Please provide S!')
else

%------------------------------------------------------------------------
% Calculate molarities for salt ions
%------------------------------------------------------------------------

REFS = 35; % reference salinity
Cl = S/1.80655; % Chlorinity 


% molalities [mol/kg]
mNa = S/REFS*0.4860597;
mMg = S/REFS*0.0547421;
mCa = S/REFS*0.0106568;
mK = S/REFS*0.0105797;
mSr = S/REFS*0.0000940;
mBr = S/REFS*0.0008728;
mF = S/REFS*0.0000708;
mSO4 = S/REFS*0.0292643;
mCl = S/REFS*0.5657647;
mBOH4 = S/REFS*0.0001045;
mHCO3 = S/REFS*0.0017803;
mOH = S/REFS*0.0000082;
mBOH3 = S/REFS*0.0003258;
mCO3 = S/REFS*0.0002477;
mCO2 = S/REFS*0.0000100;
mLi = Cl*9.36d-6/6.94/54.6762838*56.6690534;
mTBOH4 = mBOH4 + mBOH3;
mDIC = mHCO3 + mCO3 + mCO2;

 % mLi = Cl*9.36d-6 g/kg_seawater @ S = 35 -> converted to mol/kg_water 

end
