function mCH4sw = solu_ch4(press,temp,sal)

% SOLU_CH4 -  calculates the equilibrium concentration of CH4 in seawater 
%     with a gas phase or hydrate phase. 
%     The phase at given parameters p,T,S is determined and the appropriate
%     function for the calculation of the solubility is called.
%
%     Usage:  m = solu_ch4(p,T,S)
%
%     Input parameters (scalars or vectors): 
%           p        -  pressure [MPa], 
%           T        -  temperature [degC]
%           S        -  practical salinity 
%     
%     Output parameter: 
%           m        - molality of CH4 in seawater [mol/kg]
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history:
%       September 2011: name changed in function call: 
%                       phase_ch4h2o -> phase_ch4sw
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_CH4:Input','SOLU_CH4: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Determine the phase of CH4 and use the corresponding equations for gas
%   phase or hydrate phase.
%------------------------------------------------------------------------

% call external function to calculate the phase boundaries
[~,pdissw] = phase_ch4sw(temp,sal);  

% resize matrices to account for pressure vector
Pdissw= zeros(length(press),length(temp),length(sal));

% create matrices for phase boundaries
for i=1:size(press,2)
Pdissw(i,:,:) = pdissw(:,:); 
end

% calculate dissolved CH4 in hydrate phase
[mCH4sw] = solu_ch4gh(press,temp,sal);

 % dissolved CH4 in gas phase
mCH4swg = solu_ch4gas(press,temp,sal);

% Determine, if PTS-values are within the hydrate stability field. Decide,
% which equations to use for calculation of solubility.

[P] = ndgrid(press,temp,sal);

    decide = P < Pdissw;
    mCH4sw(decide) = mCH4swg(decide);

end

