function [mCH4sw, xsw] = solu_ch4gas(press,temp,sal)

% SOLU_CH4GAS -  calculates the equilibrium concentration of CH4 in seawater
%     with respect to the gas phase as well as the resulting mole fraction
%     of water in the CH4 gas phase
%
%     Usage:   [mCH4sw xsw] = solu_ch4gas(p,T,S)
%
%     Input parameters (scalars or vectors): 
%         p        -  pressure [MPa]     (0.1 < P < 200)
%         T        -  temperature [degC] (0 < T < 250)
%         S        -  practical salinity (S < 350)
%     
%     Output parameter: 
%         mCH4sw   - molality of CH4 in seawater [mol/kg]
%         xsw      - mole fraction of H2O in gas phase
%
%     References:
%     Duan & Mao(2006) GCA 70, 3369-3386.
%     Duan et al. (1992) GCA 56, 2605-2617.
%     Wagner & Pruss (1993) J.Phys.Chem.Ref.Data 22,783-787
%     Shibue (2003) Fluid Phase Equilibria 213, 39-51.
%
%     Comments: 
%     This script does not check, if the pressure and temperature
%     conditions are within the hydrate stability zone. Use solu_ch4 to
%     discriminate between 2 phase and 3 phase solubilities.
%
%     No solubility for gaseous CH4 in SO4-free porewater available so far.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       December 2009: concentration changed from molarity to molality (l.78)
%                      Mole fraction for NaCl(needed for subroutine
%                      satpress) changed from mole fraction of ionic
%                      components to molefraction of salt (line 244, see
%                      comment in lines 229-242)
%       September 2011: output parameter xsw added
%       November 2012: syntax changed to Matlab2012 style
%       February 2013: function call for seasalt updated, mNaCl substituded
%                      by ionic strength equivalent, check for NaN changed
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_CH4GAS:Input','SOLU_CH4GAS: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>200) && ~any(temp<0 | temp>250) ...
        && ~any( sal<0 | sal>350)
else
    bad = find(press<0.1 | press > 200);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>250);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>350);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLU_CH4GAS:OutOfParameterRange','SOLU_CH4GAS: Input values out of range!')
    warning('SOLU_CH4GAS:OutOfParameterRange','Equations valid for p = 0.1-200 MPa, T= 0-250 degC, S = 0-350.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T,S] = ndgrid(press.*10,temp+273.15,sal); % P [bar], T [K], S [g/kg]
R2006 = 83.14472; % gas constant in bar cm^3/mol/K for Duan 2006
%R1992 = 0.08314467; % gas constant in bar dm^3/k/mol for Duan 1992
 
mmh2o    = 18.0152; % molar mass of H2O [g/mol]
mh2o = 1/mmh2o * 1000; % [mol/g * g/m^3] = [mol/kg]
[mNa,mMg,mCa,mK,~,~,mCl,~,~,mSO4] = seasalt(S); % m [mol/kg]
mNaCl = i_sw(S); % use ionic strength as a surrogate for NaCl concentration

%------------------------------------------------------------------------
% Calculation of chemical potential (uch4), Pitzer parameters lambda and zeta
% fugacity coefficient of water (fch2o) and CH4 (phich4), molar volume of 
% water and NaCl and saturation pressure of water from multiparametric equations.
%------------------------------------------------------------------------

[uch4] = par1(P,T);
[lambdaNa] = par2(P,T);
[zetaNaCl] = par4;
[~,phich4] = eosch4(P,T);
[fch2o] = fugcoeff(P,T);
[mfh2o,mfnacl] = molfrac(mNa,mMg,mCa,mK,mCl,mSO4,mh2o,mNaCl);
[mvh2o] = molvol(T);
[psh2o] = satpress(mfnacl,T);

%------------------------------------------------------------------------
% Calculation of molality of CH4 in solution 
%------------------------------------------------------------------------

% calculation of mole fraction of water in gas phase
xsw = mfh2o.*psh2o./fch2o./P.*exp(mvh2o.*(P-psh2o)./R2006./T); % sw = saltwater
% calculation of mole fraction of CH4 in gas phase
xch4sw = 1 - xsw; 

lnmch4 = log(xch4sw.*phich4.*P) - uch4 ...
        - 2.*lambdaNa.*(mNa + mK + 2.*mCa + 2.*mMg) ... % molalities in mol/kg
        - 0.1328.*mSO4 - zetaNaCl.*(mNa + mK + 2.*mCa + 2.*mMg)...
        .*(mCl + 2.*mSO4);
mCH4sw = exp(lnmch4); % Molality of dissolved CH4 [mol/kg]
%cCH4sw = mCH4sw.*rsw./1000; % Molarity [mol/l]

end
% ----------------------------------------------------------

function [uch4,duch4dp] = par1(P,T)
% parametric calculation of the chemical potential
% and its derivation with respect to pressure

c1 = 8.3143711;
c2 = -0.00072772168;
c3 = 2148.9858;
c4 = -0.000014019672;
c5 = -667434.49;
c6 = 0.0076985890;
c7 = -0.0000050253331;
c8 = -3.0092013;
c9 = 484.68502;

uch4 = c1 + c2.*T + c3./T + c4.*T.^2 + c5./T.^2 + c6.*P ...
     + c7.*P.*T + c8.*P./T + c9.*P./T.^2;
 
duch4dp = c6 + c7.*T + c8./T + c9./T.^2;

% ----------------------------------------------------------

function [lambdaNa,dlNadp] = par2(P,T)
% parametric calculation of interaction parameter lambda
% and its derivation with respect to pressure

c1 = -0.81222036;
c2 = 0.0010635172;
c3 = 188.94036;
c6 = 0.000044105635;
c10 = -0.000000000046797718;

lambdaNa = c1 + c2.*T + c3./T + c6.*P + c10.*P.^2.*T;

dlNadp = c6 + 2*c10.*P.*T;

% ----------------------------------------------------------

function [zetaNaCl] = par4
% parametric calculation of interaction parameter zeta

c1 = -0.0029903571;
zetaNaCl = c1;

% ----------------------------------------------------------

function [Zch4,phich4,fugch4] = eosch4(P,T) %,R1992)

% Calculation of fugacity coefficient and fugacity of CH4 from the 
% Equation of state 

Pcrit = 46.41; %bar
Tcrit = 273.15 + (-82.55); % K
%Vcrit = R1992*Tcrit/Pcrit; % dm^3/mol
Pred = P./Pcrit;
Tred = T./Tcrit;
Vred= zeros(size(P,1),size(P,2),size(P,3));
a1 = 0.0872553928;
a2 = -0.752599476;
a3 = 0.375419887;
a4 = 0.0107291342;
a5 = 0.0054962636;
a6 = -0.0184772802;
a7 = 0.000318993183;
a8 = 0.000211079375;
a9 = 0.0000201682801;
a10 = -0.0000165606189;
a11 = 0.000119614546;
a12 = -0.000108087289;
alpha = 0.0448262295;
beta = 0.75397;
gamma = 0.077167;
BB = a1 + a2./Tred.^2 + a3./Tred.^3;
CC = a4 + a5./Tred.^2 + a6./Tred.^3;
DD = a7 + a8./Tred.^2 + a9./Tred.^3;
EE = a10 + a11./Tred.^2 + a12./Tred.^3;
FF = alpha./Tred.^3;

for i=1:size(P,1)
    for k=1:size(P,2)
        if isnan(Pred(i,k));
            Vred(i,k,:) = NaN*1;
        elseif isnan(Tred(i,k));
            Vred(i,k,:) = NaN*1;
        else  
            % calculate reduced volume by numerical iteration
    Vred(i,k,:) = fzero(@volch4,10,[],Pred(i,k),Tred(i,k),BB(i,k),CC(i,k),...
    DD(i,k),EE(i,k),FF(i,k),beta,gamma);
        end
    end
end

GG = FF./2./gamma.*(beta + 1 - (beta + 1 + gamma./Vred.^2) ...
     .*exp(-gamma./Vred.^2));

Zch4 = Pred.*Vred./Tred; %equation of state
phich4 = exp(Zch4 - 1 - log(Zch4) + BB./Vred + CC./2./Vred.^2 ...
             + DD./4./Vred.^4 + EE./5./Vred.^5 + GG); % fugacity coefficient
fugch4 = phich4.*P; % fugacity

% -------------------------------------------------------

function CH4 = volch4(Vred,Pred,Tred,BB,CC,DD,EE,FF,beta,gamma)

CH4 = -Pred.*Vred./Tred + 1 + BB./Vred + CC./Vred.^2 ...
    + DD./Vred.^4 + EE./Vred.^5 ...
    + FF./Vred.^2.*(beta + gamma./Vred.^2).*exp(-gamma./Vred.^2);

% ------------------------------------------------------------

function [mfh2o,mfnacl,mfsalt] = molfrac(mNa,mMg,mCa,mK,mCl,mSO4,mh2o, mNaCl)

% Calculation of mole fraction of water, NaCl and salt in the liquid.
%
% The mole fraction of NaCl is needed for the calculation of the saturation
% pressure.  The equations for p_sat were evaluated as a function of NaCl
% or CaCl2 content. Duan does not state in his paper, how he is treating
% seasalt within this equation. 
%
% The mole fraction of seasalt is needed for Duan's equation. Duan
% explicitly does not consider the mole fraction of the salt, but the mole
% fraction of the salt ions. Therefore, the mole fraction can be calculated
% from the molalities of the salt components.

mfh2o = mh2o./(mh2o + mNa + mCl + mK + mCa + mMg + mSO4);
mfnacl = (mNaCl )./(mh2o +  mNaCl);
mfsalt=(mNa + mCl + mK + mCa + mMg + mSO4)./(mh2o + mNa + mCl + mK + mCa + mMg + mSO4);


% ------------------------------------------------------------

function [mvh2o] = molvol(T)

% calculation of water molar volume, approximated as saturated 
% liquid phase volume in cm^3/mole

mmolh2o = 18.0153; % in g/mol
Tc = 647.096;
rhoc = 322; % in kg/m^3 or g/dm^3 or mg/cm^3 or ...
tau = 1 - T./Tc;
b1 = 1.99274064;
b2 = 1.09965342;
b3 = -0.510839303;
b4 = -1.75493479;
b5 = -45.5170352;
b6 = -674694.45;

rhoh2o = rhoc.*(1 + b1.*tau.^(1./3) + b2.*tau.^(2./3) + b3.*tau.^(5./3) ...
        + b4.*tau.^(16./3) + b5.*tau.^(43./3) + b6.*tau.^(110./3));
    
  mvh2o = mmolh2o./rhoh2o.*1000;

% ------------------------------------------------------------

function [fch2o] = fugcoeff(P,T)

% calculation of the fugacity coefficient of water

a1 = -0.0142006707;
a2 =  0.0108369910;
a3 = -0.00000159213160;
a4 = -0.0000110804676;
a5 = -3.14287155;
a6 = 0.00106338095;

fch2o = exp(a1 + a2.*P + a3.*P.^2 + a4.*P.*T + a5.*P./T + a6.*P.^2./T);

% ------------------------------------------------------------

function [psh2o] = satpress(mfnacl,T)

% calculation of saturation pressure of water [bar]

Pc = 22.064; % in MPa
Tc = 647.096;
Tr = T./Tc;
a1 = 0.128746;
a2 = -0.731097;
a3 = -315.058;
b1 = 392.767;
b2 = -2464.4;
u = 0.024;
c1 = -7.85951783;
c2 = 1.84408259;
c3 = -11.7866497;
c4 = 22.6807411;
c5 = -15.9618719;
c6 = 1.80122502;
x = mfnacl;


lnppc = 1./Tr.*( c1.*(1-Tr) + c2.*(1-Tr).^(1.5) + c3.*(1-Tr).^3) ...
        + 1./Tr.*(c4.*(1-Tr).^(3.5) + c5.*(1-Tr).^4 + c6.*(1-Tr).^(7.5));
    
  if (x < u)
      h = a2.*x./(x+a1.^2)+a3.*x.^2;
  else
      h = ((a1.^2.*a2)./(u + a1.^2).^2 + 2.*a3.*u).*(x - u) + b1.*(x - u).^2 ...
        + b2 .* (x-u).*(x.^2-u.^2) + a2.*u./(u+a1.^2) + a3.*u.^2;
  end
    
  lnppc = lnppc + h;
    
psh2o = Pc.* exp(lnppc)*10;   % pressure in bar 


% ------------------------------------------------------------