function [mco2sw,mco2pw] = solu_co2gh(press,temp,sal)

% SOLU_CO2GH -  calculates the solubility of CO2 in seawater and 
%     SO4-free porewater in equilibrium with CO2 hydrate (no gas phase).
%     
%     Usage: [mco2pw,mco2sw] = solu_co2gh(p,T,S,pgh)
%
%     Input parameters (scalars or vectors): 
%         p        -  pressure [MPa]     (0.1 < P < 50)
%         T        -  temperature [degC] (0 < T < 12)
%         S        -  practical salinity (S < 40)
%     
%     Output parameter: 
%         mco2sw     - molality of CO2 in seawater [mol/kg]
%         mco2pw     - molality of CO2 in SO4-free porewater [mol/kg]
%
%     References: 
%     Tishchenko et al. (2009) 8th International CO2 Conference, Jena.
%
%     Comments: 
%     This script does not check, if the pressure and temperature conditions
%     are outside the hydrate stability zone. Use solu_co2 to discriminate
%     between 2-phase and 3-phase solubilities.
%     The equations are valid up to 10 degC. Extrapolation to 12 degC has
%     been allowed to cover the full parameter range of the hydrate model.
%                
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       September 2011: name changed in function call: 
%                       phase_co2h2o -> phase_co2sw
%       September 2011: variable names changed: 
%       co2ghsw -> mco2sw; co2ghpw -> mco2pw
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------



%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_CO2GH:Input','SOLU_CO2GH: Missing input parameter! Please provide p, T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>50) && ~any(temp<0 | temp>12) ...
        && ~any( sal<0 | sal> 40)
else
    bad = find(press<0.1 | press > 50);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>12);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>40);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLU_CO2GH:OutOfParameterRange','SOLU_CO2GH: Input values out of range!')
    warning('SOLU_CO2GH:OutOfParameterRange','Equations valid for p = 0.1-50 MPa, T= 0-12 degC, S = 0-263.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T,S] = ndgrid(press,temp+273.15,sal); % P [MPa], T [K]

% call external function to calculate the phase boundaries
[~,~,~,pgh] = phase_co2sw(temp,sal); 
% resize matrices to account for pressure vector
Pgh= zeros(length(press),length(temp),length(sal));

for i=1:size(press,2) % create matrices for phase boundaries
Pgh(i,:,:) = pgh(:,:); 
end

%------------------------------------------------------------------------
% Calculation of solubilities
%------------------------------------------------------------------------

% a) in seawater 
% 1. at the hydrate dissociation pressure Pgh
% (-> for 3 coexisting phases: gas, hydrate, seawater)
lnco2ghswPdis = -1.36963D1 + 1.4756D-1.*S.^(1/2) ...
    + 3.0383D-1.*S - 8.56934D-2.*S.^(3/2) + ...
    + T .* (3.28732D-2 - 1.0374D-3.*S.^(1/2) ...
    - 2.2708D-3.*S + 6.1955D-4.*S.^(3/2)) ...
    + T.^2 .* (6.05926D-5 + 1.8304D-6.*S.^(1/2) ...
    + 4.2199D-6.*S - 1.11943D-6.*S.^(3/2)) ;
%
% 2. at hydrostatic pressure
% (-> seawater in equilibrium with CO2 hydrate; no CO2g/CO2l)
lnco2ghsw = lnco2ghswPdis + ...
    (P-Pgh) .* (-1.02473D-2 - 5.725D-6.*S ...
    + T .* (2.8636D-5 + 2.512D-8.*S)) ...
    + (P-Pgh).^2 .* (9.0411D-5 + 1.102D-8.*S ...
    + T .* (-2.540D-7 - 5.568D-11.*S)) ;
mco2sw = exp(lnco2ghsw); % [mol/kg]
%

% b) in anoxic porewater (i.e., Cso4=0)
% 1. at the hydrate dissociation pressure Pgh
% (-> for 3 coexisting phases: gas, hydrate, seawater)
lnco2ghpwPdis = -13.6966 + 1.5516D-1.*S.^(1/2) ...
    + 2.7581D-1.*S - 8.46976D-2.*S.^(3/2) ...
    + T .* (3.287496D-2 - 1.0901442D-3.*S.^(1/2) ...
    - 2.049705D-3.*S + 6.12828D-4.*S.^(3/2)) ...
    + T.^2 .* (6.059D-5 + 1.9215D-6.*S.^(1/2) ...
    + 3.78223D-6.*S - 1.10812D-6.*S.^(3/2)) ;
% 
% 2. at hydrostatic pressure
% (-> seawater in equilibrium with CO2 hydrate; no CO2g/CO2l)
lnco2ghpw = lnco2ghpwPdis + ...
    (P-Pgh) .* (-1.02473D-2 - 5.725D-6.*S ...
    + T .* (2.8636D-5 + 2.512D-8.*S)) ...
    + (P-Pgh).^2 .* (9.0411D-5 + 1.102D-8.*S ...
    + T .* (-2.540D-7 - 5.568D-11.*S)) ;
mco2pw = exp(lnco2ghpw); % [mol/kg]
end
