function m = solu_quartz(press,temp,sal,choice)

% SOLU_QUARTZ -  calculates the solubility of quartz in seawater
%       The user can choose 2 formalisms for different pTS conditions.
%
%       Usage: m = solu_quartz(p,T,S,c)
%
%       Input parameters (scalars or vectors): 
%           p     - pressure [MPa]       
%           T     - temperature [degC]   
%           S     - practical salinity 
%           c     - choice of algorithm
%                   'FOURNIER' (default) (T=0-350 degC, p=0.1-100 MPa)
%                   'DAMM' (for higher pT) 
%
%       Output parameters: 
%          m      - solubility of quartz [mol/kg]
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (November 2012)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_QUARTZ:Input','SOLU_QUARTZ: Missing input parameter! Please provide p, T and S!')
else

%------------------------------------------------------------------------
% Choose algorithm based on parameter range
%------------------------------------------------------------------------
if nargin == 3
    if ~any(press<0.1 | press>100) && ~any(temp< 0 | temp>350) ...
        && ~any(sal < 0 | sal>80)

        choice = 'FOURNIER'; % p=0.1-100 MPa, T=0-350 degC, S=0-80

    elseif ~any(press<0.1 | press>986) && ~any( temp<45 | temp>900) ...
        && ~any( sal<0 | sal>80)

        choice = 'DAMM'; % p=0.1-986 MPa, T=45-900 degC, S=0-80

    else
        warning('SOLU_QUARTZ:OutOfParameterRange','SOLU_QUARTZ: Input values out of range!')
        warning('SOLU_QUARTZ:OutOfParameterRange','SOLU_OPALFOURNIER valid for p=0.1-100 MPa, T=0-350 degC, S=0-80.')
        warning('SOLU_QUARTZ:OutOfParameterRange','Extended Equations valid for p=0.1-986 MPa, T=45-900 degC, S=0-80.')
    end
end

%------------------------------------------------------------------------
% Algorithm choices
%------------------------------------------------------------------------
switch choice
    case 'DAMM'
        m = solu_quartzDamm(press,temp,sal);
    case 'FOURNIER' 
        m = solu_quartzFournier(press,temp,sal);
    otherwise % use Fournier's formalism as default
        m = solu_quartzFournier(press,temp,sal);
end

end
