function m = solu_quartzDamm(press,temp,sal)

% SOLU_QUARTZDAMM -  calculates the solubility of quartz in seawater for
%      hydrothermal conditions
%
%      Usage: m = solu_quartzDamm(p,T,S)
%
%      Input parameters (scalars or vectors): 
%          p      - pressure [MPa]       (0.1 < p < 986)
%          T      - temperature [degC]   (45 < T < 900)
%          S      - practical salinity   (0 < S < 80) 
%   
%      Output parameter:
%          m      - solubility of quartz [mol/kg]
%
%      References:
%      von Damm et al. (1991) AJS 291, 977-1007.
%      Sun et al. (2008) DSR I 55, 1304-1310. (salinity correction density)
%
%      Comments:
%      The equation does not include salinity effects. Von Damm et al.
%      state that salinity effects are within the range of uncertainty
%      for moderate pressures and temperatures. 
%      For high p and T, non-neglible salinity effects are expected.
%      According to Newton & Manning (2000) GCA 64, 2993-3005, the
%      salinity dependence of the solubility at high pressures (> 10000 bar)
%      is an exponential function, but they do not provide an equation.
%      Von Damm et al. suggest to correct parameter 'a' in their equation
%      for the amount of water that is immobilzed as a hydration layer 
%      around the salt ions. 
%      The salinity correction for the density is valid in the liquid
%      phase only for 0.1 < p < 100, 0 < T < 374, 0 < S < 40 
%      (Von Damm et al. state that extrapolation up to S = 80 is ok).
%      The calculated density values for T < 350 degC compare well with
%      the data from Fontaine et al. (2007) EPSL 275, 132-145.
%      Data deviate for higher temperatures. 
%      
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       January 2013: name in internal function call changed: eos_H2O -> eos_h2o 
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_QUARTZDAMM:Input','SOLU_QUARTZDAMM: Missing input parameter! Please provide p,T and S!')
else

    

%------------------------------------------------------------------------
% Check data range validity!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>986) && ~any(temp<(45) | temp>900) && ~any(sal<(0) | sal>80) 
        
else
    bad = find(press<0.1 | press > 986);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(45) | temp>900);
    temp(bad)=NaN*ones(size(bad));
        bad = find(sal<(0) | sal>80);
    sal(bad)=NaN*ones(size(bad));
 warning('SOLU_QUARTZDAMM:OutOfParameterRange','SOLU_QUARTZDAMM: Input values out of range!')
    warning('SOLU_QUARTZDAMM:OutOfParameterRange','Equations valid for p = 0.1-986 MPa, T=45-900 degC, S<80.')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press*10,temp + 273.15, sal*1.005); % p in bar T in K, absolte salinity
[PM, TC] = ndgrid(press, temp, sal); % p in MPa, T in C
rho = zeros(size(P));

B(1) = -7.99992230e-1;
B(2) =  2.40936500e-3;
B(3) = -2.58052775e-5;
B(4) =  6.85608405e-8;
B(5) =  6.29761106e-4;
B(6) = -9.36263713e-7;

% call external function to calculate density of water, approximate density
% of solution by density of water
rho_i = eos_h2o(press,temp); 

% resize density matrix
for i = 1: size(rho,3)
    rho(:,:,i) = rho_i;
end

% calculate correction for salinity
B = S.*B(1) + S.*TC.*B(2) + S.*TC.^2.*B(3) + S.*TC.^3.*B(4) + PM.*S.*B(5) ...
    + PM.^2.*S.*B(6);

rho = (rho -B)./1000;

a = -2.32888;
b = 1.79547;
c = -2263.62;
d = 0.00407350;
e = 0.0398808;

%------------------------------------------------------------------------
% Calculation of solubility
%------------------------------------------------------------------------

m = exp(a + b .* log(rho) + (c + d.*T.^2)./T + e.*P./T);

end
