function [cw, xw] = solu_swco2(press,temp,sal)

% SOLU_SWCO2 -  calculates the solubility of water in liquid CO2 as a 
%      function of pressure, temperature and salinity from a 
%      multiparameter equation.
%
%      Usage: [cw xw] = solu_swco2(p,T,S)
%
%      Input parameters (scalars or vectors): 
%           p       - pressure [MPa]     (5 < p < 20)
%           T       - temperature [degC] (10 < T < 40)
%           S       - practical salinity (0 < S < 40)
%
%      Output parameters:
%           cw      - molality of H2O dissolved in CO2 [mol/kg{CO2}]
%           xw      - mole fraction of H2O in liquid CO2 
%
%      References:
%      Tishchenko et al. (unpublished).
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       November 2009: Pressure conversion to Pa corrected (line 60)
%       September 2011: Name changed from solu_h2oco2 -> solu_swco2
%       March 2015: Output in "real" molalities: mol/kg(CO2) instead of
%       mol/kg(solution)
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_SWCO2:Input','SOLU_SWCO2: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<5 | press>20) && ~any(temp< 10 | temp>40) ...
        && ~any(sal < 0 | sal>40)
else
    bad = find(press<5 | press > 20);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp< 10 | temp>40);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>40);
    temp(bad)=NaN*ones(size(bad));
    warning('SOLU_SWCO2:OutOfParameterRange','SOLU_SWCO2: Input values out of range!')
    warning('SOLU_SWCO2:OutOfParameterRange','Equations valid for p=5-20 MPa, T=10-40 degC, S=0-40.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T,S] = ndgrid(press.*1e6,temp+273.15,sal); %pressure in Pa, temperature in K

%------------------------------------------------------------------------
% Liquid CO2 in contact with pure water: calculation of water mole fraction
% in CO2
%------------------------------------------------------------------------

ln1xw0 = 0.44675 - 2.9976e-3.*T + 4.9994e-6.*T.^2 + ...
    (-4.2333 + 2.9086e-2 .*T - 5.0027e-5 .*T.^2).*1e-8 .*P + ...
    (7.9385 - 5.4587e-2 .*T + 9.3898e-5 .*T.^2).*1e-16 .*P.^2; % ln(1 - xw0)

xw0 = -exp(ln1xw0) + 1; % molefraction of pure water in CO2

%------------------------------------------------------------------------
% Liquid CO2 in contact with seawater: calculation of water mole fraction
% in CO2
%------------------------------------------------------------------------

xw = xw0 .* exp(-5.3577e-4 .*S); % mole fraction of H2O in solution

cw = xw./(44e-3.*(1-xw)); % molality of H2O in solution [mol/kg]

end
