function [m, KH, KHx] = soluhenry_ch4sw(press,temp,sal,pp)

% SOLUHENRY_CH4SW -  calculates the equilibrium concentration of CH4 
%     in seawater from Henry's law
%   
%     Usage:  [m KH KHx]  = soluhenry_ch4sw(p,T,S,pp)
%
%     Input parameters (scalars or vectors, pp: scalar or matrix of size p x T x S): 
%           p        -  pressure [MPa]      (0.1 < p < 200)
%           T        -  temperature [degC]  (0 < T < 250)
%           S        -  practical salinity  (0 < S < 350)
%           pp       -  partial pressure of CH4 [MPa] (pp < p)
%                       pp is set to p if no input value is provided 
%     
%     Output parameter: 
%           m        - molality of CH4 in seawater [mol/kg]
%           KH       - pressure and salinity corrected Henry constant [mol/kg/MPa]
%           KHx      - pressure and salinity corrected Henry constant [1/MPa]
%
%     References: 
%     Harvey (1996) AIChE 42, 1491-1494. (Henry constant)
%     Duan & Mao (2006) GCA 70, 3369-3386. (gamma term + partial molar volume of CH4)
%
%     Comments: 
%     Henry's law is not well suited for high pressures and should be used
%     with care at high pressures: Keep in mind that results are expected
%     to be inaccurate.
%
% -----------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (January 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% -----------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLUHENRY_CH4SW:Input','SOLUHENRY_CH4SW: Missing input parameter! Please provide p,T,S and pp(optional)!')
else
 
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>50) && ~any(temp<0 | temp>360) ...
        && ~any( sal<0 | sal>40)
else
    bad = find(press<0.1 | press > 50);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>360);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>40);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLUHENRY_CH4SW:OutOfParameterRange','SOLUHENRY_CH4SW: Input values out of range!')
    warning('SOLUHENRY_CH4SW:OutOfParameterRange','Equations valid for p = 0.1-50 MPa, T= 0-360 degC, S = 0-40.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press,temp+273.15,sal); % temperature in [K]

   if nargin == 3 % if no partial pressure is provided by user, partial pressure is set to system pressure
       pp = P; 
    end

R = 83.131; % gas constant [(bar cm^3)/(mol K)]

mmh2o    = 18.0152; % molar mass of H2O [g/mol]
mh2o = 1/mmh2o * 1000; % [mol/g * g/m^3] = [mol/kg]

% call external function to calculate the composition of seawater
[mNa,mMg,mCa,mK,~,~,mCl,~,~,mSO4] = seasalt(S); % m [mol/kg]

mNaCl = i_sw(S); % use ionic strength as a surrogate for NaCl concentration
mfnacl = (mNaCl )./(mh2o +  mNaCl);

%------------------------------------------------------------------------
%Calculation of water saturation pressure of from a multiparametric equation.
%------------------------------------------------------------------------
[psh2o] = satpress(mfnacl,T);

%------------------------------------------------------------------------
% Calculation of partial molar volume
%------------------------------------------------------------------------

Vch4 = molvolch4(P.*10,T,R,mNa, mMg, mCa, mK); % molar volume of CH4 in seawater


% create empty matrix
phi  = zeros(size(T));

[~, ~, ~, ~, ~, ~, phi0] = eos_ch4(press,temp);
 
% enlarge array
for i = 1:length(sal)
    phi(:,:,i) = phi0(:,:);
end

f = pp.*phi; % gas phase fugacity of solute

% call function to calculate Henry constant
KHx = henry_ch4(T); % [1/MPa], KHx = x/f

%------------------------------------------------------------------------
% Calculation of activity coefficient
%------------------------------------------------------------------------
[lambdaNa] = par2(P*10,T);
[zetaNaCl] = par4;


gamma = exp( 2.*lambdaNa.*(mNa + mK + 2.*mCa + 2.*mMg) ... % molalities in mol/kg
        + 0.1328.*mSO4 + zetaNaCl.*(mNa + mK + 2.*mCa + 2.*mMg)...
        .*(mCl + 2.*mSO4));

%------------------------------------------------------------------------
% Calculation of pressure correction
%------------------------------------------------------------------------

% pressure correction using the relation:
%  int(V_mol/R/T dp)
%  where V_mol = partial molar volume of solvent, pressure in [Pa]

IntVCH4 = Vch4.*(P-psh2o).*10;   % integrated molar volume of dissolved CH4 [cm^3/mol*bar]

pcor = exp(IntVCH4./R./T);  %pressure correction factor

%------------------------------------------------------------------------
% Calculation of solubility
%------------------------------------------------------------------------
KHx = KHx./gamma./pcor; % pressure and salinty correction for Henry constant

% convert Henry constant to [mol/kg/MPa]
KH = 55.5*KHx./(1 - f.*KHx);

m = f.*KH; % [mol/kg]

end
% ------------------------------------------------------------
% ------------------------------------------------------------

function [psh2o] = satpress(mfnacl,T)

% calculation of saturation pressure of water [bar]

Pc = 22.064; % in MPa
Tc = 647.096;
Tr = T./Tc;
a1 = 0.128746;
a2 = -0.731097;
a3 = -315.058;
b1 = 392.767;
b2 = -2464.4;
u = 0.024;
c1 = -7.85951783;
c2 = 1.84408259;
c3 = -11.7866497;
c4 = 22.6807411;
c5 = -15.9618719;
c6 = 1.80122502;
x = mfnacl;


lnppc = 1./Tr.*( c1.*(1-Tr) + c2.*(1-Tr).^(1.5) + c3.*(1-Tr).^3) ...
        + 1./Tr.*(c4.*(1-Tr).^(3.5) + c5.*(1-Tr).^4 + c6.*(1-Tr).^(7.5));
    
  if (x < u)
      h = a2.*x./(x+a1.^2)+a3.*x.^2;
  else
      h = ((a1.^2.*a2)./(u + a1.^2).^2 + 2.*a3.*u).*(x - u) + b1.*(x - u).^2 ...
        + b2 .* (x-u).*(x.^2-u.^2) + a2.*u./(u+a1.^2) + a3.*u.^2;
  end
    
  lnppc = lnppc + h;
    
psh2o = Pc.* exp(lnppc);   % pressure in[MPa]

%------------------------------------------------------------------------


function Vch4 = molvolch4(P,T,R,mNa, mMg, mCa, mK)
% This function calculates the partial molar volume of CH4 dissolved in 
% seawater 


% calculation of the derivative of mu/RT with respect to pressure from a
% multiparametric equation
c6 = 0.0076985890;
c7 = -0.0000050253331;
c8 = -3.0092013;
c9 = 484.68502;

duch4dp = c6 + c7.*T + c8./T + c9./T.^2;

% calculation of derivative of the Pitzer coefficient with respect to 
% pressure from amultiparametric equation

c6_l = 0.000044105635;
c10_l = -0.000000000046797718;

dlNadp = c6_l + 2*c10_l.*P.*T;

% calculation of molar volume of dissolved CH4: 
% V/RT = d(mu/RT)/dP + sum(2*m_cation*dlambda_cation/dP),
% the Pitzer coefficient for anions is defined as zero

Vch4 = R.*T.*(duch4dp + 2.* dlNadp.*(mNa + mK + 2.*mCa + 2.* mMg));%[cm^3/mol]

%------------------------------------------------------------------------
function KHch4 = henry_ch4(T)
% calculation of Henry constant

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

Tc = 647.096;       % critical temperature of water
Pc = 22.064;        % critical pressure of water
tau = 1 - T./Tc;    % dimensionless temperature
Ts = T./Tc;         % reduced temperature

%------------------------------------------------------------------------
% Calculation of water vapor pressure
%------------------------------------------------------------------------

a1 = -7.85951783;
a2 = 1.84408259;
a3 = -11.7866497;
a4 = 22.6807411;
a5 = -15.9618719;
a6 = 1.80122502;

% vapor pressure of water
pvap = Pc.*exp(Tc./T.*(a1.*tau + a2.*tau.^1.5 + a3.*tau.^3 + a4.*tau.^3.5...
    + a5.*tau.^4 + a6.*tau.^7.5));

%------------------------------------------------------------------------
% Calculation of Henry constant
%------------------------------------------------------------------------

A = -11.0094;
B = 4.8362;
C = 12.5220;

% Henry constant in [MPa]
kh_H = exp(log(pvap) + A./Ts + B.*(1 - Ts).^0.355 ./Ts + ...
    C.*exp(1 - Ts).*Ts.^(-0.41));

KHch4 = 1./kh_H;%  Henry constant in [1/MPa]

% ----------------------------------------------------------

function [lambdaNa,dlNadp] = par2(P,T)
% parametric calculation of interaction parameter lambda
% and its derivation with respect to pressure

c1 = -0.81222036;
c2 = 0.0010635172;
c3 = 188.94036;
c6 = 0.000044105635;
c10 = -0.000000000046797718;

lambdaNa = c1 + c2.*T + c3./T + c6.*P + c10.*P.^2.*T;

dlNadp = c6 + 2*c10.*P.*T;

% ----------------------------------------------------------

function [zetaNaCl] = par4
% parametric calculation of interaction parameter zeta

c1 = -0.0029903571;
zetaNaCl = c1;

% ----------------------------------------------------------