function [m, KH, KHx] = soluhenry_h2ssw(press,temp,sal,pp)

% SOLUHENRY_H2SSW -  calculates the equilibrium concentration of H2S
%     in seawater from Henry's law
%   
%     Usage:  [m KH KHx]  = soluhenry_h2ssw(p,T,S,pp)
%
%     Input parameters (scalars or vectors, pp: scalar or matrix of size p x T x S): 
%           p        -  pressure [MPa]      (0.1 < p < 20)
%           T        -  temperature [degC]  (0 < T < 127)
%           S        -  practical salinity  (0 < S < 350)
%           pp       -  partial pressure of H2S [MPa] (pp < p)
%                       pp is set to p if no input value is provided 
%     
%     Output parameter: 
%           m        - molality of H2S in seawater [mol/kg]
%           KH       - pressure and salinity corrected Henry constant [mol/kg/MPa]
%           KHx      - pressure and salinity corrected Henry constant [1/MPa]
%
%     References:
%     Harvey (1996) AIChE 42, 1491-1494. (Henry constant)
%     Plyasunov et al. (2000) GCA 64, 2779-2795 (partial molar volume of H2S)
%     Duan et al. (2007) Energy & Fuels 21, 2056-2065. (gamma term)
%
%     Comments: 
%     Henry's law is not well suited for high pressures and should be used
%     with care at high pressures: Keep in mind that results are expected
%     to be inaccurate.
%
% -----------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (January 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% -----------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLUHENRY_H2SSW:Input','SOLUHENRY_H2SSW: Missing input parameter! Please provide p,T,S and pp(optional)!')
else
 
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>35) && ~any(temp<0 | temp>80) ...
        && ~any( sal<0 | sal>120)
else
    bad = find(press<0.1 | press > 35);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>120);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLUHENRY_H2SSW:OutOfParameterRange','SOLUHENRY_H2SSW: Input values out of range!')
    warning('SOLUHENRY_H2SSW:OutOfParameterRange','Equations valid for p = 0.1-35 MPa, T= 0-80 degC, S = 0-120.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press,temp+273.15,sal); % temperature in [K]


% check, if H2S is liquid at given pT conditions 
  pv = pvaph2s(temp + 273.15);
    for i = 1:length(temp)
            for k = 1:length(press)
                if P(k,i,1) > pv(i) || isnan(pv(i))
                    P(k,i,:) = NaN;
                    warning('SOLUHENRY_H2SSW:OutOfParameterRange','Equations valid for p < p_vap!')
                end
            end
    end

   if nargin == 3 % if no partial pressure is provided by user, partial pressure is set to system pressure
       pp = P; 
    end

% H2S specific constants
a = -2.8d9;
b = 1.23d-3;
N = 1.57;
sigma12 = 2.87d-8; % [cm]
ek12 = 767;

Omega = 55.51d-6; % 
c1 = 0.0033; % [m^3/kg]
c2 = 0.002; %[m^3/kg]
sigma11 = 2.561d-8; % [cm]
ek11 = 1255;
lambda = 1.22;
Na = 6.02215d23; % Avogadro's number
R = 8.3145; % gas constant [J/mol/K]

mmh2o    = 18.0152; % molar mass of H2O [g/mol]
mh2o = 1/mmh2o * 1000; % [mol/g * g/m^3] = [mol/kg]

% call external function to calculate the composition of seawater
[mNa,mMg,mCa,mK,~,~,mCl,~,~,mSO4] = seasalt(S); % m [mol/kg]

mNaCl = i_sw(S); % use ionic strength as a surrogate for NaCl concentration
mfnacl = (mNaCl )./(mh2o +  mNaCl);

%------------------------------------------------------------------------
%Calculation of water saturation pressure of from a multiparametric equation.
%------------------------------------------------------------------------
[psh2o] = satpress(mfnacl,T);

% create empty matrices
rhow  = zeros(size(T));
kappa  = zeros(size(T));
phi  = zeros(size(T));

% call external function to calculate water density and compressibility
% factor
[rho_w , ~ , ~ , ~ , ~ , ~ , ~ , ~ , ~ , ~, kappa0]  = eos_sw(press,temp,0);


[~, phi0] = eos_h2s(press,temp); % fugacity coefficient of gaseous H2S
 
% enlarge arrays
for i = 1:length(sal)
    rhow(:,:,i) = rho_w(:,:);
    kappa(:,:,i) = kappa0(:,:);
    phi(:,:,i) = phi0(:,:);
end

V10 = 1./rhow./55.51; % molar volume of water [m^3/mol]
f = pp.*phi; % gas phase fugacity of solute

% call function to calculate Henry constant
KHx = henry_h2s(T); % [1/MPa], KHx = x/f

%------------------------------------------------------------------------
% Calculation of second virial coefficients
%------------------------------------------------------------------------
B11 = 2.*pi.*Na.*sigma11.^3./3.*(1 - (lambda.^3 - 1).*(exp(ek11./T)-1));
B12 = 2.*pi.*Na.*sigma12.^3./3.*(1 - (lambda.^3 - 1).*(exp(ek12./T)-1));

%------------------------------------------------------------------------
% Calculation of solute infinite dilution partial molar volume [m^3/mol]
% No salt effects included!
%------------------------------------------------------------------------

V20 = N.*V10 + kappa.*R.*T.*(1 - N) + kappa.*R.*T.*rhow.*(2.*Omega.*...
    (B12 - N.*B11).*exp(-c1.*rhow) + (a./T.^5 + b).*(exp(c2.*rhow) - 1));

%------------------------------------------------------------------------
% Calculation of activity coefficient
%------------------------------------------------------------------------

C1l = 8.5004999d-2; 
C2l = 3.5330378d-5;
C3l = -1.5882605;
C6l = 1.1894926d-5;

C1x = -1.0832589d-2;

% calculation of lambda
lambda = C1l + C2l.*T + C3l./T + C6l.*(P*10);

% calculation of xi
xi = C1x;

gamma = exp(2.*lambda.*(mNa + mK + 2*mCa + 2*mMg)...
    + xi.*(mNa + mK + 2*mCa + 2*mMg).*(mCl + 2*mSO4) + 0.18*mSO4);

%------------------------------------------------------------------------
% Calculation of pressure correction
%------------------------------------------------------------------------

% pressure correction using the relation:
%  int(V_mol/R/T dp)
%  where V_mol = partial molar volume of solvent, pressure in [Pa]

IntVh2s = V20.*(P-psh2o).*1d6;   % integrated molar volume of dissolved H2S

pcor = exp(IntVh2s./R./T);  %pressure correction factor

%------------------------------------------------------------------------
% Calculation of solubility
%------------------------------------------------------------------------
KHx = KHx./gamma./pcor; % pressure and salinty correction for Henry constant

% convert Henry constant to [mol/kg/MPa]
KH = 55.5*KHx./(1 - f.*KHx);

m = f.*KH; % [mol/kg]

end
% ------------------------------------------------------------


function [psh2o] = satpress(mfnacl,T)

% calculation of saturation pressure of water [bar]

Pc = 22.064; % in MPa
Tc = 647.096;
Tr = T./Tc;
a1 = 0.128746;
a2 = -0.731097;
a3 = -315.058;
b1 = 392.767;
b2 = -2464.4;
u = 0.024;
c1 = -7.85951783;
c2 = 1.84408259;
c3 = -11.7866497;
c4 = 22.6807411;
c5 = -15.9618719;
c6 = 1.80122502;
x = mfnacl;


lnppc = 1./Tr.*( c1.*(1-Tr) + c2.*(1-Tr).^(1.5) + c3.*(1-Tr).^3) ...
        + 1./Tr.*(c4.*(1-Tr).^(3.5) + c5.*(1-Tr).^4 + c6.*(1-Tr).^(7.5));
    
  if (x < u)
      h = a2.*x./(x+a1.^2)+a3.*x.^2;
  else
      h = ((a1.^2.*a2)./(u + a1.^2).^2 + 2.*a3.*u).*(x - u) + b1.*(x - u).^2 ...
        + b2 .* (x-u).*(x.^2-u.^2) + a2.*u./(u+a1.^2) + a3.*u.^2;
  end
    
  lnppc = lnppc + h;
    
psh2o = Pc.* exp(lnppc);   % pressure in[MPa]


%------------------------------------------------------------------------
function KHh2s = henry_h2s(T)
% calculation of Henry constant

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

Tc = 647.096;       % critical temperature of water
Pc = 22.064;        % critical pressure of water
tau = 1 - T./Tc;    % dimensionless temperature
Ts = T./Tc;         % reduced temperature

%------------------------------------------------------------------------
% Calculation of water vapor pressure
%------------------------------------------------------------------------

a1 = -7.85951783;
a2 = 1.84408259;
a3 = -11.7866497;
a4 = 22.6807411;
a5 = -15.9618719;
a6 = 1.80122502;

% vapor pressure of water
pvap = Pc.*exp(Tc./T.*(a1.*tau + a2.*tau.^1.5 + a3.*tau.^3 + a4.*tau.^3.5...
    + a5.*tau.^4 + a6.*tau.^7.5));

%------------------------------------------------------------------------
% Calculation of Henry constant
%------------------------------------------------------------------------

A = -5.7131;
B =  5.3727;
C =  5.4227;

% Henry constant in [MPa]
kh_H = exp(log(pvap) + A./Ts + B.*(1 - Ts).^0.355 ./Ts + ...
    C.*exp(1 - Ts).*Ts.^(-0.41));

KHh2s = 1./kh_H;%  Henry constant in [1/MPa]

% ------------------------------------------------------------
function [Pvh2s] = pvaph2s(T)

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

Tc = 373.1; % critical temperature of H2S [K]
Pc = 8.9987; % critical pressure of H2S [MPa]
Tt = 187.6991; % triple point temperature of H2S [K]
Pt = 23.2d-3; % triple point pressure of H2S [MPa]
Ptr = Pt/Pc; % reduced pressure at triple point
Ttr = Tt/Tc; % reduced temperature at triple point
Tbr = 0.57049;
Tr = T./Tc; % reduced temperature
t = (T - Tt)./(Tc -Tt);
tb = (Tbr - Ttr)./(1 - Ttr);

a1 = 0.090204;
b0 = -0.154;

phi = (1 - t)./(1 + a1.*t)./(1 + b0.*t.*(t - tb));

lnpr = phi.*Ttr.*log(Ptr)./Tr;
Pvh2s = exp(lnpr)*Pc;
