function eta = visco_ch4(press,temp)

% VISCO_CH4 - calculates the viscosity of pure CH4 from a multiparametric
%       equation, including real gas effects
%
%       Usage: eta = visco_ch4(p,T)
%
%       Input parameters (scalars or vectors): 
%           p       - pressure [MPa]     (0 < P < 50)
%           T       - temperature [degC] (-178 < T < 227)
%   
%       Output parameter:
%           eta     -   viscosity of methane [Pa*s]
%           
%       References:
%       Hanley et al. (1977) J.Phys.Chem.Ref.Data 6, 597-609.
%
%       Comments: 
%       The equation for eta_r was corrected for a misplaced paranthesis
%       (see deviation between Eq. 4 in the paper and the corresponding
%       equation in Hanley et al. (1975) Cryogenics).
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('VISCO_CH4:Input','VISCO_CH4: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0 | press>50) && ~any(temp< -178 | temp>227)
else
    bad = find(press<0 | press > 50);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-178 | temp>227);
    temp(bad)=NaN*ones(size(bad));
    warning('VISCO_CH4:OutOfParameterRange','VISCO_CH4: Input values out of range!')
    warning('VISCO_CH4:OutOfParameterRange','Equations valid for p = 0-50 MPa, T=(-178) - 227 degC.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[~, T] = ndgrid(press,temp+273.15); % temperature in K

ek= 168; % (energy parameter)/(Boltzmann constant) [K]
rhoc = 0.1628; % critical density [g/cm^3]

% call external functions to calculate density of CH4
[rho] = density_ch4(press,temp); %[kg/m^3]
rho = rho./1000;  % [g/cm^3]

%------------------------------------------------------------------------
% Calculation of the ideal gas part of the viscosity: eta_0
%------------------------------------------------------------------------

G1 = -2.090975e5;
G2 =  2.647269e5;
G3 = -1.472818e5;
G4 =  4.716740e4;
G5 = -9.491872e3;
G6 =  1.219979e3;
G7 = -9.627993e1;
G8 =  4.274152;
G9 = -8.141531e-2;

eta_0 = G1./T + G2.*T.^(-2/3) + G3.*T.^(-1/3) + G4 + G5.*T.^(1/3) ...
    + G6.*T.^(2/3) + G7.*T + G8.*T.^(4/3) + G9.*T.^(5/3);

%------------------------------------------------------------------------
% Calculation of the dilute gas contribution to the viscosity: eta_1
%------------------------------------------------------------------------

A = 1.696985927;
B = -0.133372346;
C = 1.4;

eta_1 = A + B.*(C - log(T./ek)).^2;

%------------------------------------------------------------------------
% Calculation of the viscosity remainder: eta_r
%------------------------------------------------------------------------

theta = (rho - rhoc)./rhoc; % reduced density

E= 1;
j1 = -1.035060586e1;
j2 =  1.7571599671e1;
j3 = -3.0193918656e3;
j4 =  1.8873011594e2;
j5 =  4.2903609488e-2;
j6 =  1.4529023444e2;
j7 =  6.1276818706e3;

eta_r = E.*exp(j1 + j4./T) .* (exp(rho.^(0.1) .* (j2 + j3./T.^(3/2)) + ...
    theta .* rho.^(0.5) .*(j5 + j6./T + j7./T.^2)) -1);

%------------------------------------------------------------------------
% Calculation of the total viscosity: eta
%------------------------------------------------------------------------

eta = (eta_0 + eta_1.*rho + eta_r).*1e-7; % eta in [Pa*s]

end
