function eta = visco_co2(press,temp)

% VISCO_CO2 - calculates the dynamic viscosity of CO2 from a
%     multiparametric equation
%
%     Usage: eta = visco_co2(p,T)
%
%     Input parameters (scalars or vectors): 
%           p        -  pressure [MPa]     (0.1-300 MPa), 
%           T        -  temperature [degC] ((-56) - 827 degC) 
%
%     Output parameter: 
%           eta      - dynamic viscosity [Pa*s]
%
%     References:
%     Fenghour et al. (1998) J.Phys.Chem.Ref.Data 27(1), 31-44.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('VISCO_CO2:Input','VISCO_CO2: Missing input parameter! Please provide p and T!')
else


%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>300) &&  ~any( temp<(-56) | temp>827)
else
    bad = find(press<0.1 | press > 300);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(-56) | temp>827);
    temp(bad)=NaN*ones(size(bad));
    warning('VISCO_CO2:OutOfParameterRange','VISCO_CO2: Input values out of range!')
    warning('VISCO_CO2:OutOfParameterRange','Equations valid for p=0.1-300 MPa, T=(-56)-827 degC.')
end


%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[~,T] = ndgrid(press,temp+273.15); % P [MPa], T [K]

% call external function to calculate density of CO2
rho = density_co2(press,temp);

redT = T ./ 251.296; % reduced temperature

%------------------------------------------------------------------------
% Calculation of viscosity
%------------------------------------------------------------------------

% viscosity is approximated as:
% eta(rho,T) = eta0(T) + deta(rho,T) + detac(rho,T)
% where: eta0  = viscosity at limit of zero density
%        deta  = excess viscosity at elevated density
%        detac = viscosity enhancement near critical point
% eta [uPa s], T [K], rho [kg/m^3]

% zero density viscosity
eta0 = 1.00697.*sqrt(T) ./ exp(0.235156 ...
    - 0.491266.*log(redT) ...
    + 5.211155D-2.*(log(redT)).^2 ...
    + 5.347906D-2.*(log(redT)).^3 ...
    - 1.537102D-2.*(log(redT)).^4) ;
% excess viscosity
deta = 0.4071119D-2.*rho + 0.7198037D-4.*rho.^2 ...
    + 0.2411697D-16.*rho.^6 ./ redT.^3 ...
    + 0.2971072D-22.*rho.^8 ...
    - 0.1627888D-22.*rho.^8 ./ redT ;
% critical enhancement
detac = 0;

eta = (eta0 + deta + detac)./1e6; % viscosity [Pa s]

% alternative formulation:
% after Olchowy & Sengers (1988) Phys.Rev.Lett. 61(1), 15-18.

%deta1 = 5.5934D-3.*rho + 6.1757D-5.*rho.^2 + 2.643D-11.*rho.^4;
%detac1 = 0;
%eta1 = eta0 + deta1 + detac1;

end
%------------------------------------------------------------------------
% Check values
%------------------------------------------------------------------------
%                               without detac     with detac
%rho = 2.440;   %P=1;T=220;     11.0608
%rho = 1.773;   %P=1;T=300;     15.0186
%rho = 0.662;   %P=1;T=800;     35.0849
%rho = 254.320; %P=70;T=304;    20.8996
%rho = 1194.86; %P=150;T=220;   269.4599
%rho = 1029.27; %P=500;T=300;   132.5585
%rho = 407.828; %P=750;T=800;   48.7368

% after Olchowy & Sengers (1988) Phys.Rev.Lett. 61(1), 15-18.
%                               wthout detac     with detac
%rho = 2.440;   %P=1;T=220;     11.0645
%rho = 1.773;   %P=1;T=300;     15.0212
%rho = 0.662;   %P=1;T=800;     35.0859
%rho = 254.320; %P=70;T=304;    20.7321
%rho = 1194.86; %P=150;T=220;   159.7759
%rho = 1029.27; %P=500;T=300;   115.8564
%rho = 407.828; %P=750;T=800;   48.3661

