function eta = visco_co2sw(press,temp,sal,mco2)

% VISCO_CO2SW - calculates the dynamic viscosity of CO2-seawater solutions
%
%       Usage: eta = visco_co2_sw(p,T,S) 
%
%       Input parameters 
%           scalars or vectors: 
%           p        -  pressure [MPa]     (10 < p < 20)
%           T        -  temperature [degC] (20 < T < 60)
%           S        -  practical salinity
%           scalar or matrix of size p x T x S
%           mco2     -  molality of CO2 in solution [mol/kg]
%                       saturation with CO2 is assumed if no value is provided
%
%       Output parameter: 
%           eta  -   dynamic viscosity [Pa*s]
%
%       References:     
%       Bando et al. (2004) J. Chem. Eng. Data 49, 1328-1332.
%
%       Comments: 
%       The valid parameter range is very restricted (p=10-20 MPa, T=30-60 degC,
%       NaCl mass fractions of 0-0.03 and mole fractions of CO2 < 0.02),
%       hence exclusion of pTS values outside this range is not feasible.
%       Experimental data of Kumagi & Yokoyama (1998) (Int.J.Thermophys. 19,
%       1315-1323) show a deviation of up to 10% for temperatures of 0-5 degC
%       compared to values calculated with this script.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%     Modification history: 
%       November 2009: units of viscosity set consistently to [Pa s]
%       September 2011: Format for mco2 specified in header
%       November 2012: syntax changed to Matlab2012 style
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('VISCO_CO2SW:Input','VISCO_CO2SW: Missing input parameter! Please provide p, T and S!')
else
if nargin == 3 % saturation is assumed if mco2 is not specified
    
    %------------------------------------------------------------------------
    % Definition of constants and thermodynamic variables
    %------------------------------------------------------------------------

    [~,T] = ndgrid(press,temp,sal); 
    x=1; % ratio x of actual mole fraction and saturation mole fraction

    %------------------------------------------------------------------------
    % Call external function to calculate viscosity of seawater without CO2
    %------------------------------------------------------------------------

    Eta_sw = visco_sw(press,temp,sal); % [Pa s]

else 
%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T,S,Mco2] = ndgrid(press,temp,sal,mco2); 

Eta_sw = zeros(size(P)); % empty array for seawater viscosity

%------------------------------------------------------------------------
% Calculate (mole fraction of CO2)/(mole fraction of CO2 at saturation)
%------------------------------------------------------------------------


% calculate ratio x of actual mole fraction and saturation molefraction

    mco2s_2 = zeros(size(P)); % empty array for saturation concentration
    % Call external function to calculate the molality of dissolved CO2 in
    % seawater at saturation
    mco2s = solu_co2(press,temp,sal);
    
     for i = 1:size(P,4)      % adapt array size 
        mco2s_2(:,:,:,i)= mco2s;
    end
    
   % call external function to calculate ionic strength of seawater
   I_2 = i_sw(S);
    
    % calculate concentration of Na+ and Cl- that result in the same ionic
    % strength as seawater with salinity S
    mNa = I_2;
    mCl = I_2;
 
    % calculate mole fraction of CO2 at given molality
    xco2 = Mco2./(1000./18 + Mco2 + mNa + mCl);
    
    % calculate mole fraction of CO2 at saturation
    xco2s = mco2s_2./(1000./18 + mco2s_2 + mNa + mCl);
 
    
    x = xco2./xco2s;
    
%------------------------------------------------------------------------
% Call external function to calculate viscosity of seawater without CO2
%------------------------------------------------------------------------

    eta_sw = visco_sw(press,temp,sal); % [Pa s]

    for i = 1:size(P,4)      % adapt array size 
        Eta_sw(:,:,:,i)= eta_sw;
    end
end


%------------------------------------------------------------------------
% Calculate viscosity of CO2-seawater solution
%------------------------------------------------------------------------

eta = (Eta_sw.*(1 + (-4.069e-3.*T + 0.2531).*x)); % [Pa s]
end
