function eta = visco_h2o(press,temp)

% VISCO_H2O -  calculates the viscosity of pure water as function of
%      pressure and temperature based on a multiparametric equation
%
%      Usage: eta = visco_h2o(p,T)
%
%      Input parameters (scalars or vectors): 
%           p       - pressure [MPa]       (0.1 < p < 167.5 )
%           T       - temperature [degC]   (0.1 < T < 800 )
%
%      Output parameter:
%          eta      - dynamic viscosity [Pa*s]
%
%      Range of validity in the original paper:
%           for   p <= 300 MPa              and     T <= 900 degC
%           for   300 MPa < p <= 350 MPa    and     T <= 600 degC
%           for   350 MPa < p <= 500 MPa    and     T <= 160 degC
%           for   500 MPa < p <=1000 MPa    and     T <= 100 degC
%      The calculation of the viscosity requires the density of H2O.
%      Since the current script for density calculation is valid only
%      up to 167.5 MPa, the range of validity for the viscosity
%      calculation is reduced to 167.5 MPa.
%
%      Equations are not valid for ice phase (at temperatures below
%      the melting point temperature).
%
%      References:
%      Release on the IAWPS Formulation 2008 for the Viscosity of 
%           Ordinary Water Substance. http://iapws.org 
%
%      Comments:
%      The critical enhancement close to the critical point 
%      (645.91 < T < 650.77 [K], 245.8 < rho < 405.3 [kg/m^3]) is
%      omitted to save computation time. 
%      For calculations close to the critical point the viscosity shows
%      a singularity and the critical enhancement term as given in
%      the reference has to be considered.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2011)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
%       January 2013: name in internal function call changed: eos_H2O -> eos_h2o 
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('VISCO_H2O:Input','VISCO_H2O: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check if input values are outside maximum permitted parameter range. 
%------------------------------------------------------------------------

if ~any(press<0.1 | press>167.5) && ~any(temp<(0.1) | temp>800) 
        
else
    bad = find(press<0.1 | press > 167.5);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(0.1) | temp>800);
    temp(bad)=NaN*ones(size(bad));
   
 warning('VISCO_H2O:OutOfParameterRange','VISCO_H2O: Input values out of range!')
 warning('VISCO_H2O:OutOfParameterRange','Equations are generally not valid for p > 167.5 MPa, T > 800 degC.')
end    
    
%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

[~, T] = ndgrid(press,(temp+273.15)); % temperature in Kelvin

rho     = eos_h2o(press,temp); % call external function to calculate density
Tr      = 647.096;  % reference temperature [K]
rhor    = 322;      % reference density [kg/m^3]
etar    = 1e-6;     % reference viscosity [Pa s]

Td      = T/Tr;     % dimensionless temperature
rhod    = rho/rhor; % dimensionless density
 


%------------------------------------------------------------------------
% Calculation of dimensionless viscosity in dilute gas limit eta_0
%------------------------------------------------------------------------

H = zeros(4,1);

H(1) =  1.67752;
H(2) =  2.20462;
H(3) =  0.6366564;
H(4) = -0.241605;

eta_0 = 100 .*sqrt(Td)./(H(1) + H(2)./Td + H(3)./Td.^2 + H(4)./Td.^3);

%------------------------------------------------------------------------
% Calculation of dimensionless viscosity contribution due to finite density
% eta_1
%------------------------------------------------------------------------

H = zeros(6,7);

H(1,1) =  5.20094e-1;
H(2,1) =  8.50895e-2;
H(3,1) = -1.08374;
H(4,1) = -2.89555e-1;
H(1,2) = 2.22531e-1;
H(2,2) = 9.99115e-1;
H(3,2) = 1.88797;
H(4,2) = 1.26613;
H(6,2) = 1.20573e-1;
H(1,3) = -2.81378e-1;
H(2,3) = -9.06851e-1;
H(3,3) = -7.72479e-1;
H(4,3) = -4.89837e-1;
H(5,3) = -2.57040e-1;
H(1,4) = 1.61913e-1;
H(2,4) = 2.57399e-1;
H(1,5) = -3.25372e-2;
H(4,5) = 6.98452e-2;
H(5,6) = 8.72102e-3;
H(4,7) = -4.35673e-3;
H(6,7) = -5.93264e-4;

lneta_1 = zeros(size(T)); % empty matrix for viscosity contribution

for i = 1:6
    te      = zeros(size(T)); % empty matrix for term in equation
    for j=1:7
        te= te + H(i,j).*(rhod - 1).^(j-1);
    end
    lneta_1 = lneta_1 + rhod.*(1./Td-1).^(i-1).*te;
end

 
eta_1 = exp(lneta_1);

%------------------------------------------------------------------------
% Calculation of dimensionless critical enhancement term eta_2
%------------------------------------------------------------------------


eta_2 = 1;  
% eta_2 is only needed close to the critical point: 645.91 < T < 650.77 [K],
% 245.8 < rho < 405.3 [kg/m^3]. To simplify the calculations, it is set to
% 1


%------------------------------------------------------------------------
% Calculation of viscosity
%------------------------------------------------------------------------

etad = eta_0.*eta_1.*eta_2; % dimensionless viscosity

eta = etad.*etar;

end


