function [P_VLH,mNaCl] = vlh_naclh2o(temp)


% VLH_NACLH2O -  calculates the vapor-liquid-halite (VLH) coexistence point
%      for a given temperature from a multiparametric equation
%
%      Usage: [p_vlh,mnacl] = vlh_naclh2o(T)
%
%      Input parameter (vector or scalar): 
%           T            - temperature [degC]   (0 < T < 1000)
%
%      Output parameter:
%         p_vlh          - VLH coexistence pressure [MPa] 
%         mnacl          - molality of NaCl at saturation [mol/kg]
%
%      References:
%      Driesner & Heinrich (2007) GCA 71, 4880-4901.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2011)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       January 2013: name in internal function call changed: solu_NaCl -> solu_nacl 
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 1
     warning('VLH_NaClH2O:Input','VLH_NaClH2O: Missing input parameter! Please provide  T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

TtNaCl = 800.7; % triple point temperature of NaCl [C]
PtNaCl = 5e-4; % triple point pressure of NaCl [bar]

if  ~any(temp<(0) | temp>1000) 

else
    bad = find(temp<(0) | temp>1000);
    temp(bad)=NaN*ones(size(bad));


 warning('VLH_NaClH2O:OutOfParameterRange','VLH_NaClH2O: Input values out of range!')
 warning('VLH_NaClH2O:OutOfParameterRange','Equations valid for T=0-1000 degC.')
end

if any (temp > TtNaCl) % VLH coexistence exists only below tripel point
    warning('VLH_NaClH2O:OutOfParameterRange','Liquid vapor halite coexistence does not exist above triple point.')
    warning('VLH_NaClH2O:OutOfParameterRange','Higher temperatures are set to NaN.')
    temp(temp > TtNaCl) = NaN;
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------


f(1) =  4.64e-3;
f(2) =  5.0e-7;
f(3) =  1.69078e1;
f(4) = -2.69148e2;
f(5) =  7.63204e3;
f(6) = -4.95636e4;
f(7) =  2.33119e5;
f(8) = -5.13556e5;
f(9) =  5.49708e5;
f(10)= -2.84628e5;
f(11)= PtNaCl - (f(1) + f(2) + f(3) + f(4) + f(5) + f(6) + f(7) + f(8) + f(9) + f(10));




%------------------------------------------------------------------------
% Calculation of the vapor liquid halite coexistence pressure
%------------------------------------------------------------------------

P_VLH = f(1) + f(2).*(temp./TtNaCl) + f(3).*(temp./TtNaCl).^2 ...
    + f(4).*(temp./TtNaCl).^3 + f(5).*(temp./TtNaCl).^4 + f(6).*(temp./TtNaCl).^5 ...
    + f(7).*(temp./TtNaCl).^6 + f(8).*(temp./TtNaCl).^7 + f(9).*(temp./TtNaCl).^8 ...
    + f(10).*(temp./TtNaCl).^9 + f(11).*(temp./TtNaCl).^10; % [bar]

P_VLH = P_VLH./10; % pressure in MPa

%------------------------------------------------------------------------
% Calculation of mole fraction xNaCl at vapor halite liquid coexistence
%------------------------------------------------------------------------

mNaCl = zeros(length(temp),1); % empty matrix for mole fraction

% call external function to calculate saturation mole fraction
for i = 1:length(temp)
    mNaCl(i) = solu_nacl(P_VLH(i),temp(i));
end 

end